@@ -1,9 +1,10 @@
+/* $Id: GuestControlSvc.h $ */
 /** @file
  * Guest control service - Common header for host service and guest clients.
  */
 
 /*
- * Copyright (C) 2011-2015 Oracle Corporation
+ * Copyright (C) 2011-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,16 +24,16 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_HostService_GuestControlService_h
-#define ___VBox_HostService_GuestControlService_h
+#ifndef VBOX_INCLUDED_HostServices_GuestControlSvc_h
+#define VBOX_INCLUDED_HostServices_GuestControlSvc_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
-#include <VBox/types.h>
-#include <VBox/VMMDev.h>
-#include <VBox/VBoxGuest2.h>
+#include <VBox/VMMDevCoreTypes.h>
+#include <VBox/VBoxGuestCoreTypes.h>
 #include <VBox/hgcmsvc.h>
-#include <VBox/log.h>
 #include <iprt/assert.h>
-#include <iprt/string.h>
 
 /* Everything defined in this file lives in this namespace. */
 namespace guestControl {
@@ -80,136 +81,6 @@ namespace guestControl {
 #define VBOX_GUESTCTRL_FILTER_BY_SESSION(uSession) \
     (VBOX_GUESTCTRL_CONTEXTID_MAKE_SESSION(uSession) | 0xF8000000)
 
-/**
- * Process status when executed in the guest.
- */
-enum eProcessStatus
-{
-    /** Process is in an undefined state. */
-    PROC_STS_UNDEFINED = 0,
-    /** Process has been started. */
-    PROC_STS_STARTED = 1,
-    /** Process terminated normally. */
-    PROC_STS_TEN = 2,
-    /** Process terminated via signal. */
-    PROC_STS_TES = 3,
-    /** Process terminated abnormally. */
-    PROC_STS_TEA = 4,
-    /** Process timed out and was killed. */
-    PROC_STS_TOK = 5,
-    /** Process timed out and was not killed successfully. */
-    PROC_STS_TOA = 6,
-    /** Service/OS is stopping, process was killed. */
-    PROC_STS_DWN = 7,
-    /** Something went wrong (error code in flags). */
-    PROC_STS_ERROR = 8
-};
-
-/** @todo r=bird: Most defines in this file needs to be scoped a little
- *        better!  For instance INPUT_FLAG_NONE is very generic. */
-
-/**
- * Input flags, set by the host. This is needed for
- * handling flags on the guest side.
- * Note: Has to match Main's ProcessInputFlag_* flags!
- */
-#define INPUT_FLAG_NONE                     0x0
-#define INPUT_FLAG_EOF                      RT_BIT(0)
-
-/**
- * Guest session creation flags.
- * Only handled internally at the moment.
- */
-#define SESSIONCREATIONFLAG_NONE            0x0
-
-/**
- * Guest directory removement flags.
- * Essentially using what IPRT's RTDIRRMREC_F_
- * defines have to offer.
- */
-#define DIRREMOVE_FLAG_RECURSIVE            RT_BIT(0)
-/** Delete the content of the directory and the directory itself. */
-#define DIRREMOVE_FLAG_CONTENT_AND_DIR      RT_BIT(1)
-/** Only delete the content of the directory, omit the directory it self. */
-#define DIRREMOVE_FLAG_CONTENT_ONLY         RT_BIT(2)
-/** Mask of valid flags. */
-#define DIRREMOVE_FLAG_VALID_MASK           UINT32_C(0x00000003)
-
-/** @name EXECUTEPROCESSFLAG_XXX Guest process creation flags.
- * @note Has to match Main's ProcessCreateFlag_* flags!
- */
-#define EXECUTEPROCESSFLAG_NONE             UINT32_C(0x0)
-#define EXECUTEPROCESSFLAG_WAIT_START       RT_BIT(0)
-#define EXECUTEPROCESSFLAG_IGNORE_ORPHANED  RT_BIT(1)
-#define EXECUTEPROCESSFLAG_HIDDEN           RT_BIT(2)
-#define EXECUTEPROCESSFLAG_NO_PROFILE       RT_BIT(3)
-#define EXECUTEPROCESSFLAG_WAIT_STDOUT      RT_BIT(4)
-#define EXECUTEPROCESSFLAG_WAIT_STDERR      RT_BIT(5)
-#define EXECUTEPROCESSFLAG_EXPAND_ARGUMENTS RT_BIT(6)
-#define EXECUTEPROCESSFLAG_UNQUOTED_ARGS    RT_BIT(7)
-/** @} */
-
-/**
- * Pipe handle IDs used internally for referencing to
- * a certain pipe buffer.
- */
-#define OUTPUT_HANDLE_ID_STDOUT_DEPRECATED  0 /* Needed for VBox hosts < 4.1.0. */
-#define OUTPUT_HANDLE_ID_STDOUT             1
-#define OUTPUT_HANDLE_ID_STDERR             2
-
-/**
- * Guest path rename flags.
- * Essentially using what IPRT's RTPATHRENAME_FLAGS_
- * defines have to offer.
- */
-/** Do not replace anything. */
-#define PATHRENAME_FLAG_NO_REPLACE          UINT32_C(0)
-/** This will replace attempt any target which isn't a directory. */
-#define PATHRENAME_FLAG_REPLACE             RT_BIT(0)
-/** Don't allow symbolic links as part of the path. */
-#define PATHRENAME_FLAG_NO_SYMLINKS         RT_BIT(1)
-/** Mask of valid flags. */
-#define PATHRENAME_FLAG_VALID_MASK          UINT32_C(0x00000002)
-
-/**
- * Defines for guest process array lengths.
- */
-#define GUESTPROCESS_MAX_CMD_LEN            _1K
-#define GUESTPROCESS_MAX_ARGS_LEN           _1K
-#define GUESTPROCESS_MAX_ENV_LEN            _64K
-#define GUESTPROCESS_MAX_USER_LEN           128
-#define GUESTPROCESS_MAX_PASSWORD_LEN       128
-#define GUESTPROCESS_MAX_DOMAIN_LEN         256
-
-/** @name Internal tools built into VBoxService which are used in order to
- *        accomplish tasks host<->guest.
- * @{
- */
-#define VBOXSERVICE_TOOL_CAT        "vbox_cat"
-#define VBOXSERVICE_TOOL_LS         "vbox_ls"
-#define VBOXSERVICE_TOOL_RM         "vbox_rm"
-#define VBOXSERVICE_TOOL_MKDIR      "vbox_mkdir"
-#define VBOXSERVICE_TOOL_MKTEMP     "vbox_mktemp"
-#define VBOXSERVICE_TOOL_STAT       "vbox_stat"
-/** @} */
-
-/**
- * Input status, reported by the client.
- */
-enum eInputStatus
-{
-    /** Input is in an undefined state. */
-    INPUT_STS_UNDEFINED = 0,
-    /** Input was written (partially, see cbProcessed). */
-    INPUT_STS_WRITTEN = 1,
-    /** Input failed with an error (see flags for rc). */
-    INPUT_STS_ERROR = 20,
-    /** Process has abandoned / terminated input handling. */
-    INPUT_STS_TERMINATED = 21,
-    /** Too much input data. */
-    INPUT_STS_OVERFLOW = 30
-};
-
 /**
  * Structure keeping the context of a host callback.
  */
@@ -240,6 +111,19 @@ typedef struct VBoxGuestCtrlHostCallback
 
 } VBOXGUESTCTRLHOSTCALLBACK, *PVBOXGUESTCTRLHOSTCALLBACK;
 
+
+/** @name Host message destiation flags.
+ *
+ * This is ORed into the context ID parameter Main after extending it to 64-bit.
+ *
+ * @internal Host internal.
+ * @{ */
+#define VBOX_GUESTCTRL_DST_ROOT_SVC     RT_BIT_64(63)
+#define VBOX_GUESTCTRL_DST_SESSION      RT_BIT_64(62)
+#define VBOX_GUESTCTRL_DST_BOTH         ( VBOX_GUESTCTRL_DST_ROOT_SVC | VBOX_GUESTCTRL_DST_SESSION )
+/** @} */
+
+
 /**
  * The service functions which are callable by host.
  */
@@ -271,7 +155,7 @@ enum eHostFn
     /**
      * Gets the current status of a running process, e.g.
      * new data on stdout/stderr, process terminated etc.
-     ** Note: Legacy (VBox < 4.3) command.
+     * @note Legacy (VBox < 4.3) command.
      */
     HOST_EXEC_GET_OUTPUT = 102,
     /**
@@ -324,75 +208,312 @@ enum eHostFn
     /**
      * Renames a path on the guest.
      */
-    HOST_PATH_RENAME = 330
+    HOST_PATH_RENAME = 330,
+    /**
+     * Retrieves the user's documents directory.
+     */
+    HOST_PATH_USER_DOCUMENTS = 331,
+    /**
+     * Retrieves the user's home directory.
+     */
+    HOST_PATH_USER_HOME = 332
 };
 
+
 /**
- * The service functions which are called by guest. The numbers may not change,
- * so we hardcode them.
+ * Translates a guest control host function function enum to at string.
+ * @returns Enum string name.
+ * @param   enmFunction     The function name to translate.
+ */
+DECLINLINE(const char *) GstCtrlHostFnName(enum eHostFn enmFunction)
+{
+    switch (enmFunction)
+    {
+        RT_CASE_RET_STR(HOST_CANCEL_PENDING_WAITS);
+        RT_CASE_RET_STR(HOST_SESSION_CREATE);
+        RT_CASE_RET_STR(HOST_SESSION_CLOSE);
+        RT_CASE_RET_STR(HOST_EXEC_CMD);
+        RT_CASE_RET_STR(HOST_EXEC_SET_INPUT);
+        RT_CASE_RET_STR(HOST_EXEC_GET_OUTPUT);
+        RT_CASE_RET_STR(HOST_EXEC_TERMINATE);
+        RT_CASE_RET_STR(HOST_EXEC_WAIT_FOR);
+        RT_CASE_RET_STR(HOST_FILE_OPEN);
+        RT_CASE_RET_STR(HOST_FILE_CLOSE);
+        RT_CASE_RET_STR(HOST_FILE_READ);
+        RT_CASE_RET_STR(HOST_FILE_READ_AT);
+        RT_CASE_RET_STR(HOST_FILE_WRITE);
+        RT_CASE_RET_STR(HOST_FILE_WRITE_AT);
+        RT_CASE_RET_STR(HOST_FILE_SEEK);
+        RT_CASE_RET_STR(HOST_FILE_TELL);
+        RT_CASE_RET_STR(HOST_DIR_REMOVE);
+        RT_CASE_RET_STR(HOST_PATH_RENAME);
+        RT_CASE_RET_STR(HOST_PATH_USER_DOCUMENTS);
+        RT_CASE_RET_STR(HOST_PATH_USER_HOME);
+    }
+    return "Unknown";
+}
+
+
+/**
+ * The service functions which are called by guest.
+ *
+ * @note The function numbers cannot be changed.  Please use the first non-zero
+ *       number that's not in use when adding new functions.
+ *
+ * @note Remember to update service.cpp when adding new functions/events for
+ *       Main, as it validates all incoming commands before passing them on.
  */
 enum eGuestFn
 {
-    /**
-     * Guest waits for a new message the host wants to process on the guest side.
+    /** Guest waits for a new message the host wants to process on the guest side.
      * This is a blocking call and can be deferred.
+     *
+     * @note This command is rather odd.  The above description isn't really
+     *       correct.  Yes, it (1) waits for a new message and will return the
+     *       mesage number and parameter count when one is available.   However, it
+     *       is also (2) used to retrieve the message parameters.   For some weird
+     *       reasons it was decided that it should always return VERR_TOO_MUCH_DATA
+     *       when used in the first capacity.
+     *
+     * @note Has a problem if the guest kernel module cancels the HGCM call, as the
+     *       guest cannot resume waiting till the host issues a message for it and
+     *       the cancelled call returns.  The new message may potentially end up in
+     *       /dev/null depending and hang the message conversation between the guest
+     *       and the host (SIGCHLD).
+     *
+     * @deprecated Replaced by GUEST_MSG_PEEK_WAIT, GUEST_MSG_GET and
+     *             GUEST_MSG_CANCEL.
      */
     GUEST_MSG_WAIT = 1,
-    /**
-     * Guest asks the host to cancel all pending waits the guest itself waits on.
-     * This becomes necessary when the guest wants to quit but still waits for
-     * commands from the host.
+    /** Cancels pending calls for this client session.
+     *
+     * This should be used if a GUEST_MSG_PEEK_WAIT or GUEST_MSG_WAIT call gets
+     * interrupted on the client end, so as to prevent being rebuffed with
+     * VERR_RESOURCE_BUSY when restarting the call.
+     *
+     * @retval  VINF_SUCCESS if cancelled any calls.
+     * @retval  VWRN_NOT_FOUND if no callers.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @since   6.0
      */
-    GUEST_CANCEL_PENDING_WAITS = 2,
-    /**
-     * Guest disconnected (terminated normally or due to a crash HGCM
+    GUEST_MSG_CANCEL = 2,
+    /** Guest disconnected (terminated normally or due to a crash HGCM
      * detected when calling service::clientDisconnect().
+     *
+     * @note This is a host side notification message that has no business in this
+     *       enum.  The guest cannot use this function number, host will reject it.
      */
     GUEST_DISCONNECTED = 3,
-    /**
-     * Sets a message filter to only get messages which have a certain
+    /** Sets a message filter to only get messages which have a certain
      * context ID scheme (that is, a specific session, object etc).
      * Since VBox 4.3+.
+     * @deprecated  Replaced by GUEST_SESSION_ACCEPT.
      */
     GUEST_MSG_FILTER_SET = 4,
-    /**
-     * Unsets (and resets) a previously set message filter.
+    /** Unsets (and resets) a previously set message filter.
+     * @retval  VERR_NOT_IMPLEMENTED since 6.0.
+     * @deprecated  Never needed or used,
      */
     GUEST_MSG_FILTER_UNSET = 5,
+    /** Peeks at the next message, returning immediately.
+     *
+     * Returns two 32-bit parameters, first is the message ID and the second the
+     * parameter count.  May optionally return additional 32-bit parameters with the
+     * sizes of respective message parameters.  To distinguish buffer sizes from
+     * integer parameters, the latter gets their sizes inverted (uint32_t is ~4U,
+     * uint64_t is ~8U).
+     *
+     * Does also support the VM restore checking as in GUEST_MSG_PEEK_WAIT (64-bit
+     * param \# 0), see documentation there.
+     *
+     * @retval  VINF_SUCCESS if a message was pending and is being returned.
+     * @retval  VERR_TRY_AGAIN if no message pending.
+     * @retval  VERR_VM_RESTORED if first parameter is a non-zero 64-bit value that
+     *          does not match VbglR3GetSessionId() any more.  The new value is
+     *          returned.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_MSG_PEEK_NOWAIT = 6,
+    /** Peeks at the next message, waiting for one to arrive.
+     *
+     * Returns two 32-bit parameters, first is the message ID and the second the
+     * parameter count.  May optionally return additional 32-bit parameters with the
+     * sizes of respective message parameters.  To distinguish buffer sizes from
+     * integer parameters, the latter gets their sizes inverted (uint32_t is ~4U,
+     * uint64_t is ~8U).
+     *
+     * To facilitate VM restore checking, the first parameter can be a 64-bit
+     * integer holding the VbglR3GetSessionId() value the guest knowns.  The
+     * function will then check this before going to sleep and return
+     * VERR_VM_RESTORED if it doesn't match, same thing happens when the VM is
+     * restored.
+     *
+     * @retval  VINF_SUCCESS if info about an pending message is being returned.
+     * @retval  VINF_TRY_AGAIN and message set to HOST_CANCEL_PENDING_WAITS if
+     *          cancelled by GUEST_MSG_CANCEL.
+     * @retval  VERR_RESOURCE_BUSY if another thread already made a waiting call.
+     * @retval  VERR_VM_RESTORED if first parameter is a non-zero 64-bit value that
+     *          does not match VbglR3GetSessionId() any more.  The new value is
+     *          returned.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @note    This replaces GUEST_MSG_WAIT.
+     * @since   6.0
+     */
+    GUEST_MSG_PEEK_WAIT = 7,
+    /** Gets the next message, returning immediately.
+     *
+     * All parameters are specific to the message being retrieved, however if the
+     * first one is an integer value it shall be an input parameter holding the
+     * ID of the message being retrieved.  While it would be nice to add a separate
+     * parameter for this purpose, this is difficult without breaking GUEST_MSG_WAIT
+     * compatibility.
+     *
+     * @retval  VINF_SUCCESS if message retrieved and removed from the pending queue.
+     * @retval  VERR_TRY_AGAIN if no message pending.
+     * @retval  VERR_MISMATCH if the incoming message ID does not match the pending.
+     * @retval  VERR_BUFFER_OVERFLOW if a parmeter buffer is too small.  The buffer
+     *          size was updated to reflect the required size.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @note    This replaces GUEST_MSG_WAIT.
+     * @since   6.0
+     */
+    GUEST_MSG_GET = 8,
+    /** Skip message.
+     *
+     * This skips the current message, replying to the main backend as best it can.
+     * Takes between zero and two parameters.  The first parameter is the 32-bit
+     * VBox status code to pass onto Main when skipping the command, defaults to
+     * VERR_NOT_SUPPORTED.  The second parameter is the 32-bit message ID of the
+     * command to skip, by default whatever is first in the queue is removed.  This
+     * is also the case if UINT32_MAX is specified.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_NOT_FOUND if no message pending.
+     * @retval  VERR_MISMATCH if the specified message ID didn't match.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @since   6.0
+     */
+    GUEST_MSG_SKIP = 9,
     /**
      * Skips the current assigned message returned by GUEST_MSG_WAIT.
      * Needed for telling the host service to not keep stale
      * host commands in the queue.
+     * @deprecated  Replaced by GUEST_MSG_SKIP.
      */
-    GUEST_MSG_SKIP = 10,
-    /**
-     * General reply to a host message. Only contains basic data
-     * along with a simple payload.
+    GUEST_MSG_SKIP_OLD = 10,
+    /** General reply to a host message.
+     * Only contains basic data along with a simple payload.
+     * @todo proper docs.
      */
     GUEST_MSG_REPLY = 11,
-    /**
-     * General message for updating a pending progress for
-     * a long task.
+    /** General message for updating a pending progress for a long task.
+     * @todo proper docs.
      */
     GUEST_MSG_PROGRESS_UPDATE = 12,
+    /** Sets the caller as the master.
+     *
+     * Called by the root VBoxService to explicitly tell the host that's the master
+     * service.  Required to use main VBoxGuest device node.  No parameters.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_ACCESS_DENIED if not using main VBoxGuest device not
+     * @retval  VERR_RESOURCE_BUSY if there is already a master.
+     * @retval  VERR_VERSION_MISMATCH if VBoxGuest didn't supply requestor info.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @since   6.0
+     */
+    GUEST_MAKE_ME_MASTER = 13,
+    /** Prepares the starting of a session.
+     *
+     * VBoxService makes this call before spawning a session process (must be
+     * master). The first parameter is the session ID and the second is a one time
+     * key for identifying the right session process.  First parameter is a 32-bit
+     * session ID with a value between 1 and 0xfff0.  The second parameter is a byte
+     * buffer containing a key that GUEST_SESSION_ACCEPT checks against, minimum
+     * length is 64 bytes, maximum 16384 bytes.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_OUT_OF_RESOURCES if too many pending sessions hanging around.
+     * @retval  VERR_OUT_OF_RANGE if the session ID outside the allowed range.
+     * @retval  VERR_BUFFER_OVERFLOW if key too large.
+     * @retval  VERR_BUFFER_UNDERFLOW if key too small.
+     * @retval  VERR_ACCESS_DENIED if not master or in legacy mode.
+     * @retval  VERR_DUPLICATE if the session ID has been prepared already.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_SESSION_PREPARE = 14,
+    /** Cancels a prepared session.
+     *
+     * VBoxService makes this call to clean up after spawning a session process
+     * failed.  One parameter, 32-bit session ID.  If UINT32_MAX is passed, all
+     * prepared sessions are cancelled.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VWRN_NOT_FOUND if no session with the specified ID.
+     * @retval  VERR_ACCESS_DENIED if not master or in legacy mode.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_SESSION_CANCEL_PREPARED = 15,
+    /** Accepts a prepared session.
+     *
+     * The session processes makes this call to accept a prepared session.  The
+     * session ID is then uniquely associated with the HGCM client ID of the caller.
+     * The parameters must be identical to the matching GUEST_SESSION_PREPARE call.
+     *
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_NOT_FOUND if the specified session ID wasn't found.
+     * @retval  VERR_OUT_OF_RANGE if the session ID outside the allowed range.
+     * @retval  VERR_BUFFER_OVERFLOW if key too large.
+     * @retval  VERR_BUFFER_UNDERFLOW if key too small.
+     * @retval  VERR_ACCESS_DENIED if we're in legacy mode or is master.
+     * @retval  VERR_RESOURCE_BUSY if the client is already associated with a session.
+     * @retval  VERR_MISMATCH if the key didn't match.
+     * @retval  VERR_INVALID_CLIENT_ID
+     * @retval  VERR_WRONG_PARAMETER_COUNT
+     * @retval  VERR_WRONG_PARAMETER_TYPE
+     * @since   6.0
+     */
+    GUEST_SESSION_ACCEPT = 16,
     /**
      * Guest reports back a guest session status.
+     * @todo proper docs.
      */
     GUEST_SESSION_NOTIFY = 20,
     /**
      * Guest wants to close a specific guest session.
+     * @todo proper docs.
      */
     GUEST_SESSION_CLOSE = 21,
+
     /**
      * Guests sends output from an executed process.
+     * @todo proper docs.
      */
     GUEST_EXEC_OUTPUT = 100,
     /**
      * Guest sends a status update of an executed process to the host.
+     * @todo proper docs.
      */
     GUEST_EXEC_STATUS = 101,
     /**
      * Guests sends an input status notification to the host.
+     * @todo proper docs.
      */
     GUEST_EXEC_INPUT_STATUS = 102,
     /**
@@ -400,18 +521,59 @@ enum eGuestFn
      * a stdout, stderr or a stdin event. The actual event only tells
      * how many data is available / can be sent without actually
      * transmitting the data.
+     * @todo proper docs.
      */
     GUEST_EXEC_IO_NOTIFY = 210,
     /**
      * Guest notifies the host about some directory event.
+     * @todo proper docs.
      */
     GUEST_DIR_NOTIFY = 230,
     /**
      * Guest notifies the host about some file event.
+     * @todo proper docs.
      */
     GUEST_FILE_NOTIFY = 240
 };
 
+/**
+ * Translates a guest control host function function enum to at string.
+ * @returns Enum string name.
+ * @param   enmFunction     The function name to translate.
+ */
+DECLINLINE(const char *) GstCtrlGuestFnName(enum eGuestFn enmFunction)
+{
+    switch (enmFunction)
+    {
+        RT_CASE_RET_STR(GUEST_MSG_WAIT);
+        RT_CASE_RET_STR(GUEST_MSG_CANCEL);
+        RT_CASE_RET_STR(GUEST_DISCONNECTED);
+        RT_CASE_RET_STR(GUEST_MSG_FILTER_SET);
+        RT_CASE_RET_STR(GUEST_MSG_FILTER_UNSET);
+        RT_CASE_RET_STR(GUEST_MSG_PEEK_NOWAIT);
+        RT_CASE_RET_STR(GUEST_MSG_PEEK_WAIT);
+        RT_CASE_RET_STR(GUEST_MSG_GET);
+        RT_CASE_RET_STR(GUEST_MSG_SKIP_OLD);
+        RT_CASE_RET_STR(GUEST_MSG_REPLY);
+        RT_CASE_RET_STR(GUEST_MSG_PROGRESS_UPDATE);
+        RT_CASE_RET_STR(GUEST_MSG_SKIP);
+        RT_CASE_RET_STR(GUEST_MAKE_ME_MASTER);
+        RT_CASE_RET_STR(GUEST_SESSION_PREPARE);
+        RT_CASE_RET_STR(GUEST_SESSION_CANCEL_PREPARED);
+        RT_CASE_RET_STR(GUEST_SESSION_ACCEPT);
+        RT_CASE_RET_STR(GUEST_SESSION_NOTIFY);
+        RT_CASE_RET_STR(GUEST_SESSION_CLOSE);
+        RT_CASE_RET_STR(GUEST_EXEC_OUTPUT);
+        RT_CASE_RET_STR(GUEST_EXEC_STATUS);
+        RT_CASE_RET_STR(GUEST_EXEC_INPUT_STATUS);
+        RT_CASE_RET_STR(GUEST_EXEC_IO_NOTIFY);
+        RT_CASE_RET_STR(GUEST_DIR_NOTIFY);
+        RT_CASE_RET_STR(GUEST_FILE_NOTIFY);
+    }
+    return "Unknown";
+}
+
+
 /**
  * Guest session notification types.
  * @sa HGCMMsgSessionNotify.
@@ -475,8 +637,9 @@ enum GUEST_FILE_NOTIFYTYPE
 };
 
 /**
- * Guest file seeking types. Has to
- * match FileSeekType in Main.
+ * Guest file seeking types. Has to match FileSeekType in Main.
+ *
+ * @note This is not compatible with RTFileSeek, which is an unncessary pain.
  */
 enum GUEST_FILE_SEEKTYPE
 {
@@ -497,7 +660,7 @@ enum GUEST_FILE_SEEKTYPE
  */
 typedef struct HGCMMsgCmdWaitFor
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /**
      * The returned command the host wants to
      * run on the guest.
@@ -515,7 +678,7 @@ typedef struct HGCMMsgCmdWaitFor
  */
 typedef struct HGCMMsgCmdFilterSet
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Value to filter for after filter mask
      *  was applied. */
     HGCMFunctionParameter value;         /* IN uint32_t */
@@ -533,7 +696,7 @@ typedef struct HGCMMsgCmdFilterSet
  */
 typedef struct HGCMMsgCmdFilterUnset
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Unset flags; currently unused. */
     HGCMFunctionParameter flags;    /* IN uint32_t */
 } HGCMMsgCmdFilterUnset;
@@ -545,7 +708,7 @@ typedef struct HGCMMsgCmdFilterUnset
  */
 typedef struct HGCMMsgCmdSkip
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Skip flags; currently unused. */
     HGCMFunctionParameter flags;    /* IN uint32_t */
 } HGCMMsgCmdSkip;
@@ -556,12 +719,12 @@ typedef struct HGCMMsgCmdSkip
  */
 typedef struct HGCMMsgCancelPendingWaits
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
 } HGCMMsgCancelPendingWaits;
 
 typedef struct HGCMMsgCmdReply
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Message type. */
@@ -577,7 +740,7 @@ typedef struct HGCMMsgCmdReply
  */
 typedef struct HGCMMsgSessionOpen
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The guest control protocol version this
@@ -598,7 +761,7 @@ typedef struct HGCMMsgSessionOpen
  */
 typedef struct HGCMMsgSessionClose
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Session termination flags. */
@@ -610,7 +773,7 @@ typedef struct HGCMMsgSessionClose
  */
 typedef struct HGCMMsgSessionNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -621,7 +784,7 @@ typedef struct HGCMMsgSessionNotify
 
 typedef struct HGCMMsgPathRename
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** Source to rename. */
@@ -632,12 +795,26 @@ typedef struct HGCMMsgPathRename
     HGCMFunctionParameter flags;
 } HGCMMsgPathRename;
 
+typedef struct HGCMMsgPathUserDocuments
+{
+    VBGLIOCHGCMCALL hdr;
+    /** UInt32: Context ID. */
+    HGCMFunctionParameter context;
+} HGCMMsgPathUserDocuments;
+
+typedef struct HGCMMsgPathUserHome
+{
+    VBGLIOCHGCMCALL hdr;
+    /** UInt32: Context ID. */
+    HGCMFunctionParameter context;
+} HGCMMsgPathUserHome;
+
 /**
  * Executes a command inside the guest.
  */
 typedef struct HGCMMsgProcExec
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The command to execute on the guest. */
@@ -692,7 +869,7 @@ typedef struct HGCMMsgProcExec
  */
 typedef struct HGCMMsgProcInput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID) to send the input to. */
@@ -711,7 +888,7 @@ typedef struct HGCMMsgProcInput
  */
 typedef struct HGCMMsgProcOutput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -729,7 +906,7 @@ typedef struct HGCMMsgProcOutput
  */
 typedef struct HGCMMsgProcStatus
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -747,7 +924,7 @@ typedef struct HGCMMsgProcStatus
  */
 typedef struct HGCMMsgProcStatusInput
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -769,7 +946,7 @@ typedef struct HGCMMsgProcStatusInput
  */
 typedef struct HGCMMsgProcTerminate
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -781,7 +958,7 @@ typedef struct HGCMMsgProcTerminate
  */
 typedef struct HGCMMsgProcWaitFor
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** The process ID (PID). */
@@ -794,7 +971,7 @@ typedef struct HGCMMsgProcWaitFor
 
 typedef struct HGCMMsgDirRemove
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** Directory to remove. */
@@ -808,7 +985,7 @@ typedef struct HGCMMsgDirRemove
  */
 typedef struct HGCMMsgFileOpen
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** UInt32: Context ID. */
     HGCMFunctionParameter context;
     /** File to open. */
@@ -830,7 +1007,7 @@ typedef struct HGCMMsgFileOpen
  */
 typedef struct HGCMMsgFileClose
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to close. */
@@ -842,7 +1019,7 @@ typedef struct HGCMMsgFileClose
  */
 typedef struct HGCMMsgFileRead
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to read from. */
@@ -856,7 +1033,7 @@ typedef struct HGCMMsgFileRead
  */
 typedef struct HGCMMsgFileReadAt
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to read from. */
@@ -872,7 +1049,7 @@ typedef struct HGCMMsgFileReadAt
  */
 typedef struct HGCMMsgFileWrite
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to write to. */
@@ -888,7 +1065,7 @@ typedef struct HGCMMsgFileWrite
  */
 typedef struct HGCMMsgFileWriteAt
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to write to. */
@@ -906,7 +1083,7 @@ typedef struct HGCMMsgFileWriteAt
  */
 typedef struct HGCMMsgFileSeek
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to seek. */
@@ -922,7 +1099,7 @@ typedef struct HGCMMsgFileSeek
  */
 typedef struct HGCMMsgFileTell
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** File handle to get the current position for. */
@@ -936,7 +1113,7 @@ typedef struct HGCMMsgFileTell
 
 typedef struct HGCMReplyFileNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -974,7 +1151,7 @@ typedef struct HGCMReplyFileNotify
 
 typedef struct HGCMReplyDirNotify
 {
-    VBoxGuestHGCMCallInfo hdr;
+    VBGLIOCHGCMCALL hdr;
     /** Context ID. */
     HGCMFunctionParameter context;
     /** Notification type. */
@@ -1188,5 +1365,5 @@ typedef struct CALLBACKDATA_FILE_NOTIFY
 
 } /* namespace guestControl */
 
-#endif  /* !___VBox_HostService_GuestControlService_h */
+#endif /* !VBOX_INCLUDED_HostServices_GuestControlSvc_h */
 
