@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -28,8 +28,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_intnetinline_h
-#define ___VBox_intnetinline_h
+#ifndef VBOX_INCLUDED_intnetinline_h
+#define VBOX_INCLUDED_intnetinline_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <VBox/intnet.h>
 #include <iprt/string.h>
@@ -397,6 +400,7 @@ DECLINLINE(void) IntNetRingSkipFrame(PINTNETRINGBUF pRingBuf)
  * @returns VINF_SUCCESS or VERR_BUFFER_OVERFLOW.
  * @param   pRingBuf            The ring buffer.
  * @param   cbFrame             The frame size.
+ * @param   u8Type              The header type.
  * @param   ppHdr               Where to return the frame header.
  *                              Don't touch this!
  * @param   ppvFrame            Where to return the frame pointer.
@@ -737,7 +741,7 @@ DECLINLINE(int) IntNetRingWriteFrame(PINTNETRINGBUF pRingBuf, const void *pvFram
  * Reads the next frame in the buffer and moves the read cursor past it.
  *
  * @returns Size of the frame in bytes.  0 is returned if nothing in the buffer.
- * @param   pRingBuff   The ring buffer to read from.
+ * @param   pRingBuf    The ring buffer to read from.
  * @param   pvFrameDst  Where to put the frame.  The caller is responsible for
  *                      ensuring that there is sufficient space for the frame.
  *
@@ -792,7 +796,7 @@ DECLINLINE(void) IntNetBufInit(PINTNETBUF pIntBuf, uint32_t cbBuf, uint32_t cbRe
     Assert(cbBuf >= sizeof(INTNETBUF) + cbRecv + cbSend);
     Assert(RT_ALIGN_32(cbRecv, INTNETRINGBUF_ALIGNMENT) == cbRecv);
     Assert(RT_ALIGN_32(cbSend, INTNETRINGBUF_ALIGNMENT) == cbSend);
-    Assert(ASMMemIsAll8(pIntBuf, cbBuf, '\0') == NULL);
+    Assert(ASMMemIsZero(pIntBuf, cbBuf));
 
     pIntBuf->u32Magic  = INTNETBUF_MAGIC;
     pIntBuf->cbBuf     = cbBuf;
@@ -800,7 +804,7 @@ DECLINLINE(void) IntNetBufInit(PINTNETBUF pIntBuf, uint32_t cbBuf, uint32_t cbRe
     pIntBuf->cbSend    = cbSend;
 
     /* receive ring buffer. */
-    uint32_t offBuf = RT_ALIGN_32(sizeof(INTNETBUF), INTNETRINGBUF_ALIGNMENT) - RT_OFFSETOF(INTNETBUF, Recv);
+    uint32_t offBuf = RT_ALIGN_32(sizeof(INTNETBUF), INTNETRINGBUF_ALIGNMENT) - RT_UOFFSETOF(INTNETBUF, Recv);
     pIntBuf->Recv.offStart      = offBuf;
     pIntBuf->Recv.offReadX      = offBuf;
     pIntBuf->Recv.offWriteInt   = offBuf;
@@ -808,7 +812,7 @@ DECLINLINE(void) IntNetBufInit(PINTNETBUF pIntBuf, uint32_t cbBuf, uint32_t cbRe
     pIntBuf->Recv.offEnd        = offBuf + cbRecv;
 
     /* send ring buffer. */
-    offBuf += cbRecv + RT_OFFSETOF(INTNETBUF, Recv) - RT_OFFSETOF(INTNETBUF, Send);
+    offBuf += cbRecv + RT_UOFFSETOF(INTNETBUF, Recv) - RT_UOFFSETOF(INTNETBUF, Send);
     pIntBuf->Send.offStart      = offBuf;
     pIntBuf->Send.offReadX      = offBuf;
     pIntBuf->Send.offWriteCom   = offBuf;
@@ -819,5 +823,5 @@ DECLINLINE(void) IntNetBufInit(PINTNETBUF pIntBuf, uint32_t cbBuf, uint32_t cbRe
 
 #endif /* __cplusplus */
 
-#endif
+#endif /* !VBOX_INCLUDED_intnetinline_h */
 
