@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___iprt_fs_h
-#define ___iprt_fs_h
+#ifndef IPRT_INCLUDED_fs_h
+#define IPRT_INCLUDED_fs_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
@@ -93,6 +96,11 @@ RT_C_DECLS_BEGIN
 /** Other executable (S_IXOTH). */
 #define RTFS_UNIX_IXOTH             0000001U
 
+/** All UNIX access permission bits (0777). */
+#define RTFS_UNIX_ALL_ACCESS_PERMS  0000777U
+/** All UNIX permission bits, including set id and sticky bits.  */
+#define RTFS_UNIX_ALL_PERMS         0007777U
+
 /** Named pipe (fifo) (S_IFIFO). */
 #define RTFS_TYPE_FIFO              0010000U
 /** Character device (S_IFCHR). */
@@ -111,6 +119,8 @@ RT_C_DECLS_BEGIN
 #define RTFS_TYPE_WHITEOUT          0160000U
 /** Type mask (S_IFMT). */
 #define RTFS_TYPE_MASK              0170000U
+/** The shift count to convert between RTFS_TYPE_MASK and DIRENTRYTYPE. */
+#define RTFS_TYPE_DIRENTRYTYPE_SHIFT    12
 
 /** Unix attribute mask. */
 #define RTFS_UNIX_MASK              0xffffU
@@ -216,6 +226,7 @@ typedef enum RTFSTYPE
     RTFSTYPE_SYSFS,
     RTFSTYPE_PROC,
     RTFSTYPE_OCFS2,
+    RTFSTYPE_BTRFS,
 
     /* Windows: */
     /** New Technology File System. */
@@ -225,6 +236,8 @@ typedef enum RTFSTYPE
      * limiting the file size (except, perhaps for the 64KB cluster case on
      * non-Windows hosts). */
     RTFSTYPE_FAT,
+    /** Extended File Allocation Table, main target are flash drives. */
+    RTFSTYPE_EXFAT,
 
     /* Solaris: */
     /** Zettabyte File System.  */
@@ -293,11 +306,11 @@ typedef enum RTFSOBJATTRADD
 typedef struct RTFSOBJATTRUNIX
 {
     /** The user owning the filesystem object (st_uid).
-     * This field is NIL_UID if not supported. */
+     * This field is NIL_RTUID if not supported. */
     RTUID           uid;
 
     /** The group the filesystem object is assigned (st_gid).
-     * This field is NIL_GID if not supported. */
+     * This field is NIL_RTGID if not supported. */
     RTGID           gid;
 
     /** Number of hard links to this filesystem object (st_nlink).
@@ -313,7 +326,9 @@ typedef struct RTFSOBJATTRUNIX
     /** The unique identifier (within the filesystem) of this filesystem object (st_ino).
      * Together with INodeIdDevice, this field can be used as a OS wide unique id
      * when both their values are not 0.
-     * This field is 0 if the information is not available. */
+     * This field is 0 if the information is not available.
+     *
+     * @remarks  The special '..' dir always shows up with 0 on NTFS/Windows. */
     RTINODE         INodeId;
 
     /** User flags (st_flags).
@@ -586,6 +601,17 @@ typedef RTFSPROPERTIES const *PCRTFSPROPERTIES;
  */
 RTR3DECL(int) RTFsQueryProperties(const char *pszFsPath, PRTFSPROPERTIES pProperties);
 
+/**
+ * Checks if the given volume is case sensitive or not.
+ *
+ * This may be misleading in some cases as we lack the necessary APIs to query
+ * the information on some system (or choose not to use them) and are instead
+ * returning the general position on case sensitive file name of the system.
+ *
+ * @returns @c true if case sensitive, @c false if not.
+ * @param   pszFsPath       Path within the mounted file system.
+ */
+RTR3DECL(bool) RTFsIsCaseSensitive(const char *pszFsPath);
 
 /**
  * Mountpoint enumerator callback.
@@ -608,11 +634,22 @@ typedef FNRTFSMOUNTPOINTENUM *PFNRTFSMOUNTPOINTENUM;
 RTR3DECL(int) RTFsMountpointsEnum(PFNRTFSMOUNTPOINTENUM pfnCallback, void *pvUser);
 
 
+/**
+ * A /bin/ls clone.
+ *
+ * @returns Program exit code.
+ *
+ * @param   cArgs               The number of arguments.
+ * @param   papszArgs           The argument vector.  (Note that this may be
+ *                              reordered, so the memory must be writable.)
+ */
+RTR3DECL(RTEXITCODE) RTFsCmdLs(unsigned cArgs, char **papszArgs);
+
 #endif /* IN_RING3 */
 
 /** @} */
 
 RT_C_DECLS_END
 
-#endif /* !___iprt_fs_h */
+#endif /* !IPRT_INCLUDED_fs_h */
 
