@@ -44,7 +44,7 @@
 #include "wingdi.h"
 #include "winuser.h"
 #else
-#include <windows.h>
+# include <iprt/win/windows.h>
 #endif
 #include "wine/debug.h"
 #include "wine/unicode.h"
