@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2017 Oracle Corporation
+ * Copyright (C) 2011-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef VBOXMF_H
-#define VBOXMF_H
+#ifndef GA_INCLUDED_SRC_WINNT_Mouse_NT5_VBoxMF_h
+#define GA_INCLUDED_SRC_WINNT_Mouse_NT5_VBoxMF_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/cdefs.h>
 #define LOG_GROUP LOG_GROUP_DRV_MOUSE
@@ -81,4 +84,4 @@ void VBoxDeviceRemoved(PVBOXMOUSE_DEVEXT pDevExt);
 
 VOID VBoxDrvNotifyServiceCB(PVBOXMOUSE_DEVEXT pDevExt, PMOUSE_INPUT_DATA InputDataStart, PMOUSE_INPUT_DATA InputDataEnd, PULONG  InputDataConsumed);
 
-#endif /*VBOXMF_H*/
+#endif /* !GA_INCLUDED_SRC_WINNT_Mouse_NT5_VBoxMF_h */
