@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,6 +19,8 @@
  */
 
 #include "vbsf.h"
+#include <iprt/err.h>
+
 
 /** Macro for copying a SHFLSTRING file name into a FILE_DIRECTORY_INFORMATION structure. */
 #define INIT_FILE_NAME(obj, str) \
@@ -110,6 +112,12 @@ NTSTATUS VBoxMRxQueryDirectory(IN OUT PRX_CONTEXT RxContext)
         Log(("VBOXSF: MrxQueryDirectory: Query single entry\n"));
         fSFFlags |= SHFL_LIST_RETURN_ONE;
     }
+    if (   RxContext->QueryDirectory.RestartScan == TRUE
+        && RxContext->QueryDirectory.InitialQuery == FALSE)
+    {
+        Log(("VBOXSF: MrxQueryDirectory: Restart scan\n"));
+        fSFFlags |= SHFL_LIST_RESTART;
+    }
 
     if (Template->Length)
     {
