@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef VBSF_H
-#define VBSF_H
+#ifndef GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsf_h
+#define GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsf_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /*
  * This must be defined before including RX headers.
@@ -205,4 +208,4 @@ NTSTATUS vbsfCloseFileHandle(PMRX_VBOX_DEVICE_EXTENSION pDeviceExtension,
                              PMRX_VBOX_NETROOT_EXTENSION pNetRootExtension,
                              PMRX_VBOX_FOBX pVBoxFobx);
 
-#endif /* VBSF_H */
+#endif /* !GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsf_h */
