@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef __VBSFHLP__H
-#define __VBSFHLP__H
+#ifndef GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsfhlp_h
+#define GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsfhlp_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/nt/nt.h> /* includes ntifs.h and wdm.h */
 #include <iprt/win/ntverp.h>
@@ -65,5 +68,5 @@ PCHAR MajorFunctionString(UCHAR MajorFunction, LONG MinorFunction);
 
 NTSTATUS vbsfShflStringFromUnicodeAlloc(PSHFLSTRING *ppShflString, const WCHAR *pwc, uint16_t cb);
 
-#endif /* !__VBSFHLP__H */
+#endif /* !GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsfhlp_h */
 
