@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,8 +18,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef VBSFSHARED_H
-#define VBSFSHARED_H
+#ifndef GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsfshared_h
+#define GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsfshared_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* The network provider name for shared folders. */
 #define MRX_VBOX_PROVIDER_NAME_U L"VirtualBox Shared Folders"
@@ -52,4 +55,4 @@
 #define IOCTL_MRX_VBOX_START         _MRX_VBOX_CONTROL_CODE(106, METHOD_BUFFERED, FILE_ANY_ACCESS)
 #define IOCTL_MRX_VBOX_STOP          _MRX_VBOX_CONTROL_CODE(107, METHOD_BUFFERED, FILE_ANY_ACCESS)
 
-#endif /* VBSFSHARED_H */
+#endif /* !GA_INCLUDED_SRC_WINNT_SharedFolders_driver_vbsfshared_h */
