@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___VBOX_CREDPROV_POLLER_H___
-#define ___VBOX_CREDPROV_POLLER_H___
+#ifndef GA_INCLUDED_SRC_WINNT_VBoxCredProv_VBoxCredProvPoller_h
+#define GA_INCLUDED_SRC_WINNT_VBoxCredProv_VBoxCredProvPoller_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/critsect.h>
 #include <iprt/thread.h>
@@ -45,5 +48,5 @@ protected:
     VBoxCredProvProvider *m_pProv;
 };
 
-#endif /* !___VBOX_CREDPROV_POLLER_H___ */
+#endif /* !GA_INCLUDED_SRC_WINNT_VBoxCredProv_VBoxCredProvPoller_h */
 
