@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,7 +24,7 @@
 #include <iprt/win/windows.h>
 #include <credentialprovider.h>
 
-#include <iprt/err.h>
+#include <iprt/errcore.h>
 #include <VBox/VBoxGuestLib.h>
 
 #include "VBoxCredentialProvider.h"
