@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___VBOX_CREDENTIALPROVIDER_H___
-#define ___VBOX_CREDENTIALPROVIDER_H___
+#ifndef GA_INCLUDED_SRC_WINNT_VBoxCredProv_VBoxCredentialProvider_h
+#define GA_INCLUDED_SRC_WINNT_VBoxCredProv_VBoxCredentialProvider_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/win/windows.h>
 #include <credentialprovider.h>
@@ -98,5 +101,5 @@ LONG VBoxCredentialProviderRefCount(void);
 HRESULT VBoxCredentialProviderCreate(REFCLSID classID,
                                      REFIID interfaceID, void **ppvInterface);
 
-#endif /* !___VBOX_CREDENTIALPROVIDER_H___ */
+#endif /* !GA_INCLUDED_SRC_WINNT_VBoxCredProv_VBoxCredentialProvider_h */
 
