@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___H_GINAHELPER
-#define ___H_GINAHELPER
+#ifndef GA_INCLUDED_SRC_WINNT_VBoxGINA_Helper_h
+#define GA_INCLUDED_SRC_WINNT_VBoxGINA_Helper_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <VBox/VBoxGuestLib.h>
 
@@ -30,5 +33,5 @@ int VBoxGINACredentialsPollerTerminate(void);
 
 int VBoxGINAReportStatus(VBoxGuestFacilityStatus enmStatus);
 
-#endif /* !___H_GINAHELPER */
+#endif /* !GA_INCLUDED_SRC_WINNT_VBoxGINA_Helper_h */
 
