@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2017 Oracle Corporation
+ * Copyright (C) 2013-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,6 +16,7 @@
  */
 
 
+#define LOG_GROUP LOG_GROUP_GUEST_DND
 #include <iprt/win/windows.h>
 #include <new> /* For bad_alloc. */
 #include <iprt/win/shlobj.h>
@@ -23,11 +24,8 @@
 #include <iprt/path.h>
 #include <iprt/semaphore.h>
 #include <iprt/uri.h>
+#include <iprt/utf16.h>
 
-#ifdef LOG_GROUP
-# undef LOG_GROUP
-#endif
-#define LOG_GROUP LOG_GROUP_GUEST_DND
 #include <VBox/log.h>
 
 #include "VBoxTray.h"
@@ -88,7 +86,7 @@ VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed
 
     if (SUCCEEDED(hr))
     {
-        int rc2 = RTSemEventCreate(&mSemEvent);
+        int rc2 = RTSemEventCreate(&mEventDropped);
         AssertRC(rc2);
 
         /*
@@ -207,13 +205,14 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
     LogFlowFunc(("mStatus=%ld\n", mStatus));
     if (mStatus == Dropping)
     {
-        LogFlowFunc(("Waiting for event ...\n"));
-        int rc2 = RTSemEventWait(mSemEvent, RT_INDEFINITE_WAIT);
-        LogFlowFunc(("rc2=%Rrc, mStatus=%ld\n", rc2, mStatus)); NOREF(rc2);
+        LogRel2(("DnD: Waiting for drop event ...\n"));
+        int rc2 = RTSemEventWait(mEventDropped, RT_INDEFINITE_WAIT);
+        LogFlowFunc(("rc2=%Rrc, mStatus=%ld\n", rc2, mStatus)); RT_NOREF(rc2);
     }
 
     if (mStatus == Dropped)
     {
+        LogRel2(("DnD: Drop event received\n"));
         LogRel3(("DnD: cfFormat=%RI16, sFormat=%s, tyMed=%RU32, dwAspect=%RU32\n",
                  pThisFormat->cfFormat, VBoxDnDDataObject::ClipboardFormatToString(pFormatEtc->cfFormat),
                  pThisFormat->tymed, pThisFormat->dwAspect));
@@ -516,7 +515,7 @@ int VBoxDnDDataObject::Abort(void)
 {
     LogFlowFunc(("Aborting ...\n"));
     mStatus = Aborted;
-    return RTSemEventSignal(mSemEvent);
+    return RTSemEventSignal(mEventDropped);
 }
 
 /* static */
@@ -673,12 +672,8 @@ void VBoxDnDDataObject::SetStatus(Status status)
 int VBoxDnDDataObject::Signal(const RTCString &strFormat,
                               const void *pvData, uint32_t cbData)
 {
-    LogFlowFunc(("Signalling ...\n"));
-
     int rc;
 
-    mStatus = Dropped;
-    mstrFormat = strFormat;
     if (cbData)
     {
         mpvData = RTMemAlloc(cbData);
@@ -694,14 +689,24 @@ int VBoxDnDDataObject::Signal(const RTCString &strFormat,
     else
         rc = VINF_SUCCESS;
 
-    if (RT_FAILURE(rc))
+    if (RT_SUCCESS(rc))
+    {
+        mStatus    = Dropped;
+        mstrFormat = strFormat;
+    }
+    else
+    {
         mStatus = Aborted;
+    }
 
     /* Signal in any case. */
-    int rc2 = RTSemEventSignal(mSemEvent);
+    LogRel2(("DnD: Signalling drop event\n"));
+
+    int rc2 = RTSemEventSignal(mEventDropped);
     if (RT_SUCCESS(rc))
         rc = rc2;
 
+    LogFunc(("mStatus=%RU32, rc=%Rrc\n", mStatus, rc));
     return rc;
 }
 
