@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2017 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef __VBOXSERVICELA__H
-#define __VBOXSERVICELA__H
+#ifndef GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxLA_h
+#define GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxLA_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
-#endif /* !__VBOXSERVICELA__H */
+#endif /* !GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxLA_h */
 
