@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,13 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___VBoxSharedFolders_h
-#define ___VBoxSharedFolders_h
+#ifndef GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxSharedFolders_h
+#define GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxSharedFolders_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 int VBoxSharedFoldersAutoUnmount(void);
 int VBoxSharedFoldersAutoMount(void);
 
-#endif /* !___VBoxSharedFolders_h */
+#endif /* !GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxSharedFolders_h */
