@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -30,7 +30,9 @@
 #include "VBoxDisplay.h"
 #include "VBoxVRDP.h"
 #include "VBoxHostVersion.h"
-#include "VBoxSharedFolders.h"
+#ifdef VBOX_WITH_SHARED_FOLDERS
+# include "VBoxSharedFolders.h"
+#endif
 #ifdef VBOX_WITH_DRAG_AND_DROP
 # include "VBoxDnD.h"
 #endif
@@ -582,7 +584,7 @@ static int vboxTrayLogCreate(const char *pszLogFile)
         RTLogFlush(g_pLoggerRelease);
     }
     else
-        MessageBoxA(GetDesktopWindow(), ErrInfo.szMsg, "VBoxTray - Logging Error", MB_OK | MB_ICONERROR);
+        VBoxTrayShowError(ErrInfo.szMsg);
 
     return rc;
 }
@@ -592,6 +594,31 @@ static void vboxTrayLogDestroy(void)
     RTLogDestroy(RTLogRelSetDefaultInstance(NULL));
 }
 
+/**
+ * Displays an error message.
+ *
+ * @returns RTEXITCODE_FAILURE.
+ * @param   pszFormat   The message text.
+ * @param   ...         Format arguments.
+ */
+RTEXITCODE VBoxTrayShowError(const char *pszFormat, ...)
+{
+    va_list args;
+    va_start(args, pszFormat);
+    char *psz = NULL;
+    RTStrAPrintfV(&psz, pszFormat, args);
+    va_end(args);
+
+    AssertPtr(psz);
+    LogRel(("Error: %s", psz));
+
+    MessageBox(GetDesktopWindow(), psz, "VBoxTray - Error", MB_OK | MB_ICONERROR);
+
+    RTStrFree(psz);
+
+    return RTEXITCODE_FAILURE;
+}
+
 static void vboxTrayDestroyToolWindow(void)
 {
     if (g_hwndToolWindow)
@@ -824,8 +851,10 @@ static int vboxTrayServiceMain(void)
 
             if (RT_SUCCESS(rc))
             {
+#ifdef VBOX_WITH_SHARED_FOLDERS
                 /* Do the Shared Folders auto-mounting stuff. */
                 rc = VBoxSharedFoldersAutoMount();
+#endif
                 if (RT_SUCCESS(rc))
                 {
                     /* Report the host that we're up and running! */
