@@ -1,10 +1,10 @@
 /* $Id: VBoxServiceCpuHotPlug.cpp $ */
 /** @file
- * VBoxService - Guest Additions CPU Hot Plugging Service.
+ * VBoxService - Guest Additions CPU Hot-Plugging Service.
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,9 +15,22 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/** @page pg_vgsvc_cpuhotplug VBoxService - CPU Hot-Plugging
+ *
+ * The CPU Hot-Plugging subservice helps execute and coordinate CPU hot-plugging
+ * between the guest OS and the VMM.
+ *
+ * CPU Hot-Plugging is useful for reallocating CPU resources from one VM to
+ * other VMs or/and the host.  It talks to the VMM via VMMDev, new hot-plugging
+ * events being signalled with an interrupt (no polling).
+ *
+ * Currently only supported for linux guests.
+ */
+
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/assert.h>
 #include <iprt/dir.h>
 #include <iprt/file.h>
@@ -34,10 +47,11 @@
 #endif
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 #ifdef RT_OS_LINUX
+
 /** @name Paths to access the CPU device
  * @{
  */
@@ -68,68 +82,89 @@ typedef struct SYSFSCPUPATH
     /** Number of entries in the array, excluding the terminator. */
     unsigned           cComponents;
     /** Directory handle */
-    PRTDIR             pDir;
+    RTDIR              hDir;
     /** Current directory to try. */
     char              *pszPath;
 } SYSFSCPUPATH, *PSYSFSCPUPATH;
 
 /** Content of uId if the path wasn't probed yet. */
-#define ACPI_CPU_PATH_NOT_PROBED UINT32_MAX
+# define ACPI_CPU_PATH_NOT_PROBED UINT32_MAX
+#endif /* RT_OS_LINUX*/
 
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
+#ifdef RT_OS_LINUX
 /** Possible combinations of all path components for level 1. */
-const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl1[] =
+static const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl1[] =
 {
     /** LNXSYSTEM:<id> */
-    {true, "LNXSYSTM:*"}
+    { true, "LNXSYSTM:*" }
 };
 
 /** Possible combinations of all path components for level 2. */
-const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl2[] =
+static const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl2[] =
 {
     /** device:<id> */
-    {true, "device:*"},
+    { true, "device:*" },
     /** LNXSYBUS:<id> */
-    {true, "LNXSYBUS:*"}
+    { true, "LNXSYBUS:*" }
 };
 
 /** Possible combinations of all path components for level 3 */
-const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl3[] =
+static const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl3[] =
 {
     /** ACPI0004:<id> */
-    {true, "ACPI0004:*"}
+    { true, "ACPI0004:*" }
 };
 
 /** Possible combinations of all path components for level 4 */
-const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl4[] =
+static const SYSFSCPUPATHCOMP g_aAcpiCpuPathLvl4[] =
 {
     /** LNXCPU:<id> */
-    {true, "LNXCPU:*"},
+    { true, "LNXCPU:*" },
     /** ACPI_CPU:<id> */
-    {true, "ACPI_CPU:*"}
+    { true, "ACPI_CPU:*" }
 };
 
 /** All possible combinations. */
-SYSFSCPUPATH g_aAcpiCpuPath[] =
+static SYSFSCPUPATH g_aAcpiCpuPath[] =
 {
     /** Level 1 */
-    {ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl1, RT_ELEMENTS(g_aAcpiCpuPathLvl1), NULL, NULL},
+    { ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl1, RT_ELEMENTS(g_aAcpiCpuPathLvl1), NULL, NULL },
     /** Level 2 */
-    {ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl2, RT_ELEMENTS(g_aAcpiCpuPathLvl2), NULL, NULL},
+    { ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl2, RT_ELEMENTS(g_aAcpiCpuPathLvl2), NULL, NULL },
     /** Level 3 */
-    {ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl3, RT_ELEMENTS(g_aAcpiCpuPathLvl3), NULL, NULL},
+    { ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl3, RT_ELEMENTS(g_aAcpiCpuPathLvl3), NULL, NULL },
     /** Level 4 */
-    {ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl4, RT_ELEMENTS(g_aAcpiCpuPathLvl4), NULL, NULL},
+    { ACPI_CPU_PATH_NOT_PROBED, g_aAcpiCpuPathLvl4, RT_ELEMENTS(g_aAcpiCpuPathLvl4), NULL, NULL },
 };
-#endif
+
+/**
+ * Possible directories to get to the topology directory for reading core and package id.
+ *
+ * @remark: This is not part of the path above because the eject file is not in one of the directories
+ *          below and would make the hot unplug code fail.
+ */
+static const char *g_apszTopologyPath[] =
+{
+    "sysdev",
+    "physical_node"
+};
+
+#endif /* RT_OS_LINUX*/
+
 
 #ifdef RT_OS_LINUX
+
 /**
  * Probes for the correct path to the ACPI CPU object in sysfs for the
  * various different kernel versions and distro's.
  *
  * @returns VBox status code.
  */
-static int VBoxServiceCpuHotPlugProbePath(void)
+static int vgsvcCpuHotPlugProbePath(void)
 {
     int rc = VINF_SUCCESS;
 
@@ -156,11 +191,11 @@ static int VBoxServiceCpuHotPlugProbePath(void)
                 PCSYSFSCPUPATHCOMP pPathComponent = &pAcpiCpuPathLvl->aComponentsPossible[iCompCurr];
 
                 /* Open the directory */
-                PRTDIR pDirCurr = NULL;
+                RTDIR hDirCurr = NIL_RTDIR;
                 char *pszPathTmp = RTPathJoinA(pszPath, pPathComponent->pcszName);
                 if (pszPathTmp)
                 {
-                    rc = RTDirOpenFiltered(&pDirCurr, pszPathTmp, RTDIRFILTER_WINNT, 0);
+                    rc = RTDirOpenFiltered(&hDirCurr, pszPathTmp, RTDIRFILTER_WINNT, 0 /*fFlags*/);
                     RTStrFree(pszPathTmp);
                 }
                 else
@@ -177,7 +212,7 @@ static int VBoxServiceCpuHotPlugProbePath(void)
                 if (pPathComponent->fNumberedSuffix)
                     cchName--;
 
-                while (RT_SUCCESS(RTDirRead(pDirCurr, &DirFolderContent, NULL))) /* Assumption that szName has always enough space */
+                while (RT_SUCCESS(RTDirRead(hDirCurr, &DirFolderContent, NULL))) /* Assumption that szName has always enough space */
                 {
                     if (   DirFolderContent.cbName >= cchName
                         && !strncmp(DirFolderContent.szName, pPathComponent->pcszName, cchName))
@@ -196,7 +231,7 @@ static int VBoxServiceCpuHotPlugProbePath(void)
                         break;
                     }
                 }
-                RTDirClose(pDirCurr);
+                RTDirClose(hDirCurr);
 
                 if (fFound)
                     break;
@@ -207,13 +242,14 @@ static int VBoxServiceCpuHotPlugProbePath(void)
                 break;
         } /* For every level */
 
-        VBoxServiceVerbose(1, "Final path after probing %s rc=%Rrc\n", pszPath, rc);
+        VGSvcVerbose(1, "Final path after probing %s rc=%Rrc\n", pszPath, rc);
         RTStrFree(pszPath);
     }
 
     return rc;
 }
 
+
 /**
  * Returns the path of the ACPI CPU device with the given core and package ID.
  *
@@ -222,13 +258,13 @@ static int VBoxServiceCpuHotPlugProbePath(void)
  * @param   idCpuCore    The core ID of the CPU.
  * @param   idCpuPackage The package ID of the CPU.
  */
-static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCpuCore, uint32_t idCpuPackage)
+static int vgsvcCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCpuCore, uint32_t idCpuPackage)
 {
     int rc = VINF_SUCCESS;
 
     AssertPtrReturn(ppszPath, VERR_INVALID_PARAMETER);
 
-    rc = VBoxServiceCpuHotPlugProbePath();
+    rc = vgsvcCpuHotPlugProbePath();
     if (RT_SUCCESS(rc))
     {
         /* Build the path from all components. */
@@ -252,7 +288,7 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
         }
 
         /* Open the directory */
-        rc = RTDirOpenFiltered(&pAcpiCpuPathLvl->pDir, pszPath, RTDIRFILTER_WINNT, 0);
+        rc = RTDirOpenFiltered(&pAcpiCpuPathLvl->hDir, pszPath, RTDIRFILTER_WINNT, 0 /*fFlags*/);
         if (RT_SUCCESS(rc))
         {
             RTStrFree(pszPath);
@@ -262,7 +298,7 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
             {
                 /* Get the next directory. */
                 RTDIRENTRY DirFolderContent;
-                rc = RTDirRead(pAcpiCpuPathLvl->pDir, &DirFolderContent, NULL);
+                rc = RTDirRead(pAcpiCpuPathLvl->hDir, &DirFolderContent, NULL);
                 if (RT_SUCCESS(rc))
                 {
                     /* Create the new path. */
@@ -277,24 +313,42 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
                     if (iLvlCurr == RT_ELEMENTS(g_aAcpiCpuPath) - 1)
                     {
                         /* Get the sysdev */
-                        uint32_t idCore    = RTLinuxSysFsReadIntFile(10, "%s/sysdev/topology/core_id",
-                                                                     pszPathCurr);
-                        uint32_t idPackage = RTLinuxSysFsReadIntFile(10, "%s/sysdev/topology/physical_package_id",
-                                                                     pszPathCurr);
+                        uint32_t idCore = 0;
+                        uint32_t idPackage = 0;
+
+                        for (unsigned i = 0; i < RT_ELEMENTS(g_apszTopologyPath); i++)
+                        {
+                            int64_t i64Core    = 0;
+                            int64_t i64Package = 0;
+
+                            int rc2 = RTLinuxSysFsReadIntFile(10, &i64Core, "%s/%s/topology/core_id",
+                                                              pszPathCurr, g_apszTopologyPath[i]);
+                            if (RT_SUCCESS(rc2))
+                                rc2 = RTLinuxSysFsReadIntFile(10, &i64Package, "%s/%s/topology/physical_package_id",
+                                                              pszPathCurr, g_apszTopologyPath[i]);
+
+                            if (RT_SUCCESS(rc2))
+                            {
+                                idCore = (uint32_t)i64Core;
+                                idPackage = (uint32_t)i64Package;
+                                break;
+                            }
+                        }
+
                         if (   idCore    == idCpuCore
                             && idPackage == idCpuPackage)
                         {
                             /* Return the path */
                             pszPath = pszPathCurr;
                             fFound = true;
-                            VBoxServiceVerbose(3, "CPU found\n");
+                            VGSvcVerbose(3, "CPU found\n");
                             break;
                         }
                         else
                         {
                             /* Get the next directory. */
                             RTStrFree(pszPathCurr);
-                            VBoxServiceVerbose(3, "CPU doesn't match, next directory\n");
+                            VGSvcVerbose(3, "CPU doesn't match, next directory\n");
                         }
                     }
                     else
@@ -302,11 +356,11 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
                         /* Go deeper */
                         iLvlCurr++;
 
-                        VBoxServiceVerbose(3, "Going deeper (iLvlCurr=%u)\n", iLvlCurr);
+                        VGSvcVerbose(3, "Going deeper (iLvlCurr=%u)\n", iLvlCurr);
 
                         pAcpiCpuPathLvl = &g_aAcpiCpuPath[iLvlCurr];
 
-                        Assert(!pAcpiCpuPathLvl->pDir);
+                        Assert(pAcpiCpuPathLvl->hDir == NIL_RTDIR);
                         Assert(!pAcpiCpuPathLvl->pszPath);
                         pAcpiCpuPathLvl->pszPath = pszPathCurr;
                         PCSYSFSCPUPATHCOMP pPathComponent = &pAcpiCpuPathLvl->aComponentsPossible[pAcpiCpuPathLvl->uId];
@@ -320,10 +374,10 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
                             break;
                         }
 
-                        VBoxServiceVerbose(3, "New path %s\n", pszPathDir);
+                        VGSvcVerbose(3, "New path %s\n", pszPathDir);
 
                         /* Open the directory */
-                        rc = RTDirOpenFiltered(&pAcpiCpuPathLvl->pDir, pszPathDir, RTDIRFILTER_WINNT, 0);
+                        rc = RTDirOpenFiltered(&pAcpiCpuPathLvl->hDir, pszPathDir, RTDIRFILTER_WINNT, 0 /*fFlags*/);
                         if (RT_FAILURE(rc))
                             break;
                     }
@@ -333,14 +387,14 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
                     /* Go back one level and try to get the next entry. */
                     Assert(iLvlCurr > 0);
 
-                    RTDirClose(pAcpiCpuPathLvl->pDir);
+                    RTDirClose(pAcpiCpuPathLvl->hDir);
                     RTStrFree(pAcpiCpuPathLvl->pszPath);
-                    pAcpiCpuPathLvl->pDir = NULL;
+                    pAcpiCpuPathLvl->hDir = NIL_RTDIR;
                     pAcpiCpuPathLvl->pszPath = NULL;
 
                     iLvlCurr--;
                     pAcpiCpuPathLvl = &g_aAcpiCpuPath[iLvlCurr];
-                    VBoxServiceVerbose(3, "Directory not found, going back (iLvlCurr=%u)\n", iLvlCurr);
+                    VGSvcVerbose(3, "Directory not found, going back (iLvlCurr=%u)\n", iLvlCurr);
                 }
             } /* while not found */
         } /* Successful init */
@@ -348,11 +402,11 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
         /* Cleanup */
         for (unsigned i = 0; i < RT_ELEMENTS(g_aAcpiCpuPath); i++)
         {
-            if (g_aAcpiCpuPath[i].pDir)
-                RTDirClose(g_aAcpiCpuPath[i].pDir);
+            if (g_aAcpiCpuPath[i].hDir)
+                RTDirClose(g_aAcpiCpuPath[i].hDir);
             if (g_aAcpiCpuPath[i].pszPath)
                 RTStrFree(g_aAcpiCpuPath[i].pszPath);
-            g_aAcpiCpuPath[i].pDir = NULL;
+            g_aAcpiCpuPath[i].hDir = NIL_RTDIR;
             g_aAcpiCpuPath[i].pszPath = NULL;
         }
         if (pszPathDir)
@@ -366,42 +420,16 @@ static int VBoxServiceCpuHotPlugGetACPIDevicePath(char **ppszPath, uint32_t idCp
 
     return rc;
 }
+
 #endif /* RT_OS_LINUX */
 
-
-/** @copydoc VBOXSERVICE::pfnPreInit */
-static DECLCALLBACK(int) VBoxServiceCpuHotPlugPreInit(void)
-{
-    return VINF_SUCCESS;
-}
-
-
-/** @copydoc VBOXSERVICE::pfnOption */
-static DECLCALLBACK(int) VBoxServiceCpuHotPlugOption(const char **ppszShort, int argc, char **argv, int *pi)
-{
-    NOREF(ppszShort);
-    NOREF(argc);
-    NOREF(argv);
-    NOREF(pi);
-
-    return -1;
-}
-
-
-/** @copydoc VBOXSERVICE::pfnInit */
-static DECLCALLBACK(int) VBoxServiceCpuHotPlugInit(void)
-{
-    return VINF_SUCCESS;
-}
-
-
 /**
  * Handles VMMDevCpuEventType_Plug.
  *
  * @param   idCpuCore       The CPU core ID.
  * @param   idCpuPackage    The CPU package ID.
  */
-static void VBoxServiceCpuHotPlugHandlePlugEvent(uint32_t idCpuCore, uint32_t idCpuPackage)
+static void vgsvcCpuHotPlugHandlePlugEvent(uint32_t idCpuCore, uint32_t idCpuPackage)
 {
 #ifdef RT_OS_LINUX
     /*
@@ -410,19 +438,19 @@ static void VBoxServiceCpuHotPlugHandlePlugEvent(uint32_t idCpuCore, uint32_t id
      * and enable every CPU which is not online already.
      * Because the directory might not be available immediately we try a few times.
      *
-     * @todo: Maybe use udev to monitor hot-add events from the kernel
      */
+    /** @todo Maybe use udev to monitor hot-add events from the kernel */
     bool fCpuOnline = false;
     unsigned cTries = 5;
 
     do
     {
-        PRTDIR pDirDevices = NULL;
-        int rc = RTDirOpen(&pDirDevices, SYSFS_CPU_PATH);
+        RTDIR hDirDevices = NULL;
+        int rc = RTDirOpen(&hDirDevices, SYSFS_CPU_PATH);
         if (RT_SUCCESS(rc))
         {
             RTDIRENTRY DirFolderContent;
-            while (RT_SUCCESS(RTDirRead(pDirDevices, &DirFolderContent, NULL))) /* Assumption that szName has always enough space */
+            while (RT_SUCCESS(RTDirRead(hDirDevices, &DirFolderContent, NULL))) /* Assumption that szName has always enough space */
             {
                 /** @todo r-bird: This code is bringing all CPUs online; the idCpuCore and
                  *        idCpuPackage parameters are unused!
@@ -454,20 +482,21 @@ static void VBoxServiceCpuHotPlugHandlePlugEvent(uint32_t idCpuCore, uint32_t id
                         RTFileClose(hFileCpuOnline);
                         if (RT_SUCCESS(rc))
                         {
-                            VBoxServiceVerbose(1, "CpuHotPlug: CPU %u/%u was brought online\n", idCpuPackage, idCpuCore);
+                            VGSvcVerbose(1, "CpuHotPlug: CPU %u/%u was brought online\n", idCpuPackage, idCpuCore);
                             fCpuOnline = true;
                             break;
                         }
                         /* Error means CPU not present or online already  */
                     }
                     else
-                        VBoxServiceError("CpuHotPlug: Failed to open \"%s/%s/online\" rc=%Rrc\n",
-                                         SYSFS_CPU_PATH, DirFolderContent.szName, rc);
+                        VGSvcError("CpuHotPlug: Failed to open '%s/%s/online' rc=%Rrc\n",
+                                   SYSFS_CPU_PATH, DirFolderContent.szName, rc);
                 }
             }
+            RTDirClose(hDirDevices);
         }
         else
-            VBoxServiceError("CpuHotPlug: Failed to open path %s rc=%Rrc\n", SYSFS_CPU_PATH, rc);
+            VGSvcError("CpuHotPlug: Failed to open path %s rc=%Rrc\n", SYSFS_CPU_PATH, rc);
 
         /* Sleep a bit */
         if (!fCpuOnline)
@@ -487,41 +516,40 @@ static void VBoxServiceCpuHotPlugHandlePlugEvent(uint32_t idCpuCore, uint32_t id
  * @param   idCpuCore       The CPU core ID.
  * @param   idCpuPackage    The CPU package ID.
  */
-static void VBoxServiceCpuHotPlugHandleUnplugEvent(uint32_t idCpuCore, uint32_t idCpuPackage)
+static void vgsvcCpuHotPlugHandleUnplugEvent(uint32_t idCpuCore, uint32_t idCpuPackage)
 {
 #ifdef RT_OS_LINUX
     char *pszCpuDevicePath = NULL;
-    int rc = VBoxServiceCpuHotPlugGetACPIDevicePath(&pszCpuDevicePath, idCpuCore, idCpuPackage);
+    int rc = vgsvcCpuHotPlugGetACPIDevicePath(&pszCpuDevicePath, idCpuCore, idCpuPackage);
     if (RT_SUCCESS(rc))
     {
         RTFILE hFileCpuEject;
-        rc = RTFileOpenF(&hFileCpuEject, RTFILE_O_WRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE,
-                         "%s/eject", pszCpuDevicePath);
+        rc = RTFileOpenF(&hFileCpuEject, RTFILE_O_WRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE, "%s/eject", pszCpuDevicePath);
         if (RT_SUCCESS(rc))
         {
             /* Write a 1 to eject the CPU */
             rc = RTFileWrite(hFileCpuEject, "1", 1, NULL);
             if (RT_SUCCESS(rc))
-                VBoxServiceVerbose(1, "CpuHotPlug: CPU %u/%u was ejected\n", idCpuPackage, idCpuCore);
+                VGSvcVerbose(1, "CpuHotPlug: CPU %u/%u was ejected\n", idCpuPackage, idCpuCore);
             else
-                VBoxServiceError("CpuHotPlug: Failed to eject CPU %u/%u rc=%Rrc\n", idCpuPackage, idCpuCore, rc);
+                VGSvcError("CpuHotPlug: Failed to eject CPU %u/%u rc=%Rrc\n", idCpuPackage, idCpuCore, rc);
 
             RTFileClose(hFileCpuEject);
         }
         else
-            VBoxServiceError("CpuHotPlug: Failed to open \"%s/eject\" rc=%Rrc\n", pszCpuDevicePath, rc);
+            VGSvcError("CpuHotPlug: Failed to open '%s/eject' rc=%Rrc\n", pszCpuDevicePath, rc);
         RTStrFree(pszCpuDevicePath);
     }
     else
-        VBoxServiceError("CpuHotPlug: Failed to get CPU device path rc=%Rrc\n", rc);
+        VGSvcError("CpuHotPlug: Failed to get CPU device path rc=%Rrc\n", rc);
 #else
 # error "Port me"
 #endif
 }
 
 
-/** @copydoc VBOXSERVICE::pfnWorker */
-DECLCALLBACK(int) VBoxServiceCpuHotPlugWorker(bool volatile *pfShutdown)
+/** @interface_method_impl{VBOXSERVICE,pfnWorker} */
+static DECLCALLBACK(int) vgsvcCpuHotPlugWorker(bool volatile *pfShutdown)
 {
     /*
      * Tell the control thread that it can continue spawning services.
@@ -540,38 +568,38 @@ DECLCALLBACK(int) VBoxServiceCpuHotPlugWorker(bool volatile *pfShutdown)
      */
     for (;;)
     {
-        /* Wait for CPU hot plugging event. */
+        /* Wait for CPU hot-plugging event. */
         uint32_t            idCpuCore;
         uint32_t            idCpuPackage;
         VMMDevCpuEventType  enmEventType;
         rc = VbglR3CpuHotPlugWaitForEvent(&enmEventType, &idCpuCore, &idCpuPackage);
         if (RT_SUCCESS(rc))
         {
-            VBoxServiceVerbose(3, "CpuHotPlug: Event happened idCpuCore=%u idCpuPackage=%u enmEventType=%d\n",
-                               idCpuCore, idCpuPackage, enmEventType);
+            VGSvcVerbose(3, "CpuHotPlug: Event happened idCpuCore=%u idCpuPackage=%u enmEventType=%d\n",
+                         idCpuCore, idCpuPackage, enmEventType);
             switch (enmEventType)
             {
                 case VMMDevCpuEventType_Plug:
-                    VBoxServiceCpuHotPlugHandlePlugEvent(idCpuCore, idCpuPackage);
+                    vgsvcCpuHotPlugHandlePlugEvent(idCpuCore, idCpuPackage);
                     break;
 
                 case VMMDevCpuEventType_Unplug:
-                    VBoxServiceCpuHotPlugHandleUnplugEvent(idCpuCore, idCpuPackage);
+                    vgsvcCpuHotPlugHandleUnplugEvent(idCpuCore, idCpuPackage);
                     break;
 
                 default:
                 {
                     static uint32_t s_iErrors = 0;
                     if (s_iErrors++ < 10)
-                        VBoxServiceError("CpuHotPlug: Unknown event: idCpuCore=%u idCpuPackage=%u enmEventType=%d\n",
-                                         idCpuCore, idCpuPackage, enmEventType);
+                        VGSvcError("CpuHotPlug: Unknown event: idCpuCore=%u idCpuPackage=%u enmEventType=%d\n",
+                                   idCpuCore, idCpuPackage, enmEventType);
                     break;
                 }
             }
         }
         else if (rc != VERR_INTERRUPTED && rc != VERR_TRY_AGAIN)
         {
-            VBoxServiceError("CpuHotPlug: VbglR3CpuHotPlugWaitForEvent returned %Rrc\n", rc);
+            VGSvcError("CpuHotPlug: VbglR3CpuHotPlugWaitForEvent returned %Rrc\n", rc);
             break;
         }
 
@@ -584,40 +612,33 @@ DECLCALLBACK(int) VBoxServiceCpuHotPlugWorker(bool volatile *pfShutdown)
 }
 
 
-/** @copydoc VBOXSERVICE::pfnStop */
-static DECLCALLBACK(void) VBoxServiceCpuHotPlugStop(void)
+/** @interface_method_impl{VBOXSERVICE,pfnStop} */
+static DECLCALLBACK(void) vgsvcCpuHotPlugStop(void)
 {
     VbglR3InterruptEventWaits();
     return;
 }
 
 
-/** @copydoc VBOXSERVICE::pfnTerm */
-static DECLCALLBACK(void) VBoxServiceCpuHotPlugTerm(void)
-{
-    return;
-}
-
-
 /**
- * The 'timesync' service description.
+ * The 'CpuHotPlug' service description.
  */
 VBOXSERVICE g_CpuHotPlug =
 {
     /* pszName. */
     "cpuhotplug",
     /* pszDescription. */
-    "CPU hot plugging monitor",
+    "CPU hot-plugging monitor",
     /* pszUsage. */
     NULL,
     /* pszOptions. */
     NULL,
     /* methods */
-    VBoxServiceCpuHotPlugPreInit,
-    VBoxServiceCpuHotPlugOption,
-    VBoxServiceCpuHotPlugInit,
-    VBoxServiceCpuHotPlugWorker,
-    VBoxServiceCpuHotPlugStop,
-    VBoxServiceCpuHotPlugTerm
+    VGSvcDefaultPreInit,
+    VGSvcDefaultOption,
+    VGSvcDefaultInit,
+    vgsvcCpuHotPlugWorker,
+    vgsvcCpuHotPlugStop,
+    VGSvcDefaultTerm
 };
 
