@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2009-2015 Oracle Corporation
+ * Copyright (C) 2009-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,13 +17,16 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___CROPENGL_DRIDRV_H
-#define ___CROPENGL_DRIDRV_H
+#ifndef GA_INCLUDED_SRC_common_crOpenGL_dri_drv_h
+#define GA_INCLUDED_SRC_common_crOpenGL_dri_drv_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include "src/mesa/main/mtypes.h"
 #include "src/mesa/drivers/dri/common/dri_util.h"
 #include "src/mesa/glapi/dispatch.h"
 #include "src/mesa/main/dd.h"
 
-#endif /* ___CROPENGL_DRIDRV_H */
+#endif /* !GA_INCLUDED_SRC_common_crOpenGL_dri_drv_h */
 
