@@ -1,10 +1,11 @@
 #! /bin/sh
-#
-# Linux Additions kernel module init script ($Revision: 116785 $)
+# $Id: vboxadd.sh $
+## @file
+# Linux Additions kernel module init script ($Revision: 127855 $)
 #
 
 #
-# Copyright (C) 2006-2012 Oracle Corporation
+# Copyright (C) 2006-2019 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -49,13 +50,29 @@
 #   and stop.
 # * Uninstalling the Additions and re-installing them does not trigger warnings.
 
+export LC_ALL=C
 PATH=$PATH:/bin:/sbin:/usr/sbin
 PACKAGE=VBoxGuestAdditions
-LOG="/var/log/vboxadd-install.log"
 MODPROBE=/sbin/modprobe
 OLDMODULES="vboxguest vboxadd vboxsf vboxvfs vboxvideo"
-SCRIPTNAME=vboxadd.sh
+SERVICE="VirtualBox Guest Additions"
 QUICKSETUP=
+## systemd logs information about service status, otherwise do that ourselves.
+QUIET=
+test -z "${TARGET_VER}" && TARGET_VER=`uname -r`
+# Marker to ignore a particular kernel version which was already installed.
+SKIPFILE_BASE=/var/lib/VBoxGuestAdditions/skip
+
+setup_log()
+{
+    test -z "${LOG}" || return 0
+    # Rotate log files
+    LOG="/var/log/vboxadd-setup.log"
+    mv "${LOG}.3" "${LOG}.4" 2>/dev/null
+    mv "${LOG}.2" "${LOG}.3" 2>/dev/null
+    mv "${LOG}.1" "${LOG}.2" 2>/dev/null
+    mv "${LOG}" "${LOG}.1" 2>/dev/null
+}
 
 if $MODPROBE -c 2>/dev/null | grep -q '^allow_unsupported_modules  *0'; then
   MODPROBE="$MODPROBE --allow-unsupported-modules"
@@ -89,27 +106,39 @@ fi
 
 begin()
 {
-    test -n "${2}" && echo "${SCRIPTNAME}: ${1}."
-    logger -t "${SCRIPTNAME}" "${1}."
+    test -n "${QUIET}" || echo "${SERVICE}: ${1}"
 }
 
-succ_msg()
+info()
 {
-    logger -t "${SCRIPTNAME}" "${1}."
-}
-
-show_error()
-{
-    echo "${SCRIPTNAME}: failed: ${1}." >&2
-    logger -t "${SCRIPTNAME}" "${1}."
+    if test -z "${QUIET}"; then
+        echo "${SERVICE}: $1"
+    else
+        echo "$1"
+    fi
 }
 
 fail()
 {
-    show_error "$1"
+    log "${1}"
+    echo "$1" >&2
+    echo "The log file $LOG may contain further information." >&2
     exit 1
 }
 
+log()
+{
+    setup_log
+    echo "${1}" >> "${LOG}"
+}
+
+module_build_log()
+{
+    log "Error building the module.  Build output follows."
+    echo ""
+    echo "${1}" >> "${LOG}"
+}
+
 dev=/dev/vboxguest
 userdev=/dev/vboxuser
 config=/var/lib/VBoxGuestAdditions/config
@@ -156,7 +185,7 @@ do_vboxguest_non_udev()
                 maj=10
             fi
         fi
-        test -z "$maj" && {
+        test -n "$maj" || {
             rmmod vboxguest 2>/dev/null
             fail "Cannot locate the VirtualBox device"
         }
@@ -194,29 +223,21 @@ do_vboxguest_non_udev()
 
 start()
 {
-    begin "Starting the VirtualBox Guest Additions" console;
-    # If we got this far assume that the slow set-up has been done.
-    QUICKSETUP=yes
+    begin "Starting."
     if test -z "${INSTALL_NO_MODULE_BUILDS}"; then
-        uname -r | grep -q -E '^2\.6|^3|^4' 2>/dev/null &&
+        setup --quick
+        test -d /sys &&
             ps -A -o comm | grep -q '/*udevd$' 2>/dev/null ||
             no_udev=1
         running_vboxguest || {
             rm -f $dev || {
                 fail "Cannot remove $dev"
             }
-
             rm -f $userdev || {
                 fail "Cannot remove $userdev"
             }
-
-            $MODPROBE vboxguest >/dev/null 2>&1 || {
-                setup
-                $MODPROBE vboxguest >/dev/null 2>&1 || {
-                    ${INSTALL_DIR}/init/vboxadd-x11 cleanup
-                    fail "modprobe vboxguest failed"
-                }
-            }
+            $MODPROBE vboxguest >/dev/null 2>&1 ||
+                fail "modprobe vboxguest failed"
             case "$no_udev" in 1)
                 sleep .5;;
             esac
@@ -226,51 +247,28 @@ start()
         esac
 
         running_vboxsf || {
-            $MODPROBE vboxsf > /dev/null 2>&1 || {
-                if dmesg | grep "VbglR0SfConnect failed" > /dev/null 2>&1; then
-                    show_error "Unable to start shared folders support.  Make sure that your VirtualBox build"
-                    show_error "supports this feature."
-                else
-                    show_error "modprobe vboxsf failed"
-                fi
-            }
+            $MODPROBE vboxsf > /dev/null 2>&1 ||
+                info "modprobe vboxsf failed"
         }
     fi  # INSTALL_NO_MODULE_BUILDS
 
     # Put the X.Org driver in place.  This is harmless if it is not needed.
-    /sbin/rcvboxadd-x11 setup
-    # Install the guest OpenGL drivers.  For now we don't support
-    # multi-architecture installations
-    rm -f /etc/ld.so.conf.d/00vboxvideo.conf
-    rm -Rf /var/lib/VBoxGuestAdditions/lib
-    if /usr/bin/VBoxClient --check3d 2>/dev/null; then
-        mkdir -p /var/lib/VBoxGuestAdditions/lib
-        ln -sf "${INSTALL_DIR}/lib/VBoxOGL.so" /var/lib/VBoxGuestAdditions/lib/libGL.so.1
-        # SELinux for the OpenGL libraries, so that gdm can load them during the
-        # acceleration support check.  This prevents an "Oh no, something has gone
-        # wrong!" error when starting EL7 guests.
-        if test -e /etc/selinux/config; then
-            if command -v semanage > /dev/null; then
-                semanage fcontext -a -t lib_t "/var/lib/VBoxGuestAdditions/lib/libGL.so.1"
-            fi
-            chcon -h  -t lib_t "/var/lib/VBoxGuestAdditions/lib/libGL.so.1"
-        fi
-        echo "/var/lib/VBoxGuestAdditions/lib" > /etc/ld.so.conf.d/00vboxvideo.conf
-    fi
-    ldconfig
+    # Also set up the OpenGL library.
+    myerr=`"${INSTALL_DIR}/init/vboxadd-x11" setup 2>&1`
+    test -z "${myerr}" || log "${myerr}"
 
     # Mount all shared folders from /etc/fstab. Normally this is done by some
     # other startup script but this requires the vboxdrv kernel module loaded.
     # This isn't necessary anymore as the vboxsf module is autoloaded.
     # mount -a -t vboxsf
 
-    succ_msg
     return 0
 }
 
 stop()
 {
-    begin "Stopping VirtualBox Additions" console;
+    begin "Stopping."
+    test -n "${INSTALL_NO_MODULE_BUILDS}" || setup --quick
     if test -r /etc/ld.so.conf.d/00vboxvideo.conf; then
         rm /etc/ld.so.conf.d/00vboxvideo.conf
         ldconfig
@@ -278,13 +276,8 @@ stop()
     if ! umount -a -t vboxsf 2>/dev/null; then
         fail "Cannot unmount vboxsf folders"
     fi
-    test -n "${INSTALL_NO_MODULE_BUILDS}" && return 0
-    modprobe -q -r -a vboxvideo vboxsf vboxguest
-    egrep -q 'vboxguest|vboxsf|vboxvideo' /proc/modules &&
-        echo "You may need to restart your guest system to finish removing the guest drivers."
-    rm -f $userdev || fail "Cannot unlink $userdev"
-    rm -f $dev || fail "Cannot unlink $dev"
-    succ_msg
+    test -n "${INSTALL_NO_MODULE_BUILDS}" ||
+        info "You may need to restart your guest system to finish removing the guest drivers."
     return 0
 }
 
@@ -294,77 +287,123 @@ restart()
     return 0
 }
 
+## Update the initramfs.  Debian and Ubuntu put the graphics driver in, and
+# need the touch(1) command below.  Everyone else that I checked just need
+# the right module alias file from depmod(1) and only use the initramfs to
+# load the root filesystem, not the boot splash.  update-initramfs works
+# for the first two and dracut for every one else I checked.  We are only
+# interested in distributions recent enough to use the KMS vboxvideo driver.
+update_initramfs()
+{
+    ## kernel version to update for.
+    version="${1}"
+    depmod "${version}"
+    rm -f "/lib/modules/${version}/initrd/vboxvideo"
+    test ! -d "/lib/modules/${version}/initrd" ||
+        test ! -f "/lib/modules/${version}/misc/vboxvideo.ko" ||
+        touch "/lib/modules/${version}/initrd/vboxvideo"
+
+    # Systems without systemd-inhibit probably don't need their initramfs
+    # rebuild here anyway.
+    type systemd-inhibit >/dev/null 2>&1 || return
+    if type dracut >/dev/null 2>&1; then
+        systemd-inhibit --why="Installing VirtualBox Guest Additions" \
+            dracut -f --kver "${version}"
+    elif type update-initramfs >/dev/null 2>&1; then
+        systemd-inhibit --why="Installing VirtualBox Guest Additions" \
+            update-initramfs -u -k "${version}"
+    fi
+}
+
 # Remove any existing VirtualBox guest kernel modules from the disk, but not
 # from the kernel as they may still be in use
 cleanup_modules()
 {
-    begin "Removing existing VirtualBox kernel modules"
+    # Needed for Ubuntu and Debian, see update_initramfs
+    rm -f /lib/modules/*/initrd/vboxvideo
+    for i in /lib/modules/*/misc; do
+        KERN_VER="${i%/misc}"
+        KERN_VER="${KERN_VER#/lib/modules/}"
+        unset do_update
+        for j in ${OLDMODULES}; do
+            test -f "${i}/${j}.ko" && do_update=1 && rm -f "${i}/${j}.ko"
+        done
+        test -z "$do_update" || update_initramfs "$KERN_VER"
+        # Remove empty /lib/modules folders which may have been kept around
+        rmdir -p "${i}" 2>/dev/null || true
+        unset keep
+        for j in /lib/modules/"${KERN_VER}"/*; do
+            name="${j##*/}"
+            test -d "${name}" || test "${name%%.*}" != modules && keep=1
+        done
+        if test -z "${keep}"; then
+            rm -rf /lib/modules/"${KERN_VER}"
+            rm -f /boot/initrd.img-"${KERN_VER}"
+        fi
+    done
     for i in ${OLDMODULES}; do
         # We no longer support DKMS, remove any leftovers.
         rm -rf "/var/lib/dkms/${i}"*
-        # And remove old modules.
-        rm -f /lib/modules/*/misc/"${i}"*
-    done
-    # Remove leftover module folders.
-    for i in /lib/modules/*/misc; do
-        test -d "${i}" && rmdir -p "${i}" 2>/dev/null
     done
     rm -f /etc/depmod.d/vboxvideo-upstream.conf
-    succ_msg
+    rm -f "$SKIPFILE_BASE"-*
 }
 
 # Build and install the VirtualBox guest kernel modules
 setup_modules()
 {
-    # don't stop the old modules here -- they might be in use
-    test -z "${QUICKSETUP}" && cleanup_modules
-    # This does not work for 2.4 series kernels.  How sad.
-    test -n "${QUICKSETUP}" && test -f "${MODULE_DIR}/vboxguest.ko" && return 0
-    begin "Building the VirtualBox Guest Additions kernel modules"
+    KERN_VER="$1"
+    test -n "$KERN_VER" || return 1
+    test ! -f /lib/modules/"$KERN_VER"/misc/vboxguest.ko || return 0
+    test ! -f /lib/modules/"$KERN_VER"/misc/vboxguest.o || return 0
+    test -d /lib/modules/"$KERN_VER"/build || return 0
+    test ! -f "$SKIPFILE_BASE"-"$KERN_VER" || return 0
+    export KERN_VER
+    info "Building the modules for kernel $KERN_VER."
 
-    begin "Building the main Guest Additions module"
-    if ! $BUILDINTMP \
+    log "Building the main Guest Additions module for kernel $KERN_VER."
+    if ! myerr=`$BUILDINTMP \
         --save-module-symvers /tmp/vboxguest-Module.symvers \
         --module-source $MODULE_SRC/vboxguest \
-        --no-print-directory install >> $LOG 2>&1; then
-        show_error "Look at $LOG to find out what went wrong"
+        --no-print-directory install 2>&1`; then
+        # If check_module_dependencies.sh fails it prints a message itself.
+        module_build_log "$myerr"
+        "${INSTALL_DIR}"/other/check_module_dependencies.sh 2>&1 &&
+            info "Look at $LOG to find out what went wrong"
         return 0
     fi
-    succ_msg
-    begin "Building the shared folder support module"
-    if ! $BUILDINTMP \
+    log "Building the shared folder support module."
+    if ! myerr=`$BUILDINTMP \
         --use-module-symvers /tmp/vboxguest-Module.symvers \
         --module-source $MODULE_SRC/vboxsf \
-        --no-print-directory install >> $LOG 2>&1; then
-        show_error  "Look at $LOG to find out what went wrong"
+        --no-print-directory install 2>&1`; then
+        module_build_log "$myerr"
+        info  "Look at $LOG to find out what went wrong"
         return 0
     fi
-    succ_msg
-    begin "Building the graphics driver module"
-    if ! $BUILDINTMP \
+    log "Building the graphics driver module."
+    if ! myerr=`$BUILDINTMP \
         --use-module-symvers /tmp/vboxguest-Module.symvers \
         --module-source $MODULE_SRC/vboxvideo \
-        --no-print-directory install >> $LOG 2>&1; then
-        show_error "Look at $LOG to find out what went wrong"
+        --no-print-directory install 2>&1`; then
+        module_build_log "$myerr"
+        info "Look at $LOG to find out what went wrong"
     fi
-    succ_msg
     [ -d /etc/depmod.d ] || mkdir /etc/depmod.d
     echo "override vboxguest * misc" > /etc/depmod.d/vboxvideo-upstream.conf
     echo "override vboxsf * misc" >> /etc/depmod.d/vboxvideo-upstream.conf
     echo "override vboxvideo * misc" >> /etc/depmod.d/vboxvideo-upstream.conf
-    depmod
+    update_initramfs "${KERN_VER}"
     return 0
 }
 
 create_vbox_user()
 {
-    begin "Doing non-kernel setup of the Guest Additions"
-    echo "Creating user for the Guest Additions." >> $LOG
     # This is the LSB version of useradd and should work on recent
     # distributions
-    useradd -d /var/run/vboxadd -g 1 -r -s /bin/false vboxadd >/dev/null 2>&1
+    useradd -d /var/run/vboxadd -g 1 -r -s /bin/false vboxadd >/dev/null 2>&1 || true
     # And for the others, we choose a UID ourselves
-    useradd -d /var/run/vboxadd -g 1 -u 501 -o -s /bin/false vboxadd >/dev/null 2>&1
+    useradd -d /var/run/vboxadd -g 1 -u 501 -o -s /bin/false vboxadd >/dev/null 2>&1 || true
 
 }
 
@@ -372,7 +411,6 @@ create_udev_rule()
 {
     # Create udev description file
     if [ -d /etc/udev/rules.d ]; then
-        echo "Creating udev rule for the Guest Additions kernel module." >> $LOG
         udev_call=""
         udev_app=`which udevadm 2> /dev/null`
         if [ $? -eq 0 ]; then
@@ -404,9 +442,9 @@ create_module_rebuild_script()
     mkdir -p /etc/kernel/postinst.d /etc/kernel/prerm.d
     cat << EOF > /etc/kernel/postinst.d/vboxadd
 #!/bin/sh
-test -d "/lib/modules/\${1}/build" || exit 0
-KERN_DIR="/lib/modules/\${1}/build" MODULE_DIR="/lib/modules/\${1}/misc" \
-/sbin/rcvboxadd quicksetup
+# This only works correctly on Debian derivatives - Red Hat calls it before
+# installing the right header files.
+/sbin/rcvboxadd quicksetup "\${1}"
 exit 0
 EOF
     cat << EOF > /etc/kernel/prerm.d/vboxadd
@@ -428,17 +466,16 @@ shared_folder_setup()
     # Put the mount.vboxsf mount helper in the right place.
     ## @todo It would be nicer if the kernel module just parsed parameters
     # itself instead of needing a separate binary to do that.
-    ln -sf "$lib_path/$PACKAGE/mount.vboxsf" /sbin
+    ln -sf "${INSTALL_DIR}/other/mount.vboxsf" /sbin
     # SELinux security context for the mount helper.
     if test -e /etc/selinux/config; then
         # This is correct.  semanage maps this to the real path, and it aborts
         # with an error, telling you what you should have typed, if you specify
         # the real path.  The "chcon" is there as a back-up for old guests.
         command -v semanage > /dev/null &&
-            semanage fcontext -a -t mount_exec_t "/usr/lib/$PACKAGE/mount.vboxsf"
-        chcon -t mount_exec_t "$lib_path/$PACKAGE/mount.vboxsf"
+            semanage fcontext -a -t mount_exec_t "${INSTALL_DIR}/other/mount.vboxsf"
+        chcon -t mount_exec_t "${INSTALL_DIR}/other/mount.vboxsf"
     fi
-    succ_msg
 }
 
 # setup_script
@@ -447,19 +484,39 @@ setup()
     export BUILD_TYPE
     export USERNAME
 
-    rm -f $LOG
+    test x"$1" != x--quick || QUICKSETUP=true
+    test -n "$QUICKSETUP" || cleanup
     MODULE_SRC="$INSTALL_DIR/src/vboxguest-$INSTALL_VER"
     BUILDINTMP="$MODULE_SRC/build_in_tmp"
-    chcon -t bin_t "$BUILDINTMP" > /dev/null 2>&1
+    test ! -e /etc/selinux/config ||
+        chcon -t bin_t "$BUILDINTMP"
 
-    test -z "${INSTALL_NO_MODULE_BUILDS}" && setup_modules
+    if test -z "$INSTALL_NO_MODULE_BUILDS"; then
+        if test -z "$QUICKSETUP"; then
+            info "Building the VirtualBox Guest Additions kernel modules.  This may take a while."
+            info "To build modules for other installed kernels, run"
+            info "  /sbin/rcvboxadd quicksetup <version>"
+            for setupi in /lib/modules/*; do
+                KERN_VER="${setupi##*/}"
+                # For a full setup, mark kernels we do not want to build.
+                touch "$SKIPFILE_BASE"-"$KERN_VER"
+            done
+        fi
+        # That is, we mark all but the requested kernel.
+        rm -f "$SKIPFILE_BASE"-"$TARGET_VER"
+        for setupi in /lib/modules/*; do
+            KERN_VER="${setupi##*/}"
+            setup_modules "$KERN_VER"
+        done
+        depmod
+    fi
     create_vbox_user
     create_udev_rule
-    test -z "${INSTALL_NO_MODULE_BUILDS}" && create_module_rebuild_script
-    test -n "${QUICKSETUP}" && return 0
+    test -n "${INSTALL_NO_MODULE_BUILDS}" || create_module_rebuild_script
+    test -z "$QUICKSETUP" || return 0
     shared_folder_setup
     if  running_vboxguest || running_vboxadd; then
-        begin "Running kernel modules will not be replaced until the system is restarted"
+        info "Running kernel modules will not be replaced until the system is restarted"
     fi
     return 0
 }
@@ -479,14 +536,14 @@ cleanup()
     fi
 
     # Clean-up X11-related bits
-    ${INSTALL_DIR}/init/vboxadd-x11 cleanup
+    "${INSTALL_DIR}/init/vboxadd-x11" cleanup
 
     # Remove other files
-    rm /sbin/mount.vboxsf 2>/dev/null
     if test -z "${INSTALL_NO_MODULE_BUILDS}"; then
         rm -f /etc/kernel/postinst.d/vboxadd /etc/kernel/prerm.d/vboxadd
         rmdir -p /etc/kernel/postinst.d /etc/kernel/prerm.d 2>/dev/null
     fi
+    rm /sbin/mount.vboxsf 2>/dev/null
     rm /etc/udev/rules.d/60-vboxadd.rules 2>/dev/null
 }
 
@@ -499,6 +556,9 @@ dmnstatus()
     fi
 }
 
+case "$2" in quiet)
+    QUIET=yes;;
+esac
 case "$1" in
 start)
     start
@@ -514,8 +574,8 @@ setup)
     start
     ;;
 quicksetup)
-    QUICKSETUP=yes
-    setup
+    test -z "$2" || test ! -d /lib/modules/"$2"/build || TARGET_VER="$2"
+    setup --quick
     ;;
 cleanup)
     cleanup
@@ -524,7 +584,7 @@ status)
     dmnstatus
     ;;
 *)
-    echo "Usage: $0 {start|stop|restart|status|setup|quicksetup|cleanup}"
+    echo "Usage: $0 {start|stop|restart|status|setup|quicksetup|cleanup} [quiet]"
     exit 1
 esac
 
