@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2016 Oracle Corporation
+ * Copyright (C) 2013-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,6 +19,7 @@
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
+#include <stdio.h>
 #define LOG_GROUP LOG_GROUP_DEV_VMSVGA
 #include <VBox/vmm/pdmdev.h>
 #include <VBox/err.h>
@@ -28,7 +29,7 @@
 #include <iprt/mem.h>
 
 #include <VBox/vmm/pgm.h> /* required by DevVGA.h */
-#include <VBox/VBoxVideo.h> /* required by DevVGA.h */
+#include <VBoxVideo.h> /* required by DevVGA.h */
 
 /* should go BEFORE any other DevVGA include to make all DevVGA.h config defines be visible */
 #include "DevVGA.h"
@@ -96,6 +97,8 @@ static const VMSVGAINFOENUM g_aSVGA3dSurfaceFormats[] =
     { SVGA3D_Z_DF16             , "Z_DF16" },
     { SVGA3D_Z_DF24             , "Z_DF24" },
     { SVGA3D_Z_D24S8_INT        , "Z_D24S8_INT" },
+    { SVGA3D_R8G8B8A8_SNORM     , "R8G8B8A8_SNORM" },
+    { SVGA3D_R16G16_UNORM       , "R16G16_UNORM" },
 };
 VMSVGAINFOENUMMAP_MAKE(RT_NOTHING, g_SVGA3dSurfaceFormat2String, g_aSVGA3dSurfaceFormats, "SVGA3D_");
 
@@ -266,10 +269,7 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
      * surfaces both for OpenGL and D3D, so skip these here (don't
      * wast memory on them).
      */
-    uint32_t const fSwitchFlags = pSurface->flags
-                                & (  SVGA3D_SURFACE_HINT_INDEXBUFFER  | SVGA3D_SURFACE_HINT_VERTEXBUFFER
-                                   | SVGA3D_SURFACE_HINT_TEXTURE      | SVGA3D_SURFACE_HINT_RENDERTARGET
-                                   | SVGA3D_SURFACE_HINT_DEPTHSTENCIL | SVGA3D_SURFACE_CUBEMAP);
+    uint32_t const fSwitchFlags = pSurface->surfaceFlags & VMSVGA3D_SURFACE_HINT_SWITCH_MASK;
     if (   fSwitchFlags != SVGA3D_SURFACE_HINT_DEPTHSTENCIL
         && fSwitchFlags != (SVGA3D_SURFACE_HINT_DEPTHSTENCIL | SVGA3D_SURFACE_HINT_TEXTURE))
     {
@@ -298,7 +298,7 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
 #endif
                 {
                     Assert(pMipmapLevel->cbSurface);
-                    Assert(pMipmapLevel->cbSurface == pMipmapLevel->cbSurfacePitch * pMipmapLevel->size.height); /* correct for depth stuff? */
+                    Assert(pMipmapLevel->cbSurface == pMipmapLevel->cbSurfacePlane * pMipmapLevel->mipmapSize.depth);
 
                     /*
                      * Make sure we've got surface memory buffer.
@@ -314,23 +314,37 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
                     /*
                      * D3D specifics.
                      */
-                    HRESULT  hr;
-                    switch (fSwitchFlags)
+                    Assert(pSurface->enmD3DResType != VMSVGA3D_D3DRESTYPE_NONE);
+
+                    HRESULT hr;
+                    switch (pSurface->enmD3DResType)
                     {
-                        case SVGA3D_SURFACE_HINT_TEXTURE:
-                        case SVGA3D_SURFACE_HINT_RENDERTARGET:
-                        case SVGA3D_SURFACE_HINT_TEXTURE | SVGA3D_SURFACE_HINT_RENDERTARGET:
+                        case VMSVGA3D_D3DRESTYPE_VOLUME_TEXTURE:
+                            AssertFailed(); /// @todo
+                            break;
+
+                        case VMSVGA3D_D3DRESTYPE_SURFACE:
+                        case VMSVGA3D_D3DRESTYPE_TEXTURE:
+                        case VMSVGA3D_D3DRESTYPE_CUBE_TEXTURE:
                         {
                             /*
                              * Lock the buffer and make it accessible to memcpy.
                              */
                             D3DLOCKED_RECT LockedRect;
-                            if (fSwitchFlags & SVGA3D_SURFACE_HINT_TEXTURE)
+                            if (pSurface->enmD3DResType == VMSVGA3D_D3DRESTYPE_CUBE_TEXTURE)
+                            {
+                                hr = pSurface->u.pCubeTexture->LockRect(vmsvga3dCubemapFaceFromIndex(iFace),
+                                                                        i, /* texture level */
+                                                                        &LockedRect,
+                                                                        NULL,
+                                                                        D3DLOCK_READONLY);
+                            }
+                            else if (pSurface->enmD3DResType == VMSVGA3D_D3DRESTYPE_TEXTURE)
                             {
                                 if (pSurface->bounce.pTexture)
                                 {
                                     if (    !pSurface->fDirty
-                                        &&  fSwitchFlags == (SVGA3D_SURFACE_HINT_TEXTURE | SVGA3D_SURFACE_HINT_RENDERTARGET)
+                                        &&  RT_BOOL(fSwitchFlags & SVGA3D_SURFACE_HINT_RENDERTARGET)
                                         &&  i == 0 /* only the first time */)
                                     {
                                         /** @todo stricter checks for associated context */
@@ -381,7 +395,7 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
                             if (pMipmapLevel->cbSurfacePitch == (uint32_t)LockedRect.Pitch)
                                 memcpy(pbDst, LockedRect.pBits, pMipmapLevel->cbSurface);
                             else
-                                for (uint32_t j = 0; j < pMipmapLevel->size.height; j++)
+                                for (uint32_t j = 0; j < pMipmapLevel->cBlocksY; j++)
                                     memcpy(pbDst + j * pMipmapLevel->cbSurfacePitch,
                                            (uint8_t *)LockedRect.pBits + j * LockedRect.Pitch,
                                            pMipmapLevel->cbSurfacePitch);
@@ -405,34 +419,31 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
                             break;
                         }
 
-                        case SVGA3D_SURFACE_HINT_VERTEXBUFFER:
+                        case VMSVGA3D_D3DRESTYPE_VERTEX_BUFFER:
+                        case VMSVGA3D_D3DRESTYPE_INDEX_BUFFER:
                         {
+                            /* Current type of the buffer. */
+                            const bool fVertex = (pSurface->enmD3DResType == VMSVGA3D_D3DRESTYPE_VERTEX_BUFFER);
+
                             void *pvD3DData = NULL;
-                            hr = pSurface->u.pVertexBuffer->Lock(0, 0, &pvD3DData, D3DLOCK_READONLY);
-                            AssertMsgReturn(hr == D3D_OK, ("Lock vertex failed with %x\n", hr), VERR_INTERNAL_ERROR);
+                            if (fVertex)
+                                hr = pSurface->u.pVertexBuffer->Lock(0, 0, &pvD3DData, D3DLOCK_READONLY);
+                            else
+                                hr = pSurface->u.pIndexBuffer->Lock(0, 0, &pvD3DData, D3DLOCK_READONLY);
+                            AssertMsgReturn(hr == D3D_OK, ("Lock %s failed with %x\n", fVertex ? "vertex" : "index", hr), VERR_INTERNAL_ERROR);
 
                             memcpy(pbDst, pvD3DData, pMipmapLevel->cbSurface);
 
-                            hr = pSurface->u.pVertexBuffer->Unlock();
-                            AssertMsg(hr == D3D_OK, ("Unlock vertex failed with %x\n", hr));
-                            break;
-                        }
-
-                        case SVGA3D_SURFACE_HINT_INDEXBUFFER:
-                        {
-                            void *pvD3DData = NULL;
-                            hr = pSurface->u.pIndexBuffer->Lock(0, 0, &pvD3DData, D3DLOCK_READONLY);
-                            AssertMsgReturn(hr == D3D_OK, ("Lock index failed with %x\n", hr), VERR_INTERNAL_ERROR);
-
-                            memcpy(pbDst, pvD3DData, pMipmapLevel->cbSurface);
-
-                            hr = pSurface->u.pIndexBuffer->Unlock();
-                            AssertMsg(hr == D3D_OK, ("Unlock index failed with %x\n", hr));
+                            if (fVertex)
+                                hr = pSurface->u.pVertexBuffer->Unlock();
+                            else
+                                hr = pSurface->u.pIndexBuffer->Unlock();
+                            AssertMsg(hr == D3D_OK, ("Unlock %s failed with %x\n", fVertex ? "vertex" : "index", hr));
                             break;
                         }
 
                         default:
-                            AssertMsgFailed(("%#x\n", fSwitchFlags));
+                            AssertMsgFailed(("flags %#x, type %d\n", fSwitchFlags, pSurface->enmD3DResType));
                     }
 
 #elif defined(VMSVGA3D_OPENGL)
@@ -471,6 +482,7 @@ static int vmsvga3dSurfaceUpdateHeapBuffers(PVMSVGA3DSTATE pState, PVMSVGA3DSURF
                             break;
                         }
 
+                        case SVGA3D_SURFACE_HINT_VERTEXBUFFER | SVGA3D_SURFACE_HINT_INDEXBUFFER:
                         case SVGA3D_SURFACE_HINT_VERTEXBUFFER:
                         case SVGA3D_SURFACE_HINT_INDEXBUFFER:
                         {
@@ -819,7 +831,7 @@ void vmsvga3dAsciiPrint(PFMVMSVGAASCIIPRINTLN pfnPrintLine, void *pvUser, void c
     /*
      * Skip stuff we can't or won't need to handle.
      */
-    if (!cx || !cy)
+    if (!cx || !cy || !cchMaxX || !cchMaxY)
         return;
     switch (enmFormat)
     {
@@ -844,7 +856,7 @@ void vmsvga3dAsciiPrint(PFMVMSVGAASCIIPRINTLN pfnPrintLine, void *pvUser, void c
     uint32_t cyPerChar = cy / cchMaxY + 1;
     /** @todo try keep aspect...   */
     uint32_t const cchLine = (cx + cxPerChar - 1) / cxPerChar;
-    uint32_t const cbSrcPixel = vmsvga3dSurfaceFormatSize(enmFormat);
+    uint32_t const cbSrcPixel = vmsvga3dSurfaceFormatSize(enmFormat, NULL, NULL);
 
     /*
      * The very simple conversion we're doing in this function is based on
@@ -1035,7 +1047,7 @@ void vmsvga3dAsciiPrint(PFMVMSVGAASCIIPRINTLN pfnPrintLine, void *pvUser, void c
                     AssertMsgFailed(("%s is not implemented\n", vmsvgaLookupEnum((int)enmFormat, &g_SVGA3dSurfaceFormat2String)));
                     fHitFormatAssert = true;
                 }
-                /* fall thru */
+                RT_FALL_THRU();
             default:
                 /* Lazy programmer fallbacks. */
                 if (cbSrcPixel == 4)
@@ -1269,7 +1281,7 @@ char *vmsvga3dFormatRenderState(char *pszBuffer, size_t cbBuffer, SVGA3dRenderSt
                                 uValue.u <= SVGA3D_WRAPCOORD_ALL ? " (out of bounds" : "");
                     break;
                 default:
-                    AssertFailed();
+                    AssertFailed();  RT_FALL_THRU();
                 case 'b': //SVGA3dBlendOp
                 case 'e': //SVGA3dBlendEquation
                 case 'p': //SVGA3dCmpFunc
@@ -1534,17 +1546,15 @@ static void vmsvga3dInfoContextWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DCONTEXT pC
 
 #else
     pHlp->pfnPrintf(pHlp, "window:                  %p\n", pContext->window);
-    pHlp->pfnPrintf(pHlp, "fMapped:                 %RTbool\n", pContext->fMapped);
     if (pContext->window)
         vmsvga3dInfoHostWindow(pHlp, (uintptr_t)pContext->window);
     pHlp->pfnPrintf(pHlp, "glxContext:              %p\n", pContext->glxContext);
 
 #endif
-    pHlp->pfnPrintf(pHlp, "sidRenderTarget:         %#x\n", pContext->sidRenderTarget);
 
-    for (uint32_t i = 0; i < RT_ELEMENTS(pContext->aSidActiveTexture); i++)
-        if (pContext->aSidActiveTexture[i] != SVGA3D_INVALID_ID)
-            pHlp->pfnPrintf(pHlp, "aSidActiveTexture[%u]:    %#x\n", i, pContext->aSidActiveTexture[i]);
+    for (uint32_t i = 0; i < RT_ELEMENTS(pContext->aSidActiveTextures); i++)
+        if (pContext->aSidActiveTextures[i] != SVGA3D_INVALID_ID)
+            pHlp->pfnPrintf(pHlp, "aSidActiveTextures[%u]:    %#x\n", i, pContext->aSidActiveTextures[i]);
 
     pHlp->pfnPrintf(pHlp, "fUpdateFlags:            %#x\n", pContext->state.u32UpdateFlags);
 
@@ -1553,11 +1563,11 @@ static void vmsvga3dInfoContextWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DCONTEXT pC
             pHlp->pfnPrintf(pHlp, "aRenderState[%3d]: %s\n", i,
                             vmsvga3dFormatRenderState(szTmp, sizeof(szTmp), &pContext->state.aRenderState[i]));
 
-    for (uint32_t i = 0; i < RT_ELEMENTS(pContext->state.aTextureState); i++)
-        for (uint32_t j = 0; j < RT_ELEMENTS(pContext->state.aTextureState[i]); j++)
-            if (pContext->state.aTextureState[i][j].name != SVGA3D_TS_INVALID)
-                pHlp->pfnPrintf(pHlp, "aTextureState[%3d][%3d]: %s\n", i, j,
-                                vmsvga3dFormatTextureState(szTmp, sizeof(szTmp), &pContext->state.aTextureState[i][j]));
+    for (uint32_t i = 0; i < RT_ELEMENTS(pContext->state.aTextureStates); i++)
+        for (uint32_t j = 0; j < RT_ELEMENTS(pContext->state.aTextureStates[i]); j++)
+            if (pContext->state.aTextureStates[i][j].name != SVGA3D_TS_INVALID)
+                pHlp->pfnPrintf(pHlp, "aTextureStates[%3d][%3d]: %s\n", i, j,
+                                vmsvga3dFormatTextureState(szTmp, sizeof(szTmp), &pContext->state.aTextureStates[i][j]));
 
     AssertCompile(RT_ELEMENTS(g_apszTransformTypes) == SVGA3D_TRANSFORM_MAX);
     for (uint32_t i = 0; i < RT_ELEMENTS(pContext->state.aTransformState); i++)
@@ -1601,6 +1611,7 @@ static void vmsvga3dInfoContextWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DCONTEXT pC
     for (uint32_t i = 0; i < RT_ELEMENTS(pContext->state.aClipPlane); i++)
         if (pContext->state.aClipPlane[i].fValid)
             pHlp->pfnPrintf(pHlp, "aClipPlane[%#04x]: [ " FLOAT_FMT_STR ", " FLOAT_FMT_STR ", " FLOAT_FMT_STR ", " FLOAT_FMT_STR " ]\n",
+                            i,
                             FLOAT_FMT_ARGS(pContext->state.aClipPlane[i].plane[0]),
                             FLOAT_FMT_ARGS(pContext->state.aClipPlane[i].plane[1]),
                             FLOAT_FMT_ARGS(pContext->state.aClipPlane[i].plane[2]),
@@ -1681,8 +1692,8 @@ static void vmsvga3dInfoContextWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DCONTEXT pC
                 if (paConsts[i].ctype == SVGA3D_CONST_TYPE_FLOAT)
                     pHlp->pfnPrintf(pHlp, "%s[%#x(%u)] = [" FLOAT_FMT_STR ", " FLOAT_FMT_STR ", " FLOAT_FMT_STR ", " FLOAT_FMT_STR "] ctype=FLOAT\n",
                                     pszName, i, i,
-                                    FLOAT_FMT_ARGS(paConsts[i].value[0]), FLOAT_FMT_ARGS(paConsts[i].value[1]),
-                                    FLOAT_FMT_ARGS(paConsts[i].value[2]), FLOAT_FMT_ARGS(paConsts[i].value[3]));
+                                    FLOAT_FMT_ARGS(*(float *)&paConsts[i].value[0]), FLOAT_FMT_ARGS(*(float *)&paConsts[i].value[1]),
+                                    FLOAT_FMT_ARGS(*(float *)&paConsts[i].value[2]), FLOAT_FMT_ARGS(*(float *)&paConsts[i].value[3]));
                 else
                     pHlp->pfnPrintf(pHlp, "%s[%#x(%u)] = [%#x, %#x, %#x, %#x] ctype=%s\n",
                                     pszName, i, i,
@@ -1800,6 +1811,174 @@ static DECLCALLBACK(int) vmsvga3dInfoSharedObjectCallback(PAVLU32NODECORE pNode,
 #endif /* VMSVGA3D_DIRECT3D */
 
 
+static int vmsvga3dInfoBmpWrite(const char *pszFilename, const void *pvBits, int w, int h, uint32_t cbPixel, uint32_t u32Mask)
+{
+#ifdef RT_OS_WINDOWS
+    if (   cbPixel != 4
+        && cbPixel != 2
+        && cbPixel != 1)
+        return VERR_NOT_SUPPORTED;
+
+    /* Always write BGRX bitmap for now. */
+    const int cbBitmap = w * h * 4;
+
+    FILE *f = fopen(pszFilename, "wb");
+    if (!f)
+        return VERR_FILE_NOT_FOUND;
+
+    if (cbPixel == 4)
+    {
+        BITMAPV4HEADER bh;
+        RT_ZERO(bh);
+        bh.bV4Size          = sizeof(bh);
+        bh.bV4Width         = w;
+        bh.bV4Height        = -h;
+        bh.bV4Planes        = 1;
+        bh.bV4BitCount      = 32;
+        bh.bV4V4Compression = BI_BITFIELDS;
+        bh.bV4SizeImage     = cbBitmap;
+        bh.bV4XPelsPerMeter = 2835;
+        bh.bV4YPelsPerMeter = 2835;
+        // bh.bV4ClrUsed       = 0;
+        // bh.bV4ClrImportant  = 0;
+        bh.bV4RedMask       = 0x00ff0000;
+        bh.bV4GreenMask     = 0x0000ff00;
+        bh.bV4BlueMask      = 0x000000ff;
+        bh.bV4AlphaMask     = 0xff000000;
+        bh.bV4CSType        = LCS_WINDOWS_COLOR_SPACE;
+        // bh.bV4Endpoints     = {0};
+        // bh.bV4GammaRed      = 0;
+        // bh.bV4GammaGreen    = 0;
+        // bh.bV4GammaBlue     = 0;
+
+        BITMAPFILEHEADER bf;
+        bf.bfType = 'MB';
+        bf.bfSize = sizeof(bf) + sizeof(bh) + cbBitmap;
+        bf.bfReserved1 = 0;
+        bf.bfReserved2 = 0;
+        bf.bfOffBits = sizeof(bf) + sizeof(bh);
+
+        fwrite(&bf, 1, sizeof(bf), f);
+        fwrite(&bh, 1, sizeof(bh), f);
+    }
+    else
+    {
+        BITMAPFILEHEADER bf;
+        bf.bfType = 'MB';
+        bf.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + cbBitmap;
+        bf.bfReserved1 = 0;
+        bf.bfReserved2 = 0;
+        bf.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
+
+        BITMAPINFOHEADER bi;
+        bi.biSize = sizeof(bi);
+        bi.biWidth = w;
+        bi.biHeight = -h;
+        bi.biPlanes = 1;
+        bi.biBitCount = 32;
+        bi.biCompression = 0;
+        bi.biSizeImage = cbBitmap;
+        bi.biXPelsPerMeter = 0;
+        bi.biYPelsPerMeter = 0;
+        bi.biClrUsed = 0;
+        bi.biClrImportant = 0;
+
+        fwrite(&bf, 1, sizeof(bf), f);
+        fwrite(&bi, 1, sizeof(bi), f);
+    }
+
+    if (cbPixel == 4)
+    {
+        const uint32_t *s = (uint32_t *)pvBits;
+        int i;
+        for (i = 0; i < w * h; ++i)
+        {
+            const uint32_t u32 = *s++;
+            uint32_t u = u32 & u32Mask;
+            fwrite(&u, 1, 4, f);
+        }
+    }
+    else if (cbPixel == 2)
+    {
+        const uint16_t *s = (uint16_t *)pvBits;
+        int i;
+        for (i = 0; i < w * h; ++i)
+        {
+            const uint16_t u16 = *s++;
+            uint32_t u32 = u16;
+            uint32_t u = u32 & u32Mask;
+            fwrite(&u, 1, 4, f);
+        }
+    }
+    else if (cbPixel == 1)
+    {
+        const uint8_t *s = (uint8_t *)pvBits;
+        int i;
+        for (i = 0; i < w * h; ++i)
+        {
+            const uint8_t u8 = *s++;
+            uint32_t u32 = u8 * 0x10000 + u8 * 0x100 + u8;
+            uint32_t u = u32 & u32Mask;
+            fwrite(&u, 1, 4, f);
+        }
+    }
+
+    fclose(f);
+
+    return VINF_SUCCESS;
+#else /* !RT_OS_WINDOWS */
+    RT_NOREF6(pszFilename, pvBits, w, h, cbPixel, u32Mask);
+    return VERR_NOT_SUPPORTED;
+#endif
+}
+
+void vmsvga3dInfoSurfaceToBitmap(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pSurface,
+                                 const char *pszPath, const char *pszNamePrefix, const char *pszNameSuffix)
+{
+    static volatile uint32_t sSeq = 0;
+
+    if (!pSurface->pMipmapLevels[0].pSurfaceData)
+        return;
+
+    const uint32_t u32Seq = ASMAtomicIncU32(&sSeq);
+
+    char szFilepath[4096];
+    RTStrPrintf(szFilepath, sizeof(szFilepath),
+                "%s\\%s-%u-sid%u%s.bmp",
+                pszPath, pszNamePrefix, u32Seq, pSurface->id, pszNameSuffix);
+
+    const uint32_t cbPixel = vmsvga3dSurfaceFormatSize(pSurface->format, NULL, NULL);
+    int rc = vmsvga3dInfoBmpWrite(szFilepath,
+                                  pSurface->pMipmapLevels[0].pSurfaceData,
+                                  pSurface->pMipmapLevels[0].mipmapSize.width,
+                                  pSurface->pMipmapLevels[0].mipmapSize.height,
+                                  cbPixel, 0xFFFFFFFF);
+    if (RT_SUCCESS(rc))
+    {
+        Log(("Bitmap: %s\n", szFilepath));
+        if (pHlp)
+            pHlp->pfnPrintf(pHlp, "Bitmap: %s\n", szFilepath);
+    }
+    else
+    {
+        Log(("Bitmap: %s %Rrc\n", szFilepath, rc));
+        if (pHlp)
+            pHlp->pfnPrintf(pHlp, "Bitmap: %s %Rrc\n", szFilepath, rc);
+    }
+
+#if 0
+    /* Alpha channel alone. */
+    RTStrPrintf(szFilepath, sizeof(szFilepath),
+                "%s\\%s-%u-sid%u%s-a.bmp",
+                pszPath, pszNamePrefix, u32Seq, pSurface->id, pszNameSuffix);
+    vmsvga3dInfoBmpWrite(szFilepath,
+                         pSurface->pMipmapLevels[0].pSurfaceData,
+                         pSurface->pMipmapLevels[0].mipmapSize.width,
+                         pSurface->pMipmapLevels[0].mipmapSize.height,
+                         cbPixel, 0xFF000000);
+#endif
+}
+
 static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pSurface,
                                          bool fVerbose, uint32_t cxAscii, bool fInvY)
 {
@@ -1813,8 +1992,8 @@ static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pS
 #endif
     pHlp->pfnPrintf(pHlp, "Format:                  %s\n",
                     vmsvgaFormatEnumValueEx(szTmp, sizeof(szTmp), NULL, (int)pSurface->format, false, &g_SVGA3dSurfaceFormat2String));
-    pHlp->pfnPrintf(pHlp, "Flags:                   %#x", pSurface->flags);
-    vmsvga3dInfoU32Flags(pHlp, pSurface->flags, "SVGA3D_SURFACE_", g_aSvga3DSurfaceFlags, RT_ELEMENTS(g_aSvga3DSurfaceFlags));
+    pHlp->pfnPrintf(pHlp, "Flags:                   %#x", pSurface->surfaceFlags);
+    vmsvga3dInfoU32Flags(pHlp, pSurface->surfaceFlags, "SVGA3D_SURFACE_", g_aSvga3DSurfaceFlags, RT_ELEMENTS(g_aSvga3DSurfaceFlags));
     pHlp->pfnPrintf(pHlp, "\n");
     if (pSurface->cFaces == 0)
         pHlp->pfnPrintf(pHlp, "Faces:                   %u\n", pSurface->cFaces);
@@ -1829,9 +2008,9 @@ static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pS
         {
             pHlp->pfnPrintf(pHlp, "Face #%u, mipmap #%u[%u]:%s  cx=%u, cy=%u, cz=%u, cbSurface=%#x, cbPitch=%#x",
                             iFace, iLevel, iMipmap, iMipmap < 10 ? " " : "",
-                            pSurface->pMipmapLevels[iMipmap].size.width,
-                            pSurface->pMipmapLevels[iMipmap].size.height,
-                            pSurface->pMipmapLevels[iMipmap].size.depth,
+                            pSurface->pMipmapLevels[iMipmap].mipmapSize.width,
+                            pSurface->pMipmapLevels[iMipmap].mipmapSize.height,
+                            pSurface->pMipmapLevels[iMipmap].mipmapSize.depth,
                             pSurface->pMipmapLevels[iMipmap].cbSurface,
                             pSurface->pMipmapLevels[iMipmap].cbSurfacePitch);
             if (pSurface->pMipmapLevels[iMipmap].pSurfaceData)
@@ -1888,14 +2067,14 @@ static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pS
                     {
                         pHlp->pfnPrintf(pHlp, "--- Face #%u, mipmap #%u[%u]: cx=%u, cy=%u, cz=%u ---\n",
                                         iFace, iLevel, iMipmap,
-                                        pSurface->pMipmapLevels[iMipmap].size.width,
-                                        pSurface->pMipmapLevels[iMipmap].size.height,
-                                        pSurface->pMipmapLevels[iMipmap].size.depth);
+                                        pSurface->pMipmapLevels[iMipmap].mipmapSize.width,
+                                        pSurface->pMipmapLevels[iMipmap].mipmapSize.height,
+                                        pSurface->pMipmapLevels[iMipmap].mipmapSize.depth);
                         vmsvga3dAsciiPrint(vmsvga3dAsciiPrintlnInfo, (void *)pHlp,
                                            pSurface->pMipmapLevels[iMipmap].pSurfaceData,
                                            pSurface->pMipmapLevels[iMipmap].cbSurface,
-                                           pSurface->pMipmapLevels[iMipmap].size.width,
-                                           pSurface->pMipmapLevels[iMipmap].size.height,
+                                           pSurface->pMipmapLevels[iMipmap].mipmapSize.width,
+                                           pSurface->pMipmapLevels[iMipmap].mipmapSize.height,
                                            pSurface->pMipmapLevels[iMipmap].cbSurfacePitch,
                                            pSurface->format,
                                            fInvY,
@@ -1906,7 +2085,7 @@ static void vmsvga3dInfoSurfaceWorkerOne(PCDBGFINFOHLP pHlp, PVMSVGA3DSURFACE pS
 }
 
 
-void vmsvga3dInfoSurfaceWorker(PVGASTATE pThis, PCDBGFINFOHLP pHlp, uint32_t sid, bool fVerbose, uint32_t cxAscii, bool fInvY)
+void vmsvga3dInfoSurfaceWorker(PVGASTATE pThis, PCDBGFINFOHLP pHlp, uint32_t sid, bool fVerbose, uint32_t cxAscii, bool fInvY, const char *pszBitmapPath)
 {
     /* Warning! This code is currently racing papSurfaces reallocation! */
     /* Warning! This code is currently racing papSurfaces reallocation! */
@@ -1927,6 +2106,8 @@ void vmsvga3dInfoSurfaceWorker(PVGASTATE pThis, PCDBGFINFOHLP pHlp, uint32_t sid
                     if (fVerbose)
                         vmsvga3dSurfaceUpdateHeapBuffersOnFifoThread(pThis, sid);
                     vmsvga3dInfoSurfaceWorkerOne(pHlp, pSurface, fVerbose, cxAscii, fInvY);
+                    if (pszBitmapPath && *pszBitmapPath)
+                        vmsvga3dInfoSurfaceToBitmap(pHlp, pSurface, pszBitmapPath, "info", "");
                     return;
                 }
             }
