@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -35,10 +35,34 @@
 *********************************************************************************************************************************/
 #define VBOX_DEVICE_STRUCT_TESTCASE
 #define VBOX_WITH_HGCM                  /* grumble */
+
+/* Check that important preprocessor macros does not get redefined: */
+#include <VBox/cdefs.h>
+#include <VBox/log.h>
+#ifdef DEBUG
+# define VBOX_DEVICE_STRUCT_TESTCASE_CHECK_DEBUG
+#else
+# undef  VBOX_DEVICE_STRUCT_TESTCASE_CHECK_DEBUG
+#endif
+#ifdef LOG_ENABLED
+# define VBOX_DEVICE_STRUCT_TESTCASE_CHECK_LOG_ENABLED
+#else
+# undef  VBOX_DEVICE_STRUCT_TESTCASE_CHECK_LOG_ENABLED
+#endif
+#ifdef VBOX_STRICT
+# define VBOX_DEVICE_STRUCT_TESTCASE_CHECK_VBOX_STRICT
+#else
+# undef  VBOX_DEVICE_STRUCT_TESTCASE_CHECK_VBOX_STRICT
+#endif
+#ifdef RT_STRICT
+# define VBOX_DEVICE_STRUCT_TESTCASE_CHECK_RT_STRICT
+#else
+# undef  VBOX_DEVICE_STRUCT_TESTCASE_CHECK_RT_STRICT
+#endif
+
+/* The structures we're checking: */
 #undef LOG_GROUP
-#include "../Bus/DevPCI.cpp" /* must be first! */
-#undef LOG_GROUP
-#include "../Bus/DevPciIch9.cpp"
+#include "../Bus/DevPciInternal.h"
 #undef LOG_GROUP
 #include "../EFI/DevSmc.cpp"
 #undef LOG_GROUP
@@ -60,17 +84,11 @@
 #undef LOG_GROUP
 #include "../PC/DevRTC.cpp"
 # undef LOG_GROUP
-#ifdef VBOX_WITH_NEW_APIC
 # include "../../VMM/VMMR3/APIC.cpp"
-#else
-# include "../PC/DevAPIC.cpp"
-#endif
 #undef LOG_GROUP
-#ifdef VBOX_WITH_NEW_IOAPIC
-# include "../PC/DevIoApic.cpp"
-#else
-# include "../PC/DevIoApic_Old.cpp"
-#endif
+#include "../PC/DevIoApic.cpp"
+#undef LOG_GROUP
+#include "../PC/DevDMA.cpp"
 #undef LOG_GROUP
 #include "../Storage/DevATA.cpp"
 #ifdef VBOX_WITH_USB
@@ -91,6 +109,10 @@
 #include "../Parallel/DevParallel.cpp"
 #undef LOG_GROUP
 #include "../Serial/DevSerial.cpp"
+#undef LOG_GROUP
+#include "../Serial/DevOxPcie958.cpp"
+#undef LOG_GROUP
+#include "../Serial/UartCore.h"
 #ifdef VBOX_WITH_AHCI
 # undef LOG_GROUP
 # include "../Storage/DevAHCI.cpp"
@@ -113,15 +135,30 @@
 #endif
 #undef LOG_GROUP
 #include "../PC/DevHPET.cpp"
-# undef LOG_GROUP
-# include "../Audio/DevIchAc97.cpp"
-# undef LOG_GROUP
-# include "../Audio/DevHDA.cpp"
+#undef LOG_GROUP
+#include "../Audio/DevIchAc97.cpp"
+#undef LOG_GROUP
+#include "../Audio/DevHDA.cpp"
 #ifdef VBOX_WITH_NVME_IMPL
 # undef LOG_GROUP
 # include "../Storage/DevNVMe.cpp"
 #endif
 
+/* Check that important preprocessor macros didn't get redefined: */
+#if defined(DEBUG)       != defined(VBOX_DEVICE_STRUCT_TESTCASE_CHECK_DEBUG)
+# error "DEBUG was modified!  This may throw off structure tests."
+#endif
+#if defined(LOG_ENABLED) != defined(VBOX_DEVICE_STRUCT_TESTCASE_CHECK_LOG_ENABLED)
+# error "LOG_ENABLED was modified!  This may throw off structure tests."
+#endif
+#if defined(RT_STRICT)   != defined(VBOX_DEVICE_STRUCT_TESTCASE_CHECK_RT_STRICT)
+# error "RT_STRICT was modified!  This may throw off structure tests."
+#endif
+#if defined(VBOX_STRICT) != defined(VBOX_DEVICE_STRUCT_TESTCASE_CHECK_VBOX_STRICT)
+# error "VBOX_STRICT was modified!  This may throw off structure tests."
+#endif
+
+
 /* we don't use iprt here because we're pretending to be in GC! */
 #include <stdio.h>
 
@@ -146,14 +183,14 @@ int main()
     GEN_CHECK_OFF(PDMDEVINS, pvInstanceDataRC);
     GEN_CHECK_OFF(PDMDEVINS, achInstanceData);
 
-    /* DevPCI.cpp */
+    /* PDMPCIDEV */
     GEN_CHECK_SIZE(PDMPCIDEV);
     GEN_CHECK_SIZE(PDMPCIDEVINT);
-    GEN_CHECK_SIZE(PCIIOREGION);
+    GEN_CHECK_SIZE(PCIIOREGION); /** @todo fix name of PCIIOREGION */
     GEN_CHECK_OFF(PDMPCIDEV, abConfig);
     GEN_CHECK_OFF(PDMPCIDEV, uDevFn);
     GEN_CHECK_OFF(PDMPCIDEV, pszNameR3);
-    GEN_CHECK_OFF(PDMPCIDEV, pvReserved);
+    GEN_CHECK_OFF(PDMPCIDEV, pfnRegionLoadChangeHookR3);
     GEN_CHECK_OFF(PDMPCIDEV, Int);
     GEN_CHECK_OFF(PDMPCIDEV, Int.s.aIORegions);
     GEN_CHECK_OFF(PDMPCIDEV, Int.s.aIORegions[1]);
@@ -172,66 +209,43 @@ int main()
     GEN_CHECK_OFF(PDMPCIDEV, Int.s.pfnBridgeConfigRead);
     GEN_CHECK_OFF(PDMPCIDEV, Int.s.pfnBridgeConfigWrite);
     GEN_CHECK_PADDING(PDMPCIDEV, Int, 8);
-    GEN_CHECK_SIZE(PIIX3State);
-    GEN_CHECK_SIZE(PCIBUS);
-    GEN_CHECK_OFF(PCIBUS, iBus);
-    GEN_CHECK_OFF(PCIBUS, iDevSearch);
-    GEN_CHECK_OFF(PCIBUS, cBridges);
-    GEN_CHECK_OFF(PCIBUS, devices);
-    GEN_CHECK_OFF(PCIBUS, devices[1]);
-    GEN_CHECK_OFF(PCIBUS, pDevInsR3);
-    GEN_CHECK_OFF(PCIBUS, pPciHlpR3);
-    GEN_CHECK_OFF(PCIBUS, papBridgesR3);
-    GEN_CHECK_OFF(PCIBUS, pDevInsR0);
-    GEN_CHECK_OFF(PCIBUS, pPciHlpR0);
-    GEN_CHECK_OFF(PCIBUS, pDevInsRC);
-    GEN_CHECK_OFF(PCIBUS, pPciHlpRC);
-    GEN_CHECK_OFF(PCIBUS, PciDev);
-    GEN_CHECK_SIZE(PCIGLOBALS);
-    GEN_CHECK_OFF(PCIGLOBALS, pci_bios_io_addr);
-    GEN_CHECK_OFF(PCIGLOBALS, pci_bios_mem_addr);
-    GEN_CHECK_OFF(PCIGLOBALS, pci_irq_levels);
-    GEN_CHECK_OFF(PCIGLOBALS, pci_irq_levels[1]);
-    GEN_CHECK_OFF(PCIGLOBALS, fUseIoApic);
-    GEN_CHECK_OFF(PCIGLOBALS, pci_apic_irq_levels);
-    GEN_CHECK_OFF(PCIGLOBALS, pci_apic_irq_levels[1]);
-    GEN_CHECK_OFF(PCIGLOBALS, acpi_irq_level);
-    GEN_CHECK_OFF(PCIGLOBALS, acpi_irq);
-    GEN_CHECK_OFF(PCIGLOBALS, uConfigReg);
-    GEN_CHECK_OFF(PCIGLOBALS, pDevInsR3);
-    GEN_CHECK_OFF(PCIGLOBALS, pDevInsR0);
-    GEN_CHECK_OFF(PCIGLOBALS, pDevInsRC);
-    GEN_CHECK_OFF(PCIGLOBALS, PIIX3State);
-    GEN_CHECK_OFF(PCIGLOBALS, PciBus);
 
-    /* DevPciIch9.cpp */
-    GEN_CHECK_SIZE(ICH9PCIBUS);
-    GEN_CHECK_OFF(ICH9PCIBUS, iBus);
-    GEN_CHECK_OFF(ICH9PCIBUS, cBridges);
-    GEN_CHECK_OFF(ICH9PCIBUS, apDevices);
-    GEN_CHECK_OFF(ICH9PCIBUS, apDevices[1]);
-    GEN_CHECK_OFF(ICH9PCIBUS, pDevInsR3);
-    GEN_CHECK_OFF(ICH9PCIBUS, pPciHlpR3);
-    GEN_CHECK_OFF(ICH9PCIBUS, papBridgesR3);
-    GEN_CHECK_OFF(ICH9PCIBUS, pDevInsR0);
-    GEN_CHECK_OFF(ICH9PCIBUS, pPciHlpR0);
-    GEN_CHECK_OFF(ICH9PCIBUS, pDevInsRC);
-    GEN_CHECK_OFF(ICH9PCIBUS, pPciHlpRC);
-    GEN_CHECK_OFF(ICH9PCIBUS, aPciDev);
-    GEN_CHECK_SIZE(ICH9PCIGLOBALS);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, pDevInsR3);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, pDevInsR0);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, pDevInsRC);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uConfigReg);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uaPciApicIrqLevels);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uaPciApicIrqLevels[1]);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uPciBiosIo);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uPciBiosMmio);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uPciBiosMmio64);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, uBus);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, u64PciConfigMMioAddress);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, u64PciConfigMMioLength);
-    GEN_CHECK_OFF(ICH9PCIGLOBALS, aPciBus);
+    /* DevPciInternal.h */
+    GEN_CHECK_SIZE(DEVPCIBUS);
+    GEN_CHECK_OFF(DEVPCIBUS, iBus);
+    GEN_CHECK_OFF(DEVPCIBUS, iDevSearch);
+    GEN_CHECK_OFF(DEVPCIBUS, cBridges);
+    GEN_CHECK_OFF(DEVPCIBUS, apDevices);
+    GEN_CHECK_OFF(DEVPCIBUS, apDevices[1]);
+    GEN_CHECK_OFF(DEVPCIBUS, pDevInsR3);
+    GEN_CHECK_OFF(DEVPCIBUS, pPciHlpR3);
+    GEN_CHECK_OFF(DEVPCIBUS, papBridgesR3);
+    GEN_CHECK_OFF(DEVPCIBUS, pDevInsR0);
+    GEN_CHECK_OFF(DEVPCIBUS, pPciHlpR0);
+    GEN_CHECK_OFF(DEVPCIBUS, pDevInsRC);
+    GEN_CHECK_OFF(DEVPCIBUS, pPciHlpRC);
+    GEN_CHECK_OFF(DEVPCIBUS, PciDev);
+    GEN_CHECK_SIZE(PIIX3ISABRIDGE);
+    GEN_CHECK_SIZE(DEVPCIROOT);
+    GEN_CHECK_OFF(DEVPCIROOT, PciBus);
+    GEN_CHECK_OFF(DEVPCIROOT, pDevInsR3);
+    GEN_CHECK_OFF(DEVPCIROOT, pDevInsR0);
+    GEN_CHECK_OFF(DEVPCIROOT, pDevInsRC);
+    GEN_CHECK_OFF(DEVPCIROOT, fUseIoApic);
+    GEN_CHECK_OFF(DEVPCIROOT, u64PciConfigMMioAddress);
+    GEN_CHECK_OFF(DEVPCIROOT, u64PciConfigMMioLength);
+    GEN_CHECK_OFF(DEVPCIROOT, auPciApicIrqLevels);
+    GEN_CHECK_OFF(DEVPCIROOT, auPciApicIrqLevels[1]);
+    GEN_CHECK_OFF(DEVPCIROOT, uConfigReg);
+    GEN_CHECK_OFF(DEVPCIROOT, Piix3.iAcpiIrqLevel);
+    GEN_CHECK_OFF(DEVPCIROOT, Piix3.iAcpiIrq);
+    GEN_CHECK_OFF(DEVPCIROOT, Piix3.auPciLegacyIrqLevels);
+    GEN_CHECK_OFF(DEVPCIROOT, Piix3.auPciLegacyIrqLevels[1]);
+    GEN_CHECK_OFF(DEVPCIROOT, Piix3.PIIX3State);
+    GEN_CHECK_OFF(DEVPCIROOT, uPciBiosBus);
+    GEN_CHECK_OFF(DEVPCIROOT, uPciBiosIo);
+    GEN_CHECK_OFF(DEVPCIROOT, uPciBiosMmio);
+    GEN_CHECK_OFF(DEVPCIROOT, uPciBiosMmio64);
 
     /* EFI/DevSMC.cpp */
     GEN_CHECK_SIZE(DEVSMC);
@@ -343,20 +357,46 @@ int main()
     GEN_CHECK_OFF(VGASTATE, RefreshTimer);
     GEN_CHECK_OFF(VGASTATE, pDevInsR0);
 #ifdef VBOX_WITH_VMSVGA
-    GEN_CHECK_OFF(VGASTATE, svga.u64HostWindowId);
     GEN_CHECK_OFF(VGASTATE, svga.pFIFOR3);
     GEN_CHECK_OFF(VGASTATE, svga.pFIFOR0);
     GEN_CHECK_OFF(VGASTATE, svga.pSvgaR3State);
     GEN_CHECK_OFF(VGASTATE, svga.p3dState);
-    GEN_CHECK_OFF(VGASTATE, svga.pFrameBufferBackup);
+    GEN_CHECK_OFF(VGASTATE, svga.pbVgaFrameBufferR3);
     GEN_CHECK_OFF(VGASTATE, svga.GCPhysFIFO);
     GEN_CHECK_OFF(VGASTATE, svga.cbFIFO);
+    GEN_CHECK_OFF(VGASTATE, svga.cbFIFOConfig);
+    GEN_CHECK_OFF(VGASTATE, svga.u32SVGAId);
+    GEN_CHECK_OFF(VGASTATE, svga.fConfigured);
+    GEN_CHECK_OFF(VGASTATE, svga.fBusy);
+    GEN_CHECK_OFF(VGASTATE, svga.fTraces);
+    GEN_CHECK_OFF(VGASTATE, svga.u32GuestId);
+    GEN_CHECK_OFF(VGASTATE, svga.cScratchRegion);
+    GEN_CHECK_OFF(VGASTATE, svga.u32IrqStatus);
+    GEN_CHECK_OFF(VGASTATE, svga.u32IrqMask);
+    GEN_CHECK_OFF(VGASTATE, svga.u32PitchLock);
+    GEN_CHECK_OFF(VGASTATE, svga.u32CurrentGMRId);
+    GEN_CHECK_OFF(VGASTATE, svga.u32RegCaps);
     GEN_CHECK_OFF(VGASTATE, svga.BasePort);
+    GEN_CHECK_OFF(VGASTATE, svga.u32IndexReg);
+    GEN_CHECK_OFF(VGASTATE, svga.pSupDrvSession);
+    GEN_CHECK_OFF(VGASTATE, svga.FIFORequestSem);
+    GEN_CHECK_OFF(VGASTATE, svga.FIFOExtCmdSem);
     GEN_CHECK_OFF(VGASTATE, svga.pFIFOIOThread);
     GEN_CHECK_OFF(VGASTATE, svga.uWidth);
+    GEN_CHECK_OFF(VGASTATE, svga.uHeight);
+    GEN_CHECK_OFF(VGASTATE, svga.uBpp);
+    GEN_CHECK_OFF(VGASTATE, svga.cbScanline);
+    GEN_CHECK_OFF(VGASTATE, svga.u32MaxWidth);
+    GEN_CHECK_OFF(VGASTATE, svga.u32MaxHeight);
+    GEN_CHECK_OFF(VGASTATE, svga.viewport);
     GEN_CHECK_OFF(VGASTATE, svga.u32ActionFlags);
     GEN_CHECK_OFF(VGASTATE, svga.f3DEnabled);
     GEN_CHECK_OFF(VGASTATE, svga.fVRAMTracking);
+    GEN_CHECK_OFF(VGASTATE, svga.u8FIFOExtCommand);
+    GEN_CHECK_OFF(VGASTATE, svga.fFifoExtCommandWakeup);
+    GEN_CHECK_OFF(VGASTATE, svga.au32ScratchRegion);
+    GEN_CHECK_OFF(VGASTATE, svga.StatRegBitsPerPixelWr);
+    GEN_CHECK_OFF(VGASTATE, svga.StatRegWriteOnlyRd);
 #endif
     GEN_CHECK_OFF(VGASTATE, cMonitors);
     GEN_CHECK_OFF(VGASTATE, cMilliesRefreshInterval);
@@ -437,17 +477,21 @@ int main()
     GEN_CHECK_OFF(PS2K, fScanning);
     GEN_CHECK_OFF(PS2K, fNumLockOn);
     GEN_CHECK_OFF(PS2K, u8ScanSet);
-    GEN_CHECK_OFF(PS2K, u8Typematic);
+    GEN_CHECK_OFF(PS2K, u8TypematicCfg);
     GEN_CHECK_OFF(PS2K, enmTypematicState);
     GEN_CHECK_OFF(PS2K, keyQ);
     GEN_CHECK_OFF(PS2K, cmdQ);
     GEN_CHECK_OFF(PS2K, uTypematicDelay);
+    GEN_CHECK_OFF(PS2K, fThrottleActive);
     GEN_CHECK_OFF(PS2K, pKbdDelayTimerRC);
     GEN_CHECK_OFF(PS2K, pKbdDelayTimerR3);
     GEN_CHECK_OFF(PS2K, pKbdDelayTimerR0);
     GEN_CHECK_OFF(PS2K, pKbdTypematicTimerRC);
     GEN_CHECK_OFF(PS2K, pKbdTypematicTimerR3);
     GEN_CHECK_OFF(PS2K, pKbdTypematicTimerR0);
+    GEN_CHECK_OFF(PS2K, pThrottleTimerRC);
+    GEN_CHECK_OFF(PS2K, pThrottleTimerR3);
+    GEN_CHECK_OFF(PS2K, pThrottleTimerR0);
     GEN_CHECK_OFF(PS2K, pCritSectR3);
     GEN_CHECK_OFF(PS2K, Keyboard.IBase);
     GEN_CHECK_OFF(PS2K, Keyboard.IPort);
@@ -550,8 +594,7 @@ int main()
 #endif
     GEN_CHECK_OFF(PCNETSTATE, fGCEnabled);
     GEN_CHECK_OFF(PCNETSTATE, fR0Enabled);
-    GEN_CHECK_OFF(PCNETSTATE, fAm79C973);
-    GEN_CHECK_OFF(PCNETSTATE, u32LinkSpeed);
+    GEN_CHECK_OFF(PCNETSTATE, uDevType);
     GEN_CHECK_OFF(PCNETSTATE, StatReceiveBytes);
     GEN_CHECK_OFF(PCNETSTATE, StatTransmitBytes);
 #ifdef VBOX_WITH_STATISTICS
@@ -643,6 +686,16 @@ int main()
     GEN_CHECK_OFF(DEVPIC, StatClearedActiveSlaveIRQ);
 #endif
 
+    /* PC/DevPit-i8254.cpp */
+    GEN_CHECK_SIZE(DMAState);
+    GEN_CHECK_OFF(DMAState, DMAC);
+    GEN_CHECK_OFF(DMAState, DMAC[0].ChState[0]);
+    GEN_CHECK_OFF(DMAState, DMAC[0].ChState[1]);
+    GEN_CHECK_OFF(DMAState, DMAC[0].u8Mask);
+    GEN_CHECK_OFF(DMAState, DMAC[1].ChState[0]);
+    GEN_CHECK_OFF(DMAState, DMAC[1].ChState[1]);
+    GEN_CHECK_OFF(DMAState, DMAC[1].u8Mask);
+
     /* PC/DevPit-i8254.cpp */
     GEN_CHECK_SIZE(PITCHANNEL);
     GEN_CHECK_OFF(PITCHANNEL, pPitR3);
@@ -717,7 +770,6 @@ int main()
     GEN_CHECK_OFF(RTCSTATE, CurLogPeriod);
     GEN_CHECK_OFF(RTCSTATE, CurHintPeriod);
 
-#ifdef VBOX_WITH_NEW_APIC
     GEN_CHECK_SIZE(APIC);
     GEN_CHECK_OFF(APIC, pApicDevR0);
     GEN_CHECK_OFF(APIC, pApicDevR3);
@@ -773,69 +825,10 @@ int main()
     GEN_CHECK_OFF(APICCPU, StatMaskedByPpr);
     GEN_CHECK_OFF(APICCPU, StatTimerIcrWrite);
     GEN_CHECK_OFF(APICCPU, StatIcrLoWrite);
+    GEN_CHECK_OFF(APICCPU, StatIcrHiWrite);
+    GEN_CHECK_OFF(APICCPU, StatIcrFullWrite);
 # endif /* VBOX_WITH_STATISTICS */
-#else
-    /* PC/DevAPIC.cpp */
-    GEN_CHECK_SIZE(APICState);
-    GEN_CHECK_OFF(APICState, apicbase);
-    GEN_CHECK_OFF(APICState, id);
-    GEN_CHECK_OFF(APICState, arb_id);
-    GEN_CHECK_OFF(APICState, tpr);
-    GEN_CHECK_OFF(APICState, spurious_vec);
-    GEN_CHECK_OFF(APICState, log_dest);
-    GEN_CHECK_OFF(APICState, dest_mode);
-    GEN_CHECK_OFF(APICState, isr);
-    GEN_CHECK_OFF(APICState, isr.au32Bitmap[1]);
-    GEN_CHECK_OFF(APICState, tmr);
-    GEN_CHECK_OFF(APICState, tmr.au32Bitmap[1]);
-    GEN_CHECK_OFF(APICState, irr);
-    GEN_CHECK_OFF(APICState, irr.au32Bitmap[1]);
-    GEN_CHECK_OFF(APICState, lvt);
-    GEN_CHECK_OFF(APICState, lvt[1]);
-    GEN_CHECK_OFF(APICState, lvt[APIC_LVT_NB - 1]);
-    GEN_CHECK_OFF(APICState, esr);
-    GEN_CHECK_OFF(APICState, icr);
-    GEN_CHECK_OFF(APICState, icr[1]);
-    GEN_CHECK_OFF(APICState, divide_conf);
-    GEN_CHECK_OFF(APICState, count_shift);
-    GEN_CHECK_OFF(APICState, initial_count);
-    GEN_CHECK_OFF(APICState, initial_count_load_time);
-    GEN_CHECK_OFF(APICState, next_time);
-    GEN_CHECK_OFF(APICState, pTimerR3);
-    GEN_CHECK_OFF(APICState, pTimerR0);
-    GEN_CHECK_OFF(APICState, pTimerRC);
-    GEN_CHECK_OFF(APICState, fTimerArmed);
-    GEN_CHECK_OFF(APICState, uHintedInitialCount);
-    GEN_CHECK_OFF(APICState, uHintedCountShift);
-    GEN_CHECK_OFF(APICState, pszDesc);
-# ifdef VBOX_WITH_STATISTICS
-    GEN_CHECK_OFF(APICState, StatTimerSetInitialCount);
-    GEN_CHECK_OFF(APICState, StatTimerSetLvtNoRelevantChange);
-# endif
 
-    GEN_CHECK_SIZE(APICDeviceInfo);
-    GEN_CHECK_OFF(APICDeviceInfo, pDevInsR3);
-    GEN_CHECK_OFF(APICDeviceInfo, pApicHlpR3);
-    GEN_CHECK_OFF(APICDeviceInfo, paLapicsR3);
-    GEN_CHECK_OFF(APICDeviceInfo, pCritSectR3);
-    GEN_CHECK_OFF(APICDeviceInfo, pDevInsR0);
-    GEN_CHECK_OFF(APICDeviceInfo, pApicHlpR0);
-    GEN_CHECK_OFF(APICDeviceInfo, paLapicsR0);
-    GEN_CHECK_OFF(APICDeviceInfo, pCritSectR0);
-    GEN_CHECK_OFF(APICDeviceInfo, pDevInsRC);
-    GEN_CHECK_OFF(APICDeviceInfo, pApicHlpRC);
-    GEN_CHECK_OFF(APICDeviceInfo, paLapicsRC);
-    GEN_CHECK_OFF(APICDeviceInfo, pCritSectRC);
-    GEN_CHECK_OFF(APICDeviceInfo, enmMode);
-    GEN_CHECK_OFF(APICDeviceInfo, cTPRPatchAttempts);
-    GEN_CHECK_OFF(APICDeviceInfo, cCpus);
-# ifdef VBOX_WITH_STATISTICS
-    GEN_CHECK_OFF(APICDeviceInfo, StatMMIOReadGC);
-    GEN_CHECK_OFF(APICDeviceInfo, StatMMIOWriteHC);
-# endif
-#endif  /* VBOX_WITH_NEW_APIC */
-
-#ifdef VBOX_WITH_NEW_IOAPIC
     /* PC/DevIoApic.cpp */
     GEN_CHECK_SIZE(IOAPIC);
     GEN_CHECK_OFF(IOAPIC, pDevInsR3);
@@ -859,34 +852,14 @@ int main()
     GEN_CHECK_OFF(IOAPIC, StatSetEoiRZ);
     GEN_CHECK_OFF(IOAPIC, StatSetEoiR3);
 # endif
-#else
-    /* PC/DevIoApic_Old.cpp */
-    GEN_CHECK_SIZE(IOAPIC);
-    GEN_CHECK_OFF(IOAPIC, id);
-    GEN_CHECK_OFF(IOAPIC, ioregsel);
-    GEN_CHECK_OFF(IOAPIC, irr);
-    GEN_CHECK_OFF(IOAPIC, ioredtbl);
-    GEN_CHECK_OFF(IOAPIC, ioredtbl[1]);
-    GEN_CHECK_OFF(IOAPIC, ioredtbl[IOAPIC_NUM_PINS - 1]);
-    GEN_CHECK_OFF(IOAPIC, pDevInsR3);
-    GEN_CHECK_OFF(IOAPIC, pIoApicHlpR3);
-    GEN_CHECK_OFF(IOAPIC, pDevInsR0);
-    GEN_CHECK_OFF(IOAPIC, pIoApicHlpR0);
-    GEN_CHECK_OFF(IOAPIC, pDevInsRC);
-    GEN_CHECK_OFF(IOAPIC, pIoApicHlpRC);
-# ifdef VBOX_WITH_STATISTICS
-    GEN_CHECK_OFF(IOAPIC, StatMMIOReadGC);
-    GEN_CHECK_OFF(IOAPIC, StatSetIrqHC);
-# endif
-#endif /* VBOX_WITH_NEW_IOAPIC */
 
     /* Storage/DevATA.cpp */
     GEN_CHECK_SIZE(BMDMAState);
     GEN_CHECK_OFF(BMDMAState, u8Cmd);
     GEN_CHECK_OFF(BMDMAState, u8Status);
-    GEN_CHECK_OFF(BMDMAState, pvAddr);
+    GEN_CHECK_OFF(BMDMAState, GCPhysAddr);
     GEN_CHECK_SIZE(BMDMADesc);
-    GEN_CHECK_OFF(BMDMADesc, pBuffer);
+    GEN_CHECK_OFF(BMDMADesc, GCPhysBuffer);
     GEN_CHECK_OFF(BMDMADesc, cbBuffer);
     GEN_CHECK_SIZE(ATADevState);
     GEN_CHECK_OFF(ATADevState, fLBA48);
@@ -949,7 +922,6 @@ int main()
     GEN_CHECK_OFF(ATADevState, StatWrites);
     GEN_CHECK_OFF(ATADevState, StatBytesWritten);
     GEN_CHECK_OFF(ATADevState, StatFlushes);
-    GEN_CHECK_OFF(ATADevState, fNonRotational);
     GEN_CHECK_OFF(ATADevState, fATAPIPassthrough);
     GEN_CHECK_OFF(ATADevState, fOverwriteInquiry);
     GEN_CHECK_OFF(ATADevState, cErrors);
@@ -973,11 +945,11 @@ int main()
     GEN_CHECK_OFF(ATADevState, szModelNumber);
     GEN_CHECK_OFF(ATADevState, szModelNumber[ATA_MODEL_NUMBER_LENGTH]);
     GEN_CHECK_OFF(ATADevState, szInquiryVendorId);
-    GEN_CHECK_OFF(ATADevState, szInquiryVendorId[ATAPI_INQUIRY_VENDOR_ID_LENGTH]);
+    GEN_CHECK_OFF(ATADevState, szInquiryVendorId[SCSI_INQUIRY_VENDOR_ID_LENGTH]);
     GEN_CHECK_OFF(ATADevState, szInquiryProductId);
-    GEN_CHECK_OFF(ATADevState, szInquiryProductId[ATAPI_INQUIRY_PRODUCT_ID_LENGTH]);
+    GEN_CHECK_OFF(ATADevState, szInquiryProductId[SCSI_INQUIRY_PRODUCT_ID_LENGTH]);
     GEN_CHECK_OFF(ATADevState, szInquiryRevision);
-    GEN_CHECK_OFF(ATADevState, szInquiryRevision[ATAPI_INQUIRY_REVISION_LENGTH]);
+    GEN_CHECK_OFF(ATADevState, szInquiryRevision[SCSI_INQUIRY_REVISION_LENGTH]);
     GEN_CHECK_OFF(ATADevState, pTrackList);
     GEN_CHECK_SIZE(ATATransferRequest);
     GEN_CHECK_OFF(ATATransferRequest, iIf);
@@ -1007,9 +979,9 @@ int main()
     GEN_CHECK_OFF(ATACONTROLLER, fRedoIdle);
     GEN_CHECK_OFF(ATACONTROLLER, fRedoDMALastDesc);
     GEN_CHECK_OFF(ATACONTROLLER, BmDma);
-    GEN_CHECK_OFF(ATACONTROLLER, pFirstDMADesc);
-    GEN_CHECK_OFF(ATACONTROLLER, pLastDMADesc);
-    GEN_CHECK_OFF(ATACONTROLLER, pRedoDMABuffer);
+    GEN_CHECK_OFF(ATACONTROLLER, GCPhysFirstDMADesc);
+    GEN_CHECK_OFF(ATACONTROLLER, GCPhysLastDMADesc);
+    GEN_CHECK_OFF(ATACONTROLLER, GCPhysRedoDMABuffer);
     GEN_CHECK_OFF(ATACONTROLLER, cbRedoDMABuffer);
     GEN_CHECK_OFF(ATACONTROLLER, aIfs);
     GEN_CHECK_OFF(ATACONTROLLER, aIfs[1]);
@@ -1139,9 +1111,6 @@ int main()
     GEN_CHECK_SIZE(EHCI);
     GEN_CHECK_OFF(EHCI, PciDev);
     GEN_CHECK_OFF(EHCI, MMIOBase);
-    GEN_CHECK_OFF(EHCI, pEndOfFrameTimerR3);
-    GEN_CHECK_OFF(EHCI, pEndOfFrameTimerR0);
-    GEN_CHECK_OFF(EHCI, pEndOfFrameTimerRC);
     GEN_CHECK_OFF(EHCI, pDevInsR3);
     GEN_CHECK_OFF(EHCI, pDevInsR0);
     GEN_CHECK_OFF(EHCI, pDevInsRC);
@@ -1177,18 +1146,11 @@ int main()
     GEN_CHECK_OFF(EHCI, StatCanceledIsocUrbs);
     GEN_CHECK_OFF(EHCI, StatCanceledGenUrbs);
     GEN_CHECK_OFF(EHCI, StatDroppedUrbs);
-    GEN_CHECK_OFF(EHCI, StatTimer);
 #  endif
     GEN_CHECK_OFF(EHCI, u64TimerHz);
     GEN_CHECK_OFF(EHCI, cIdleCycles);
     GEN_CHECK_OFF(EHCI, uFrameRate);
     GEN_CHECK_OFF(EHCI, fIdle);
-    GEN_CHECK_OFF(EHCI, pEOFTimerSyncR3);
-    GEN_CHECK_OFF(EHCI, pEOFTimerSyncR0);
-    GEN_CHECK_OFF(EHCI, pEOFTimerSyncRC);
-    GEN_CHECK_OFF(EHCI, pEOFTimerNoSyncR3);
-    GEN_CHECK_OFF(EHCI, pEOFTimerNoSyncR0);
-    GEN_CHECK_OFF(EHCI, pEOFTimerNoSyncRC);
     GEN_CHECK_OFF(EHCI, hThreadFrame);
     GEN_CHECK_OFF(EHCI, hSemEventFrame);
     GEN_CHECK_OFF(EHCI, fBusStarted);
@@ -1271,9 +1233,8 @@ int main()
     GEN_CHECK_OFF(XHCI, aBellsRung);
     GEN_CHECK_OFF(XHCI, pLoad);
 #  ifdef VBOX_WITH_STATISTICS
-    GEN_CHECK_OFF(XHCI, StatCanceledIsocUrbs);
-    GEN_CHECK_OFF(XHCI, StatCanceledGenUrbs);
-    GEN_CHECK_OFF(XHCI, StatDroppedUrbs);
+    GEN_CHECK_OFF(XHCI, StatErrorIsocUrbs);
+    GEN_CHECK_OFF(XHCI, StatErrorIsocPkts);
     GEN_CHECK_OFF(XHCI, StatEventsWritten);
     GEN_CHECK_OFF(XHCI, StatEventsDropped);
     GEN_CHECK_OFF(XHCI, StatIntrsPending);
@@ -1284,8 +1245,6 @@ int main()
 # endif /* VBOX_WITH_XHCI_IMPL */
 #endif /* VBOX_WITH_USB */
 
-    /* VMMDev/VBoxDev.cpp */
-
     /* Parallel/DevParallel.cpp */
     GEN_CHECK_SIZE(PARALLELPORT);
     GEN_CHECK_OFF(PARALLELPORT, pDevInsR3);
@@ -1314,43 +1273,72 @@ int main()
     GEN_CHECK_OFF(PARALLELPORT, act_fifo_pos_read);
 #endif
 
+    /* Serial/UartCore.cpp */
+    GEN_CHECK_SIZE(UARTCORE);
+    GEN_CHECK_OFF(UARTCORE, CritSect);
+    GEN_CHECK_OFF(UARTCORE, pDevInsR3);
+    GEN_CHECK_OFF(UARTCORE, pDevInsR0);
+    GEN_CHECK_OFF(UARTCORE, pDevInsRC);
+    GEN_CHECK_OFF(UARTCORE, iLUN);
+    GEN_CHECK_OFF(UARTCORE, IBase);
+    GEN_CHECK_OFF(UARTCORE, ISerialPort);
+    GEN_CHECK_OFF(UARTCORE, pDrvBase);
+    GEN_CHECK_OFF(UARTCORE, pDrvSerial);
+    GEN_CHECK_OFF(UARTCORE, fFlags);
+    GEN_CHECK_OFF(UARTCORE, enmType);
+    GEN_CHECK_OFF(UARTCORE, pTimerRcvFifoTimeoutR3);
+    GEN_CHECK_OFF(UARTCORE, pTimerTxUnconnectedR3);
+    GEN_CHECK_OFF(UARTCORE, pfnUartIrqReqR3);
+    GEN_CHECK_OFF(UARTCORE, pTimerRcvFifoTimeoutR0);
+    GEN_CHECK_OFF(UARTCORE, pTimerTxUnconnectedR0);
+    GEN_CHECK_OFF(UARTCORE, pfnUartIrqReqR0);
+    GEN_CHECK_OFF(UARTCORE, pTimerRcvFifoTimeoutRC);
+    GEN_CHECK_OFF(UARTCORE, pTimerTxUnconnectedRC);
+    GEN_CHECK_OFF(UARTCORE, pfnUartIrqReqRC);
+    GEN_CHECK_OFF(UARTCORE, uRegDivisor);
+    GEN_CHECK_OFF(UARTCORE, uRegRbr);
+    GEN_CHECK_OFF(UARTCORE, uRegThr);
+    GEN_CHECK_OFF(UARTCORE, uRegIer);
+    GEN_CHECK_OFF(UARTCORE, uRegIir);
+    GEN_CHECK_OFF(UARTCORE, uRegFcr);
+    GEN_CHECK_OFF(UARTCORE, uRegLcr);
+    GEN_CHECK_OFF(UARTCORE, uRegMcr);
+    GEN_CHECK_OFF(UARTCORE, uRegLsr);
+    GEN_CHECK_OFF(UARTCORE, uRegMsr);
+    GEN_CHECK_OFF(UARTCORE, uRegScr);
+    GEN_CHECK_OFF(UARTCORE, fIrqCtiPending);
+    GEN_CHECK_OFF(UARTCORE, fThreEmptyPending);
+    GEN_CHECK_OFF(UARTCORE, FifoXmit);
+    GEN_CHECK_OFF(UARTCORE, FifoRecv);
+    GEN_CHECK_OFF(UARTCORE, cSymbolXferTicks);
+    GEN_CHECK_OFF(UARTCORE, cbAvailRdr);
+
     /* Serial/DevSerial.cpp */
-    GEN_CHECK_SIZE(SerialState);
-    GEN_CHECK_OFF(SerialState, CritSect);
-    GEN_CHECK_OFF(SerialState, pDevInsR3);
-    GEN_CHECK_OFF(SerialState, pDevInsR0);
-    GEN_CHECK_OFF(SerialState, pDevInsRC);
-    GEN_CHECK_OFF(SerialState, IBase);
-    GEN_CHECK_OFF(SerialState, ICharPort);
-    GEN_CHECK_OFF(SerialState, pDrvBase);
-    GEN_CHECK_OFF(SerialState, pDrvChar);
-    GEN_CHECK_OFF(SerialState, ReceiveSem);
-    GEN_CHECK_OFF(SerialState, base);
-    GEN_CHECK_OFF(SerialState, divider);
-    GEN_CHECK_OFF(SerialState, recv_fifo);
-    GEN_CHECK_OFF(SerialState, xmit_fifo);
-    GEN_CHECK_OFF(SerialState, rbr);
-    GEN_CHECK_OFF(SerialState, thr);
-    GEN_CHECK_OFF(SerialState, tsr);
-    GEN_CHECK_OFF(SerialState, ier);
-    GEN_CHECK_OFF(SerialState, iir);
-    GEN_CHECK_OFF(SerialState, lcr);
-    GEN_CHECK_OFF(SerialState, mcr);
-    GEN_CHECK_OFF(SerialState, lsr);
-    GEN_CHECK_OFF(SerialState, msr);
-    GEN_CHECK_OFF(SerialState, scr);
-    GEN_CHECK_OFF(SerialState, fcr);
-    GEN_CHECK_OFF(SerialState, fcr_vmstate);
-    GEN_CHECK_OFF(SerialState, thr_ipending);
-    GEN_CHECK_OFF(SerialState, timeout_ipending);
-    GEN_CHECK_OFF(SerialState, irq);
-    GEN_CHECK_OFF(SerialState, last_break_enable);
-    GEN_CHECK_OFF(SerialState, tsr_retry);
-    GEN_CHECK_OFF(SerialState, msr_changed);
-    GEN_CHECK_OFF(SerialState, fGCEnabled);
-    GEN_CHECK_OFF(SerialState, fR0Enabled);
-    GEN_CHECK_OFF(SerialState, fYieldOnLSRRead);
-    GEN_CHECK_OFF(SerialState, char_transmit_time);
+    GEN_CHECK_SIZE(DEVSERIAL);
+    GEN_CHECK_OFF(DEVSERIAL, pDevInsR3);
+    GEN_CHECK_OFF(DEVSERIAL, pDevInsR0);
+    GEN_CHECK_OFF(DEVSERIAL, pDevInsRC);
+    GEN_CHECK_OFF(DEVSERIAL, fR0Enabled);
+    GEN_CHECK_OFF(DEVSERIAL, fRCEnabled);
+    GEN_CHECK_OFF(DEVSERIAL, uIrq);
+    GEN_CHECK_OFF(DEVSERIAL, PortBase);
+    GEN_CHECK_OFF(DEVSERIAL, UartCore);
+
+    /* Serial/DevOxPcie958.cpp */
+    GEN_CHECK_SIZE(DEVOX958);
+    GEN_CHECK_OFF(DEVOX958, PciDev);
+    GEN_CHECK_OFF(DEVOX958, pDevInsR3);
+    GEN_CHECK_OFF(DEVOX958, pDevInsR0);
+    GEN_CHECK_OFF(DEVOX958, pDevInsRC);
+    GEN_CHECK_OFF(DEVOX958, fR0Enabled);
+    GEN_CHECK_OFF(DEVOX958, fRCEnabled);
+    GEN_CHECK_OFF(DEVOX958, u32RegIrqStsGlob);
+    GEN_CHECK_OFF(DEVOX958, u32RegIrqEnGlob);
+    GEN_CHECK_OFF(DEVOX958, u32RegIrqEnWake);
+    GEN_CHECK_OFF(DEVOX958, cUarts);
+    GEN_CHECK_OFF(DEVOX958, GCPhysMMIO);
+    GEN_CHECK_OFF(DEVOX958, aUarts);
+    GEN_CHECK_OFF(DEVOX958, aUarts[OX958_UARTS_MAX - 1]);
 
 #ifdef VBOX_WITH_AHCI
     /* Storage/DevAHCI.cpp */
@@ -1381,11 +1369,10 @@ int main()
     GEN_CHECK_OFF(AHCIPort, GCPhysAddrFb);
     GEN_CHECK_OFF(AHCIPort, fPoweredOn);
     GEN_CHECK_OFF(AHCIPort, fSpunUp);
-    GEN_CHECK_OFF(AHCIPort, fFirstD2HFisSend);
+    GEN_CHECK_OFF(AHCIPort, fFirstD2HFisSent);
     GEN_CHECK_OFF(AHCIPort, fATAPI);
-    GEN_CHECK_OFF(AHCIPort, fATAPIPassthrough);
     GEN_CHECK_OFF(AHCIPort, fPortReset);
-    GEN_CHECK_OFF(AHCIPort, fAsyncInterface);
+    GEN_CHECK_OFF(AHCIPort, fTrimEnabled);
     GEN_CHECK_OFF(AHCIPort, fResetDevice);
     GEN_CHECK_OFF(AHCIPort, fHotpluggable);
     GEN_CHECK_OFF(AHCIPort, fRedo);
@@ -1395,10 +1382,7 @@ int main()
     GEN_CHECK_OFF(AHCIPort, cMultSectors);
     GEN_CHECK_OFF(AHCIPort, uATATransferMode);
     GEN_CHECK_OFF(AHCIPort, abATAPISense);
-    GEN_CHECK_OFF(AHCIPort, cNotifiedMediaChange);
     GEN_CHECK_OFF(AHCIPort, cLogSectorsPerPhysicalExp);
-    GEN_CHECK_OFF(AHCIPort, MediaEventStatus);
-    GEN_CHECK_OFF(AHCIPort, MediaTrackType);
     GEN_CHECK_OFF(AHCIPort, iLUN);
     GEN_CHECK_OFF(AHCIPort, u32TasksFinished);
     GEN_CHECK_OFF(AHCIPort, u32QueuedTasksFinished);
@@ -1407,43 +1391,30 @@ int main()
     GEN_CHECK_OFF(AHCIPort, u32CurrentCommandSlot);
     GEN_CHECK_OFF(AHCIPort, pDrvBase);
     GEN_CHECK_OFF(AHCIPort, pDrvMedia);
-    GEN_CHECK_OFF(AHCIPort, pDrvMediaAsync);
-    GEN_CHECK_OFF(AHCIPort, pDrvMount);
+    GEN_CHECK_OFF(AHCIPort, pDrvMediaEx);
     GEN_CHECK_OFF(AHCIPort, IBase);
     GEN_CHECK_OFF(AHCIPort, IPort);
-    GEN_CHECK_OFF(AHCIPort, IPortAsync);
-    GEN_CHECK_OFF(AHCIPort, IMountNotify);
+    GEN_CHECK_OFF(AHCIPort, IMediaExPort);
     GEN_CHECK_OFF(AHCIPort, PCHSGeometry);
     GEN_CHECK_OFF(AHCIPort, Led);
     GEN_CHECK_OFF(AHCIPort, pAsyncIOThread);
 
-    GEN_CHECK_OFF(AHCIPort, aActiveTasks);
     GEN_CHECK_OFF(AHCIPort, pTaskErr);
-    GEN_CHECK_OFF(AHCIPort, pTrackList);
     GEN_CHECK_OFF(AHCIPort, hEvtProcess);
-    GEN_CHECK_OFF(AHCIPort, StatDMA);
-    GEN_CHECK_OFF(AHCIPort, StatBytesWritten);
-    GEN_CHECK_OFF(AHCIPort, StatBytesRead);
-    GEN_CHECK_OFF(AHCIPort, StatIORequestsPerSecond);
-#ifdef VBOX_WITH_STATISTICS
-    GEN_CHECK_OFF(AHCIPort, StatProfileProcessTime);
-    GEN_CHECK_OFF(AHCIPort, StatProfileReadWrite);
-#endif
     GEN_CHECK_OFF(AHCIPort, szSerialNumber);
     GEN_CHECK_OFF(AHCIPort, szSerialNumber[AHCI_SERIAL_NUMBER_LENGTH]); /* One additional byte for the termination.*/
     GEN_CHECK_OFF(AHCIPort, szFirmwareRevision);
     GEN_CHECK_OFF(AHCIPort, szFirmwareRevision[AHCI_FIRMWARE_REVISION_LENGTH]); /* One additional byte for the termination.*/
     GEN_CHECK_OFF(AHCIPort, szModelNumber);
     GEN_CHECK_OFF(AHCIPort, szModelNumber[AHCI_MODEL_NUMBER_LENGTH]); /* One additional byte for the termination.*/
-    GEN_CHECK_OFF(AHCIPort, szInquiryVendorId[AHCI_ATAPI_INQUIRY_VENDOR_ID_LENGTH]);
+    GEN_CHECK_OFF(AHCIPort, szInquiryVendorId);
+    GEN_CHECK_OFF(AHCIPort, szInquiryVendorId[AHCI_ATAPI_INQUIRY_VENDOR_ID_LENGTH]); /* One additional byte for the termination.*/
     GEN_CHECK_OFF(AHCIPort, szInquiryProductId);
-    GEN_CHECK_OFF(AHCIPort, szInquiryProductId[AHCI_ATAPI_INQUIRY_PRODUCT_ID_LENGTH]);
+    GEN_CHECK_OFF(AHCIPort, szInquiryProductId[AHCI_ATAPI_INQUIRY_PRODUCT_ID_LENGTH]); /* One additional byte for the termination.*/
     GEN_CHECK_OFF(AHCIPort, szInquiryRevision);
-    GEN_CHECK_OFF(AHCIPort, szInquiryRevision[AHCI_ATAPI_INQUIRY_REVISION_LENGTH]);
+    GEN_CHECK_OFF(AHCIPort, szInquiryRevision[AHCI_ATAPI_INQUIRY_REVISION_LENGTH]); /* One additional byte for the termination.*/
     GEN_CHECK_OFF(AHCIPort, cErrors);
     GEN_CHECK_OFF(AHCIPort, fRedo);
-    GEN_CHECK_OFF(AHCIPort, CritSectReqsFree);
-    GEN_CHECK_OFF(AHCIPort, pListReqsFree);
 
     GEN_CHECK_SIZE(AHCI);
     GEN_CHECK_OFF(AHCI, dev);
@@ -1659,7 +1630,9 @@ int main()
     GEN_CHECK_OFF(VMMDEV, mouseXAbs);
     GEN_CHECK_OFF(VMMDEV, mouseYAbs);
     GEN_CHECK_OFF(VMMDEV, fHostCursorRequested);
-    GEN_CHECK_OFF(VMMDEV, pDevIns);
+    GEN_CHECK_OFF(VMMDEV, pDevInsR3);
+    GEN_CHECK_OFF(VMMDEV, pDevInsR0);
+    GEN_CHECK_OFF(VMMDEV, pDevInsRC);
     GEN_CHECK_OFF(VMMDEV, IBase);
     GEN_CHECK_OFF(VMMDEV, IPort);
 #ifdef VBOX_WITH_HGCM
@@ -1679,6 +1652,8 @@ int main()
     GEN_CHECK_OFF(VMMDEV, fNewGuestFilterMask);
     GEN_CHECK_OFF(VMMDEV, GCPhysVMMDevRAM);
     GEN_CHECK_OFF(VMMDEV, pVMMDevRAMR3);
+    GEN_CHECK_OFF(VMMDEV, pVMMDevRAMR0);
+    GEN_CHECK_OFF(VMMDEV, pVMMDevRAMRC);
     GEN_CHECK_OFF(VMMDEV, pVMMDevHeapR3);
     GEN_CHECK_OFF(VMMDEV, GCPhysVMMDevHeap);
     GEN_CHECK_OFF(VMMDEV, guestInfo);
@@ -1706,7 +1681,7 @@ int main()
     GEN_CHECK_OFF(VMMDEV, fKeepCredentials);
     GEN_CHECK_OFF(VMMDEV, fHeapEnabled);
 #ifdef VBOX_WITH_HGCM
-    GEN_CHECK_OFF(VMMDEV, pHGCMCmdList);
+    GEN_CHECK_OFF(VMMDEV, listHGCMCmd);
     GEN_CHECK_OFF(VMMDEV, critsectHGCMCmdList);
     GEN_CHECK_OFF(VMMDEV, u32HGCMEnabled);
 #endif
@@ -1744,10 +1719,12 @@ int main()
     GEN_CHECK_OFF(BUSLOGICDEVICE, fPresent);
     GEN_CHECK_OFF(BUSLOGICDEVICE, iLUN);
     GEN_CHECK_OFF(BUSLOGICDEVICE, IBase);
-    GEN_CHECK_OFF(BUSLOGICDEVICE, ISCSIPort);
+    GEN_CHECK_OFF(BUSLOGICDEVICE, IMediaPort);
+    GEN_CHECK_OFF(BUSLOGICDEVICE, IMediaExPort);
     GEN_CHECK_OFF(BUSLOGICDEVICE, ILed);
     GEN_CHECK_OFF(BUSLOGICDEVICE, pDrvBase);
-    GEN_CHECK_OFF(BUSLOGICDEVICE, pDrvSCSIConnector);
+    GEN_CHECK_OFF(BUSLOGICDEVICE, pDrvMedia);
+    GEN_CHECK_OFF(BUSLOGICDEVICE, pDrvMediaEx);
     GEN_CHECK_OFF(BUSLOGICDEVICE, Led);
     GEN_CHECK_OFF(BUSLOGICDEVICE, cOutstandingRequests);
 
@@ -1786,20 +1763,20 @@ int main()
     GEN_CHECK_OFF(BUSLOGIC, pNotifierQueueR0);
     GEN_CHECK_OFF(BUSLOGIC, pNotifierQueueRC);
     GEN_CHECK_OFF(BUSLOGIC, CritSectIntr);
-    GEN_CHECK_OFF(BUSLOGIC, hTaskCache);
     GEN_CHECK_OFF(BUSLOGIC, VBoxSCSI);
     GEN_CHECK_OFF(BUSLOGIC, aDeviceStates);
     GEN_CHECK_OFF(BUSLOGIC, aDeviceStates[BUSLOGIC_MAX_DEVICES-1]);
     GEN_CHECK_OFF(BUSLOGIC, IBase);
     GEN_CHECK_OFF(BUSLOGIC, ILeds);
     GEN_CHECK_OFF(BUSLOGIC, pLedsConnector);
+    GEN_CHECK_OFF(BUSLOGIC, pMediaNotify);
     GEN_CHECK_OFF(BUSLOGIC, fSignalIdle);
-    GEN_CHECK_OFF(BUSLOGIC, fRedo);
     GEN_CHECK_OFF(BUSLOGIC, fWrkThreadSleeping);
     GEN_CHECK_OFF(BUSLOGIC, fBiosReqPending);
-    GEN_CHECK_OFF(BUSLOGIC, pTasksRedoHead);
     GEN_CHECK_OFF(BUSLOGIC, pSupDrvSession);
     GEN_CHECK_OFF(BUSLOGIC, hEvtProcess);
+    GEN_CHECK_OFF(BUSLOGIC, paGCPhysAddrCCBRedo);
+    GEN_CHECK_OFF(BUSLOGIC, cReqsRedo);
 # ifdef LOG_ENABLED
     GEN_CHECK_OFF(BUSLOGIC, cInMailboxesReady);
 # endif
@@ -1867,10 +1844,10 @@ int main()
     GEN_CHECK_OFF(LSILOGICSCSI, u16NextHandle);
     GEN_CHECK_OFF(LSILOGICSCSI, enmCtrlType);
     GEN_CHECK_OFF(LSILOGICSCSI, VBoxSCSI);
-    GEN_CHECK_OFF(LSILOGICSCSI, hTaskCache);
     GEN_CHECK_OFF(LSILOGICSCSI, IBase);
     GEN_CHECK_OFF(LSILOGICSCSI, ILeds);
     GEN_CHECK_OFF(LSILOGICSCSI, pLedsConnector);
+    GEN_CHECK_OFF(LSILOGICSCSI, pMediaNotify);
     GEN_CHECK_OFF(LSILOGICSCSI, pConfigurationPages);
     GEN_CHECK_OFF(LSILOGICSCSI, fSignalIdle);
     GEN_CHECK_OFF(LSILOGICSCSI, fRedo);
@@ -1914,7 +1891,7 @@ int main()
     GEN_CHECK_SIZE(AC97DRIVER);
     GEN_CHECK_OFF(AC97DRIVER, Node);
     GEN_CHECK_OFF(AC97DRIVER, pAC97State);
-    GEN_CHECK_OFF(AC97DRIVER, Flags);
+    GEN_CHECK_OFF(AC97DRIVER, fFlags);
     GEN_CHECK_OFF(AC97DRIVER, uLUN);
     GEN_CHECK_OFF(AC97DRIVER, fAttached);
     GEN_CHECK_OFF(AC97DRIVER, pConnector);
@@ -1922,18 +1899,56 @@ int main()
     GEN_CHECK_OFF(AC97DRIVER, MicIn);
     GEN_CHECK_OFF(AC97DRIVER, Out);
 
-    GEN_CHECK_SIZE(HDAINPUTSTREAM);
-    GEN_CHECK_OFF(HDAINPUTSTREAM, pStrmIn);
-    GEN_CHECK_OFF(HDAINPUTSTREAM, phStrmIn);
+    GEN_CHECK_SIZE(AC97STATE);
+    GEN_CHECK_OFF(AC97STATE, CritSect);
+    GEN_CHECK_OFF(AC97STATE, pDevInsR3);
+    GEN_CHECK_OFF(AC97STATE, pDevInsR0);
+    GEN_CHECK_OFF(AC97STATE, pDevInsRC);
+    GEN_CHECK_OFF(AC97STATE, fRZEnabled);
+    GEN_CHECK_OFF(AC97STATE, glob_cnt);
+    GEN_CHECK_OFF(AC97STATE, glob_sta);
+    GEN_CHECK_OFF(AC97STATE, cas);
+    GEN_CHECK_OFF(AC97STATE, last_samp);
+    GEN_CHECK_OFF(AC97STATE, mixer_data);
+    GEN_CHECK_OFF(AC97STATE, aStreams);
+    GEN_CHECK_OFF(AC97STATE, uTimerHz);
+    GEN_CHECK_OFF(AC97STATE, pTimerRC);
+    GEN_CHECK_OFF(AC97STATE, pTimerR3);
+    GEN_CHECK_OFF(AC97STATE, pTimerR0);
+#ifdef VBOX_WITH_STATISTICS
+    GEN_CHECK_OFF(AC97STATE, StatTimer);
+    GEN_CHECK_OFF(AC97STATE, StatIn);
+    GEN_CHECK_OFF(AC97STATE, StatOut);
+    GEN_CHECK_OFF(AC97STATE, StatBytesRead);
+    GEN_CHECK_OFF(AC97STATE, StatBytesWritten);
+#endif
+    GEN_CHECK_OFF(AC97STATE, lstDrv);
+    GEN_CHECK_OFF(AC97STATE, pMixer);
+    GEN_CHECK_OFF(AC97STATE, pSinkOut);
+    GEN_CHECK_OFF(AC97STATE, pSinkLineIn);
+    GEN_CHECK_OFF(AC97STATE, pSinkMicIn);
+    GEN_CHECK_OFF(AC97STATE, silence);
+    GEN_CHECK_OFF(AC97STATE, bup_flag);
+    GEN_CHECK_OFF(AC97STATE, IBase);
+    GEN_CHECK_OFF(AC97STATE, IOPortBase);
+    GEN_CHECK_OFF(AC97STATE, uCodecModel);
 
-    GEN_CHECK_SIZE(HDAOUTPUTSTREAM);
-    GEN_CHECK_OFF(HDAOUTPUTSTREAM, pStrmOut);
-    GEN_CHECK_OFF(HDAOUTPUTSTREAM, phStrmOut);
+    GEN_CHECK_SIZE(AC97STREAM);
+    GEN_CHECK_OFF(AC97STREAM, pAC97State);
+
+    GEN_CHECK_SIZE(AC97STREAMSTATE);
+    GEN_CHECK_OFF(AC97STREAMSTATE, CritSect);
+    GEN_CHECK_OFF(AC97STREAMSTATE, pCircBuf);
+    GEN_CHECK_OFF(AC97STREAMSTATE, tsTransferLast);
+    GEN_CHECK_OFF(AC97STREAMSTATE, cTicksPerByte);
+
+    GEN_CHECK_SIZE(HDADRIVERSTREAM);
+    GEN_CHECK_OFF(HDADRIVERSTREAM, pMixStrm);
 
     GEN_CHECK_SIZE(HDADRIVER);
     GEN_CHECK_OFF(HDADRIVER, Node);
     GEN_CHECK_OFF(HDADRIVER, pHDAState);
-    GEN_CHECK_OFF(HDADRIVER, Flags);
+    GEN_CHECK_OFF(HDADRIVER, fFlags);
     GEN_CHECK_OFF(HDADRIVER, uLUN);
     GEN_CHECK_OFF(HDADRIVER, fAttached);
     GEN_CHECK_OFF(HDADRIVER, pConnector);
@@ -1941,31 +1956,49 @@ int main()
 #ifdef VBOX_WITH_HDA_MIC_IN
     GEN_CHECK_OFF(HDADRIVER, MicIn);
 #endif
-    GEN_CHECK_OFF(HDADRIVER, Out);
+    GEN_CHECK_OFF(HDADRIVER, Front);
+#ifdef VBOX_WITH_HDA_51_SURROUND
+    GEN_CHECK_OFF(HDADRIVER, CenterLFE);
+    GEN_CHECK_OFF(HDADRIVER, Rear);
+#endif
 
     GEN_CHECK_SIZE(HDABDLESTATE);
     GEN_CHECK_OFF(HDABDLESTATE, u32BDLIndex);
     GEN_CHECK_OFF(HDABDLESTATE, cbBelowFIFOW);
-    GEN_CHECK_OFF(HDABDLESTATE, au8FIFO);
     GEN_CHECK_OFF(HDABDLESTATE, u32BufOff);
 
+    GEN_CHECK_SIZE(HDABDLEDESC);
+    GEN_CHECK_OFF(HDABDLEDESC, u64BufAddr);
+    GEN_CHECK_OFF(HDABDLEDESC, u32BufSize);
+    GEN_CHECK_OFF(HDABDLEDESC, fFlags);
+
     GEN_CHECK_SIZE(HDABDLE);
-    GEN_CHECK_OFF(HDABDLE, u64BufAdr);
-    GEN_CHECK_OFF(HDABDLE, u32BufSize);
-    GEN_CHECK_OFF(HDABDLE, fIntOnCompletion);
+    GEN_CHECK_OFF(HDABDLE, Desc);
     GEN_CHECK_OFF(HDABDLE, State);
 
+    GEN_CHECK_SIZE(HDASTREAMMAP);
+
     GEN_CHECK_SIZE(HDASTREAMSTATE);
     GEN_CHECK_OFF(HDASTREAMSTATE, uCurBDLE);
+    GEN_CHECK_OFF(HDASTREAMSTATE, fInReset);
+    GEN_CHECK_OFF(HDASTREAMSTATE, Mapping);
     GEN_CHECK_OFF(HDASTREAMSTATE, BDLE);
+    GEN_CHECK_OFF(HDASTREAMSTATE, pCircBuf);
+
+    GEN_CHECK_SIZE(HDASTREAMDBGINFORT);
+
+    GEN_CHECK_SIZE(HDASTREAMDBGINFO);
+    GEN_CHECK_OFF(HDASTREAMDBGINFO, Runtime);
 
     GEN_CHECK_SIZE(HDASTREAM);
-    GEN_CHECK_OFF(HDASTREAM, u8Strm);
+    GEN_CHECK_OFF(HDASTREAM, u8SD);
     GEN_CHECK_OFF(HDASTREAM, u64BDLBase);
     GEN_CHECK_OFF(HDASTREAM, u16FMT);
     GEN_CHECK_OFF(HDASTREAM, u16FIFOS);
     GEN_CHECK_OFF(HDASTREAM, u16LVI);
     GEN_CHECK_OFF(HDASTREAM, State);
+    GEN_CHECK_OFF(HDASTREAM, Dbg);
+    GEN_CHECK_OFF(HDASTREAM, CritSect);
 
     GEN_CHECK_SIZE(HDASTATE);
     GEN_CHECK_OFF(HDASTATE, PciDev);
@@ -1975,10 +2008,9 @@ int main()
     GEN_CHECK_OFF(HDASTATE, IBase);
     GEN_CHECK_OFF(HDASTATE, MMIOBaseAddr);
     GEN_CHECK_OFF(HDASTATE, au32Regs[0]);
-    GEN_CHECK_OFF(HDASTATE, au32Regs[HDA_NREGS]);
-    GEN_CHECK_OFF(HDASTATE, StrmStLineIn);
-    GEN_CHECK_OFF(HDASTATE, StrmStOut);
-    GEN_CHECK_OFF(HDASTATE, StrmStMicIn);
+    GEN_CHECK_OFF(HDASTATE, au32Regs[HDA_NUM_REGS]);
+    GEN_CHECK_OFF(HDASTATE, aStreams);
+    GEN_CHECK_OFF(HDASTATE, aTags);
     GEN_CHECK_OFF(HDASTATE, u64CORBBase);
     GEN_CHECK_OFF(HDASTATE, u64RIRBBase);
     GEN_CHECK_OFF(HDASTATE, u64DPBase);
@@ -1986,14 +2018,7 @@ int main()
     GEN_CHECK_OFF(HDASTATE, cbCorbBuf);
     GEN_CHECK_OFF(HDASTATE, pu64RirbBuf);
     GEN_CHECK_OFF(HDASTATE, cbRirbBuf);
-    GEN_CHECK_OFF(HDASTATE, fInReset);
-    GEN_CHECK_OFF(HDASTATE, fR0Enabled);
-    GEN_CHECK_OFF(HDASTATE, fRCEnabled);
-#ifndef VBOX_WITH_AUDIO_CALLBACKS
-    GEN_CHECK_OFF(HDASTATE, pTimer);
-    GEN_CHECK_OFF(HDASTATE, cTimerTicks);
-    GEN_CHECK_OFF(HDASTATE, uTimerTS);
-#endif
+    GEN_CHECK_OFF(HDASTATE, fRZEnabled);
 #ifdef VBOX_WITH_STATISTICS
 # ifndef VBOX_WITH_AUDIO_CALLBACKS
     GEN_CHECK_OFF(HDASTATE, StatTimer);
@@ -2004,12 +2029,19 @@ int main()
     GEN_CHECK_OFF(HDASTATE, pCodec);
     GEN_CHECK_OFF(HDASTATE, lstDrv);
     GEN_CHECK_OFF(HDASTATE, pMixer);
-    GEN_CHECK_OFF(HDASTATE, pSinkLineIn);
-#ifdef VBOX_WITH_HDA_MIC_IN
-    GEN_CHECK_OFF(HDASTATE, pSinkMicIn);
+    GEN_CHECK_OFF(HDASTATE, SinkFront);
+#ifdef VBOX_WITH_HDA_51_SURROUND
+    GEN_CHECK_OFF(HDASTATE, SinkCenterLFE);
+    GEN_CHECK_OFF(HDASTATE, SinkRear);
 #endif
-    GEN_CHECK_OFF(HDASTATE, u64BaseTS);
-    GEN_CHECK_OFF(HDASTATE, u8RespIntCnt);
+    GEN_CHECK_OFF(HDASTATE, SinkLineIn);
+#ifdef VBOX_WITH_HDA_MIC_IN
+    GEN_CHECK_OFF(HDASTATE, SinkMicIn);
+#endif
+    GEN_CHECK_OFF(HDASTATE, u64WalClk);
+    GEN_CHECK_OFF(HDASTATE, u16RespIntCnt);
+    GEN_CHECK_OFF(HDASTATE, cPosAdjustFrames);
+    GEN_CHECK_OFF(HDASTATE, u8IRQL);
 
 #ifdef VBOX_WITH_NVME_IMPL
     GEN_CHECK_SIZE(NVMEQUEUEHDR);
@@ -2081,6 +2113,10 @@ int main()
     GEN_CHECK_OFF(NVME, paQueuesCompR0);
     GEN_CHECK_OFF(NVME, paQueuesSubmRC);
     GEN_CHECK_OFF(NVME, paQueuesCompRC);
+    GEN_CHECK_OFF(NVME, pvCtrlMemBufR3);
+    GEN_CHECK_OFF(NVME, GCPhysCtrlMemBuf);
+    GEN_CHECK_OFF(NVME, cbCtrlMemBuf);
+    GEN_CHECK_OFF(NVME, u32CtrlMemBufSz);
     GEN_CHECK_OFF(NVME, pWakeQueueR3);
     GEN_CHECK_OFF(NVME, pWakeQueueR0);
     GEN_CHECK_OFF(NVME, pWakeQueueRC);
@@ -2093,6 +2129,10 @@ int main()
     GEN_CHECK_OFF(NVME, LstWrkThrds);
     GEN_CHECK_OFF(NVME, CritSectWrkThrds);
     GEN_CHECK_OFF(NVME, fSignalIdle);
+# ifdef VBOX_WITH_STATISTICS
+    GEN_CHECK_OFF(NVME, aStatMemXfer[0]);
+    GEN_CHECK_OFF(NVME, aStatMemXfer[NVME_CMBSZ_SUPP_BIT_IDX_MAX]);
+# endif
 #endif
 
     return (0);
