@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,64 +15,51 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifdef VBOX_WITH_PRECOMPILED_HEADERS
-# include <precomp.h>
-#else /* !VBOX_WITH_PRECOMPILED_HEADERS */
-
 /* Qt includes: */
-# include <QMessageBox>
-
-/* GUI includes: */
-# include "VBoxGlobal.h"
-# include "UIMachine.h"
-# include "UISelectorWindow.h"
-# include "UIModalWindowManager.h"
-# ifdef Q_WS_MAC
-#  include "VBoxUtils.h"
-#  include "UICocoaApplication.h"
-# endif /* Q_WS_MAC */
-
-/* Other VBox includes: */
-# include <iprt/buildconfig.h>
-# include <iprt/stream.h>
-# include <VBox/version.h>
-# ifdef VBOX_WITH_HARDENING
-#  include <VBox/sup.h>
-# else /* !VBOX_WITH_HARDENING */
-#  include <iprt/initterm.h>
-#  ifdef Q_WS_MAC
-#   include <iprt/asm.h>
-#  endif /* Q_WS_MAC */
-# endif /* !VBOX_WITH_HARDENING */
-# ifdef Q_WS_X11
-#  include <iprt/env.h>
-# endif /* Q_WS_X11 */
-
-#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
-
-/* Qt includes: */
-#ifdef Q_WS_X11
-# ifdef Q_OS_SOLARIS
-#  include <QPlastiqueStyle>
-# endif /* Q_OS_SOLARIS */
+#include <QApplication>
+#include <QMessageBox>
+#ifdef VBOX_WS_X11
 # ifndef Q_OS_SOLARIS
 #  include <QFontDatabase>
-# endif /* !Q_OS_SOLARIS */
-#endif /* Q_WS_X11 */
+# endif
+#endif
+
+/* GUI includes: */
+#include "VBoxGlobal.h"
+#include "UIStarter.h"
+#include "UIModalWindowManager.h"
+#ifdef VBOX_WS_MAC
+# include "VBoxUtils.h"
+# include "UICocoaApplication.h"
+#endif /* VBOX_WS_MAC */
 
 /* Other VBox includes: */
+#include <iprt/buildconfig.h>
+#include <iprt/stream.h>
+#include <VBox/err.h>
+#include <VBox/version.h>
+#ifdef VBOX_WITH_HARDENING
+# include <VBox/sup.h>
+#endif
+#if !defined(VBOX_WITH_HARDENING) || defined(VBOX_GUI_WITH_SHARED_LIBRARY) && !defined(VBOX_RUNTIME_UI)
+# include <iprt/initterm.h>
+# ifdef VBOX_WS_MAC
+#  include <iprt/asm.h>
+# endif
+#endif
+#ifdef VBOX_WS_X11
+# include <iprt/env.h>
+#endif
 #ifdef VBOX_WITH_HARDENING
 # include <iprt/ctype.h>
-#else /* !VBOX_WITH_HARDENING */
-# include <VBox/err.h>
-#endif /* !VBOX_WITH_HARDENING */
+#endif
 
 /* Other includes: */
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
 # include <dlfcn.h>
 # include <sys/mman.h>
-#endif /* Q_WS_MAC */
-#ifdef Q_WS_X11
+#endif /* VBOX_WS_MAC */
+#ifdef VBOX_WS_X11
 # include <dlfcn.h>
 # include <unistd.h>
 # include <X11/Xlib.h>
@@ -89,7 +76,7 @@
 #   define REG_PC REG_EIP
 #  endif /* !RT_ARCH_AMD64 */
 # endif /* RT_OS_LINUX && DEBUG */
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
 
 
 /* XXX Temporarily. Don't rely on the user to hack the Makefile himself! */
@@ -102,10 +89,8 @@ QString g_QStrHintLinuxNoDriver = QApplication::tr(
     "The VirtualBox Linux kernel driver (vboxdrv) is either not loaded or "
     "there is a permission problem with /dev/vboxdrv. Please reinstall the kernel "
     "module by executing<br/><br/>"
-    "  <font color=blue>'/sbin/rcvboxdrv setup'</font><br/><br/>"
-    "as root. If it is available in your distribution, you should install the "
-    "DKMS package first. This package keeps track of Linux kernel changes and "
-    "recompiles the vboxdrv kernel module if necessary."
+    "  <font color=blue>'/sbin/vboxconfig'</font><br/><br/>"
+    "as root."
     );
 
 QString g_QStrHintOtherWrongDriverVersion = QApplication::tr(
@@ -119,7 +104,7 @@ QString g_QStrHintLinuxWrongDriverVersion = QApplication::tr(
     "The VirtualBox kernel modules do not match this version of "
     "VirtualBox. The installation of VirtualBox was apparently not "
     "successful. Executing<br/><br/>"
-    "  <font color=blue>'/sbin/rcvboxdrv setup'</font><br/><br/>"
+    "  <font color=blue>'/sbin/vboxconfig'</font><br/><br/>"
     "may correct this. Make sure that you do not mix the "
     "OSE version and the PUEL version of VirtualBox."
     );
@@ -134,7 +119,7 @@ QString g_QStrHintReinstall = QApplication::tr(
     );
 
 
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
 /**
  * Mac OS X: Really ugly hack to bypass a set-uid check in AppKit.
  *
@@ -152,9 +137,9 @@ static void HideSetUidRootFromAppKit()
     if (!rc)
         ASMAtomicWriteU32((volatile uint32_t *)pvAddr, 0xccc3c031); /* xor eax, eax; ret; int3 */
 }
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-#ifdef Q_WS_X11
+#ifdef VBOX_WS_X11
 /** X11: For versions of Xlib which are aware of multi-threaded environments this function
   *      calls for XInitThreads() which initializes Xlib support for concurrent threads.
   * @returns @c non-zero unless it is unsafe to make multi-threaded calls to Xlib.
@@ -225,103 +210,187 @@ static void InstallSignalHandler()
     sigaction(SIGUSR1, &sa, 0);
 }
 # endif /* RT_OS_LINUX && DEBUG */
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
 
-/** Qt message handler, function that prints out
-  * debug messages, warnings, critical and fatal error messages.
-  * @param type describes the type of message sent to a message handler.
-  * @param pMsg holds the pointer to the message body. */
-static void QtMessageOutput(QtMsgType type, const char *pMsg)
+/** Qt5 message handler, function that prints out
+  * debug, warning, critical, fatal and system error messages.
+  * @param  type        Holds the type of the message.
+  * @param  context     Holds the message context.
+  * @param  strMessage  Holds the message body. */
+static void QtMessageOutput(QtMsgType type, const QMessageLogContext &context, const QString &strMessage)
 {
-#ifndef Q_WS_X11
-    NOREF(pMsg);
-#endif /* !Q_WS_X11 */
+    NOREF(context);
+# ifndef VBOX_WS_X11
+    NOREF(strMessage);
+# endif
     switch (type)
     {
         case QtDebugMsg:
-            Log(("Qt DEBUG: %s\n", pMsg));
+            Log(("Qt DEBUG: %s\n", strMessage.toUtf8().constData()));
             break;
         case QtWarningMsg:
-            Log(("Qt WARNING: %s\n", pMsg));
-#ifdef Q_WS_X11
+            Log(("Qt WARNING: %s\n", strMessage.toUtf8().constData()));
+# ifdef VBOX_WS_X11
             /* Needed for instance for the message ``cannot connect to X server'': */
-            RTStrmPrintf(g_pStdErr, "Qt WARNING: %s\n", pMsg);
-#endif /* Q_WS_X11 */
+            RTStrmPrintf(g_pStdErr, "Qt WARNING: %s\n", strMessage.toUtf8().constData());
+# endif
             break;
         case QtCriticalMsg:
-            Log(("Qt CRITICAL: %s\n", pMsg));
-#ifdef Q_WS_X11
+            Log(("Qt CRITICAL: %s\n", strMessage.toUtf8().constData()));
+# ifdef VBOX_WS_X11
             /* Needed for instance for the message ``cannot connect to X server'': */
-            RTStrmPrintf(g_pStdErr, "Qt CRITICAL: %s\n", pMsg);
-#endif /* Q_WS_X11 */
+            RTStrmPrintf(g_pStdErr, "Qt CRITICAL: %s\n", strMessage.toUtf8().constData());
+# endif
             break;
         case QtFatalMsg:
-            Log(("Qt FATAL: %s\n", pMsg));
-#ifdef Q_WS_X11
+            Log(("Qt FATAL: %s\n", strMessage.toUtf8().constData()));
+# ifdef VBOX_WS_X11
             /* Needed for instance for the message ``cannot connect to X server'': */
-            RTStrmPrintf(g_pStdErr, "Qt FATAL: %s\n", pMsg);
-#endif /* Q_WS_X11 */
+            RTStrmPrintf(g_pStdErr, "Qt FATAL: %s\n", strMessage.toUtf8().constData());
+# endif
+# if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
+        case QtInfoMsg:
+            /** @todo ignore? */
+            break;
+# endif
     }
 }
 
 /** Shows all available command line parameters. */
 static void ShowHelp()
 {
-    RTPrintf(VBOX_PRODUCT " Manager %s\n"
-            "(C) 2005-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
-            "All rights reserved.\n"
-            "\n"
-            "Usage:\n"
-            "  --startvm <vmname|UUID>    start a VM by specifying its UUID or name\n"
-            "  --separate                 start a separate VM process\n"
-            "  --normal                   keep normal (windowed) mode during startup\n"
-            "  --fullscreen               switch to fullscreen mode during startup\n"
-            "  --seamless                 switch to seamless mode during startup\n"
-            "  --scale                    switch to scale mode during startup\n"
-            "  --no-startvm-errormsgbox   do not show a message box for VM start errors\n"
-            "  --restore-current          restore the current snapshot before starting\n"
-            "  --no-aggressive-caching    delays caching media info in VM processes\n"
-            "  --fda <image|none>         Mount the specified floppy image\n"
-            "  --dvd <image|none>         Mount the specified DVD image\n"
+#if !defined(VBOX_GUI_WITH_SHARED_LIBRARY) || !defined(VBOX_RUNTIME_UI)
+    static const char s_szTitle[] = VBOX_PRODUCT " VM Selector";
+#else
+    static const char s_szTitle[] = VBOX_PRODUCT " VM Runner";
+#endif
+    static const char s_szUsage[] =
+#if !defined(VBOX_GUI_WITH_SHARED_LIBRARY) || defined(VBOX_RUNTIME_UI)
+        "Options:\n"
+        "  --startvm <vmname|UUID>    start a VM by specifying its UUID or name\n"
+        "  --separate                 start a separate VM process\n"
+        "  --normal                   keep normal (windowed) mode during startup\n"
+        "  --fullscreen               switch to fullscreen mode during startup\n"
+        "  --seamless                 switch to seamless mode during startup\n"
+        "  --scale                    switch to scale mode during startup\n"
+        "  --no-startvm-errormsgbox   do not show a message box for VM start errors\n"
+        "  --restore-current          restore the current snapshot before starting\n"
+        "  --no-aggressive-caching    delays caching media info in VM processes\n"
+        "  --fda <image|none>         Mount the specified floppy image\n"
+        "  --dvd <image|none>         Mount the specified DVD image\n"
 # ifdef VBOX_GUI_WITH_PIDFILE
-            "  --pidfile <file>           create a pidfile file when a VM is up and running\n"
+        "  --pidfile <file>           create a pidfile file when a VM is up and running\n"
 # endif /* VBOX_GUI_WITH_PIDFILE */
 # ifdef VBOX_WITH_DEBUGGER_GUI
-            "  --dbg                      enable the GUI debug menu\n"
-            "  --debug                    like --dbg and show debug windows at VM startup\n"
-            "  --debug-command-line       like --dbg and show command line window at VM startup\n"
-            "  --debug-statistics         like --dbg and show statistics window at VM startup\n"
-            "  --no-debug                 disable the GUI debug menu and debug windows\n"
-            "  --start-paused             start the VM in the paused state\n"
-            "  --start-running            start the VM running (for overriding --debug*)\n"
-            "\n"
+        "  --dbg                      enable the GUI debug menu\n"
+        "  --debug                    like --dbg and show debug windows at VM startup\n"
+        "  --debug-command-line       like --dbg and show command line window at VM startup\n"
+        "  --debug-statistics         like --dbg and show statistics window at VM startup\n"
+        "  --no-debug                 disable the GUI debug menu and debug windows\n"
+        "  --start-paused             start the VM in the paused state\n"
+        "  --start-running            start the VM running (for overriding --debug*)\n"
 # endif /* VBOX_WITH_DEBUGGER_GUI */
-            "Expert options:\n"
-            "  --disable-patm             disable code patching (ignored by AMD-V/VT-x)\n"
-            "  --disable-csam             disable code scanning (ignored by AMD-V/VT-x)\n"
-            "  --recompile-supervisor     recompiled execution of supervisor code (*)\n"
-            "  --recompile-user           recompiled execution of user code (*)\n"
-            "  --recompile-all            recompiled execution of all code, with disabled\n"
-            "                             code patching and scanning\n"
-            "  --execute-all-in-iem       For debugging the interpreted execution mode.\n"
-            "  --warp-pct <pct>           time warp factor, 100%% (= 1.0) = normal speed\n"
-            "  (*) For AMD-V/VT-x setups the effect is --recompile-all.\n"
-            "\n"
+        "\n"
+        "Expert options:\n"
+        "  --disable-patm             disable code patching (ignored by AMD-V/VT-x)\n"
+        "  --disable-csam             disable code scanning (ignored by AMD-V/VT-x)\n"
+        "  --recompile-supervisor     recompiled execution of supervisor code (*)\n"
+        "  --recompile-user           recompiled execution of user code (*)\n"
+        "  --recompile-all            recompiled execution of all code, with disabled\n"
+        "                             code patching and scanning\n"
+        "  --execute-all-in-iem       For debugging the interpreted execution mode.\n"
+        "  --warp-pct <pct>           time warp factor, 100%% (= 1.0) = normal speed\n"
+        "  (*) For AMD-V/VT-x setups the effect is --recompile-all.\n"
+        "\n"
 # ifdef VBOX_WITH_DEBUGGER_GUI
-            "The following environment (and extra data) variables are evaluated:\n"
-            "  VBOX_GUI_DBG_ENABLED (GUI/Dbg/Enabled)\n"
-            "                             enable the GUI debug menu if set\n"
-            "  VBOX_GUI_DBG_AUTO_SHOW (GUI/Dbg/AutoShow)\n"
-            "                             show debug windows at VM startup\n"
-            "  VBOX_GUI_NO_DEBUGGER       disable the GUI debug menu and debug windows\n"
+        "The following environment (and extra data) variables are evaluated:\n"
+        "  VBOX_GUI_DBG_ENABLED (GUI/Dbg/Enabled)\n"
+        "                             enable the GUI debug menu if set\n"
+        "  VBOX_GUI_DBG_AUTO_SHOW (GUI/Dbg/AutoShow)\n"
+        "                             show debug windows at VM startup\n"
+        "  VBOX_GUI_NO_DEBUGGER\n"
+        "                             disable the GUI debug menu and debug windows\n"
 # endif /* VBOX_WITH_DEBUGGER_GUI */
-            "\n",
-            RTBldCfgVersion());
-    /** @todo Show this as a dialog on windows. */
+#else
+        "No special options.\n"
+        "\n"
+        "If you are looking for --startvm and related options, you need to use VirtualBoxVM.\n"
+#endif
+        ;
+
+    RTPrintf("%s v%s\n"
+             "(C) 2005-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
+             "All rights reserved.\n"
+             "\n"
+             "%s",
+             s_szTitle, RTBldCfgVersion(), s_szUsage);
+
+#ifdef RT_OS_WINDOWS
+    /* Show message box. Modify the option list a little
+     * to better make it fit in the upcoming dialog. */
+    char szTitleWithVersion[sizeof(s_szTitle) + 128];
+    char szMsg[sizeof(s_szUsage) + 128];
+    char *pszDst = szMsg;
+    size_t offSrc = 0;
+    while (offSrc < sizeof(s_szUsage) - 1U)
+    {
+        char ch;
+        if (   s_szUsage[offSrc]     == ' '
+            && s_szUsage[offSrc + 1] == ' '
+            && (   (ch = s_szUsage[offSrc + 2]) == '-' /* option line */
+                || ch == 'V' /* env.var. line */))
+        {
+            /* Split option lines: */
+            if (ch == '-')
+            {
+                offSrc += 2;
+                size_t cchOption = 0;
+                while (   s_szUsage[offSrc + cchOption] != ' '
+                       || s_szUsage[offSrc + cchOption + 1] != ' ')
+                    ++cchOption;
+
+                memcpy(pszDst, &s_szUsage[offSrc], cchOption);
+                offSrc += cchOption + 2;
+                pszDst += cchOption;
+            }
+            /* Change environment variable indentation: */
+            else
+            {
+                offSrc += 2;
+                size_t cchLine = 0;
+                while ((ch = s_szUsage[offSrc + cchLine]) != '\n' && ch != '\0')
+                    ++cchLine;
+
+                memcpy(pszDst, &s_szUsage[offSrc], cchLine);
+                offSrc += cchLine + 1;
+                pszDst += cchLine;
+            }
+            *pszDst++ = '\n';
+            *pszDst++ = '\t';
+
+            while (s_szUsage[offSrc] == ' ')
+                ++offSrc;
+        }
+
+        /* Copy up to and including end of line: */
+        while ((ch = s_szUsage[offSrc++]) != '\n' && ch != '\0')
+            *pszDst++ = ch;
+        *pszDst++ = ch;
+    }
+    *pszDst = '\0';
+
+    RTStrPrintf(szTitleWithVersion, sizeof(szTitleWithVersion), "%s v%s - Command Line Options", s_szTitle, RTBldCfgVersion());
+    MessageBoxExA(NULL /*hwndOwner*/, szMsg, szTitleWithVersion, MB_OK | MB_ICONINFORMATION,
+                  MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL));
+#endif
 }
 
 extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
 {
+#ifdef RT_OS_WINDOWS
+    ATL::CComModule _Module; /* Required internally by ATL (constructor records instance in global variable). */
+#endif
+
     /* Failed result initially: */
     int iResultCode = 1;
 
@@ -331,16 +400,16 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
     /* Simulate try-catch block: */
     do
     {
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
         /* Hide setuid root from AppKit: */
         HideSetUidRootFromAppKit();
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-#ifdef Q_WS_X11
+#ifdef VBOX_WS_X11
         /* Make sure multi-threaded environment is safe: */
         if (!MakeSureMultiThreadingIsSafe())
             break;
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
 
         /* Console help preprocessing: */
         bool fHelpShown = false;
@@ -367,7 +436,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
         SUPR3HardenedVerifyInit();
 #endif /* VBOX_WITH_HARDENING */
 
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
         /* Apply font fixes (before QApplication get created and instantiated font-hints): */
         switch (VBoxGlobal::determineOsRelease())
         {
@@ -377,26 +446,30 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
             default: break;
         }
 
-# ifdef QT_MAC_USE_COCOA
         /* Instantiate own NSApplication before QApplication do it for us: */
         UICocoaApplication::instance();
-# endif /* QT_MAC_USE_COCOA */
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-#ifdef Q_WS_X11
+#ifdef VBOX_WS_X11
 # if defined(RT_OS_LINUX) && defined(DEBUG)
         /* Install signal handler to backtrace the call stack: */
         InstallSignalHandler();
 # endif /* RT_OS_LINUX && DEBUG */
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
 
         /* Install Qt console message handler: */
-        qInstallMsgHandler(QtMessageOutput);
+        qInstallMessageHandler(QtMessageOutput);
+
+        /* Enable HiDPI support: */
+        QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+#if (!defined(DEBUG_bird) || defined(RT_OS_DARWIN))
+        QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
 
         /* Create application: */
         QApplication a(argc, argv);
 
-#ifdef Q_WS_WIN
+#ifdef VBOX_WS_WIN
         /* Drag in the sound drivers and DLLs early to get rid of the delay taking
          * place when the main menu bar (or any action from that menu bar) is
          * activated for the first time. This delay is especially annoying if it
@@ -404,21 +477,14 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
          * load and slows down the load process that happens on the main GUI
          * thread to several seconds). */
         PlaySound(NULL, NULL, 0);
-#endif /* Q_WS_WIN */
-
-#ifdef Q_WS_MAC
-# ifdef VBOX_GUI_WITH_HIDPI
-        /* Enable HiDPI icons.
-         * For this we require a patched version of Qt 4.x with
-         * the changes from https://codereview.qt-project.org/#change,54636 applied. */
-        a.setAttribute(Qt::AA_UseHighDpiPixmaps);
-# endif /* VBOX_GUI_WITH_HIDPI */
+#endif /* VBOX_WS_WIN */
 
+#ifdef VBOX_WS_MAC
         /* Disable menu icons on MacOS X host: */
         ::darwinDisableIconsInMenus();
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 
-#ifdef Q_WS_X11
+#ifdef VBOX_WS_X11
         /* Make all widget native.
          * We did it to avoid various Qt crashes while testing widget attributes or acquiring winIds.
          * Yes, we aware of note that alien widgets faster to draw but the only widget we need to be fast
@@ -426,8 +492,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
         a.setAttribute(Qt::AA_NativeWindows);
 
 # ifdef Q_OS_SOLARIS
-        /* Use plastique look&feel for Solaris instead of the default motif (Qt 4.7.x): */
-        QApplication::setStyle(new QPlastiqueStyle);
+        a.setStyle("fusion");
 # endif /* Q_OS_SOLARIS */
 
 # ifndef Q_OS_SOLARIS
@@ -438,7 +503,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
         QString subFamily(QFont::substitute(currentFamily));
         bool isSubScaleable = fontDataBase.isScalable(subFamily);
         if (isCurrentScaleable && !isSubScaleable)
-            QFont::removeSubstitution(currentFamily);
+            QFont::removeSubstitutions(currentFamily);
 # endif /* !Q_OS_SOLARIS */
 
         /* Qt version check (major.minor are sensitive, fix number is ignored): */
@@ -453,13 +518,25 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
             qFatal("%s", strMsg.toUtf8().constData());
             break;
         }
-#endif /* Q_WS_X11 */
+#endif /* VBOX_WS_X11 */
 
         /* Create modal-window manager: */
         UIModalWindowManager::create();
 
-        /* Create global UI instance: */
+        /* Create UI starter: */
+        UIStarter::create();
+#ifndef VBOX_GUI_WITH_SHARED_LIBRARY
+        /* Create global app instance: */
         VBoxGlobal::create();
+#else
+# ifndef VBOX_RUNTIME_UI
+        /* Create global app instance for Selector UI: */
+        VBoxGlobal::create(VBoxGlobal::UIType_SelectorUI);
+# else
+        /* Create global app instance for Runtime UI: */
+        VBoxGlobal::create(VBoxGlobal::UIType_RuntimeUI);
+# endif
+#endif
 
         /* Simulate try-catch block: */
         do
@@ -468,21 +545,39 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
             if (!vboxGlobal().isValid())
                 break;
 
+            /* Init link between UI starter and global app instance: */
+            gStarter->init();
+
             /* Exit if VBoxGlobal pre-processed arguments: */
             if (vboxGlobal().processArgs())
                 break;
 
-            /* Exit if VBoxGlobal is unable to show UI: */
-            if (!vboxGlobal().showUI())
-                break;
+            // WORKAROUND:
+            // Initially we wanted to make that workaround for Runtime UI only,
+            // because only there we had a strict handling for proper application quit
+            // procedure.  But it appeared on X11 (as usually due to an async nature) there
+            // can happen situations that Qt application is checking whether at least one
+            // window is already shown and if not - exits prematurely _before_ it is actually
+            // shown.  That can happen for example if window is not yet shown because blocked
+            // by startup error message-box which is not treated as real window by some
+            // reason.  So we are making application exit manual everywhere.
+            qApp->setQuitOnLastWindowClosed(false);
+
+            /* Request to Start UI _after_ QApplication executed: */
+            QMetaObject::invokeMethod(gStarter, "sltStartUI", Qt::QueuedConnection);
 
             /* Start application: */
             iResultCode = a.exec();
+
+            /* Break link between UI starter and global app instance: */
+            gStarter->deinit();
         }
         while (0);
 
-        /* Destroy global UI instance: */
+        /* Destroy global app instance: */
         VBoxGlobal::destroy();
+        /* Destroy UI starter: */
+        UIStarter::destroy();
 
         /* Destroy modal-window manager: */
         UIModalWindowManager::destroy();
@@ -497,18 +592,19 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
     return iResultCode;
 }
 
-#ifndef VBOX_WITH_HARDENING
+#if !defined(VBOX_WITH_HARDENING) || (defined(VBOX_GUI_WITH_SHARED_LIBRARY) && !defined(VBOX_RUNTIME_UI))
 
 int main(int argc, char **argv, char **envp)
 {
-#ifdef Q_WS_X11
+# ifdef VBOX_WS_X11
     /* Make sure multi-threaded environment is safe: */
     if (!MakeSureMultiThreadingIsSafe())
         return 1;
-#endif /* Q_WS_X11 */
+# endif /* VBOX_WS_X11 */
 
-    /* Initialize VBox Runtime.
-     * Initialize the SUPLib as well only if we are really about to start a VM.
+    /* Initialize VBox Runtime: */
+# if !defined(VBOX_GUI_WITH_SHARED_LIBRARY) || defined(VBOX_RUNTIME_UI)
+    /* Initialize the SUPLib as well only if we are really about to start a VM.
      * Don't do this if we are only starting the selector window or a separate VM process. */
     bool fStartVM = false;
     bool fSeparateProcess = false;
@@ -519,18 +615,16 @@ int main(int argc, char **argv, char **envp)
          * otherwise there will be weird error messages. */
         if (   !::strcmp(argv[i], "--startvm")
             || !::strcmp(argv[i], "-startvm"))
-        {
             fStartVM = true;
-        }
         else if (   !::strcmp(argv[i], "--separate")
                  || !::strcmp(argv[i], "-separate"))
-        {
             fSeparateProcess = true;
-        }
     }
 
     uint32_t fFlags = fStartVM && !fSeparateProcess ? RTR3INIT_FLAGS_SUPLIB : 0;
-
+#else
+    uint32_t fFlags = 0;
+#endif
     int rc = RTR3InitExe(argc, &argv, fFlags);
 
     /* Initialization failed: */
@@ -541,10 +635,9 @@ int main(int argc, char **argv, char **envp)
         QApplication a(argc, &argv[0]);
         Q_UNUSED(a);
 
-#ifdef Q_OS_SOLARIS
-        /* Use plastique look&feel for Solaris instead of the default motif (Qt 4.7.x): */
-        QApplication::setStyle(new QPlastiqueStyle);
-#endif /* Q_OS_SOLARIS */
+# ifdef Q_OS_SOLARIS
+        a.setStyle("fusion");
+# endif
 
         /* Prepare the error-message: */
         QString strTitle = QApplication::tr("VirtualBox - Runtime Error");
@@ -556,24 +649,24 @@ int main(int argc, char **argv, char **envp)
                 strText += QApplication::tr("<b>Cannot access the kernel driver!</b><br/><br/>");
 # ifdef RT_OS_LINUX
                 strText += g_QStrHintLinuxNoDriver;
-# else /* RT_OS_LINUX */
+# else
                 strText += g_QStrHintOtherNoDriver;
-# endif /* !RT_OS_LINUX */
+# endif
                 break;
 # ifdef RT_OS_LINUX
             case VERR_NO_MEMORY:
                 strText += g_QStrHintLinuxNoMemory;
                 break;
-# endif /* RT_OS_LINUX */
+# endif
             case VERR_VM_DRIVER_NOT_ACCESSIBLE:
                 strText += QApplication::tr("Kernel driver not accessible");
                 break;
             case VERR_VM_DRIVER_VERSION_MISMATCH:
 # ifdef RT_OS_LINUX
                 strText += g_QStrHintLinuxWrongDriverVersion;
-# else /* RT_OS_LINUX */
+# else
                 strText += g_QStrHintOtherWrongDriverVersion;
-# endif /* !RT_OS_LINUX */
+# endif
                 break;
             default:
                 strText += QApplication::tr("Unknown error %2 during initialization of the Runtime").arg(rc);
@@ -589,12 +682,13 @@ int main(int argc, char **argv, char **envp)
         return 1;
     }
 
-    /* Actual main function: */
+    /* Call to actual main function: */
     return TrustedMain(argc, argv, envp);
 }
 
-#else  /* VBOX_WITH_HARDENING */
+#endif /* !VBOX_WITH_HARDENING || (VBOX_GUI_WITH_SHARED_LIBRARY && !VBOX_RUNTIME_UI) */
 
+#ifdef VBOX_WITH_HARDENING
 
 /**
  * Special entrypoint used by the hardening code when something goes south.
@@ -609,10 +703,10 @@ int main(int argc, char **argv, char **envp)
  */
 extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat, int rc, const char *pszMsgFmt, va_list va)
 {
-# ifdef Q_WS_MAC
+# ifdef VBOX_WS_MAC
     /* Hide setuid root from AppKit: */
     HideSetUidRootFromAppKit();
-# endif /* Q_WS_MAC */
+# endif /* VBOX_WS_MAC */
 
     char szMsgBuf[_16K];
 
@@ -671,19 +765,16 @@ extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat
             strText += g_QStrHintOtherNoDriver;
 # endif /* !RT_OS_LINUX */
             break;
-# ifdef RT_OS_LINUX
         case kSupInitOp_IPRT:
         case kSupInitOp_Misc:
-            if (rc == VERR_NO_MEMORY)
-                strText += g_QStrHintLinuxNoMemory;
-            else
-# endif /* RT_OS_LINUX */
             if (rc == VERR_VM_DRIVER_VERSION_MISMATCH)
-# ifdef RT_OS_LINUX
-                strText += g_QStrHintLinuxWrongDriverVersion;
-# else /* RT_OS_LINUX */
+# ifndef RT_OS_LINUX
                 strText += g_QStrHintOtherWrongDriverVersion;
-# endif /* !RT_OS_LINUX */
+# else
+                strText += g_QStrHintLinuxWrongDriverVersion;
+            else if (rc == VERR_NO_MEMORY)
+                strText += g_QStrHintLinuxNoMemory;
+# endif
             else
                 strText += g_QStrHintReinstall;
             break;
@@ -696,11 +787,11 @@ extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat
             break;
     }
 
-# ifdef Q_WS_X11
+# ifdef VBOX_WS_X11
     /* We have to to make sure that we display the error-message
      * after the parent displayed its own message. */
     sleep(2);
-# endif /* Q_WS_X11 */
+# endif /* VBOX_WS_X11 */
 
     /* Update strText with strDetails: */
     if (!strDetails.isEmpty())
@@ -716,5 +807,3 @@ extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat
 }
 
 #endif /* VBOX_WITH_HARDENING */
-
-
