@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2016 Oracle Corporation
+ * Copyright (C) 2010-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,19 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___UIDesktopServices_h___
-#define ___UIDesktopServices_h___
+#ifndef FEQT_INCLUDED_SRC_platform_UIDesktopServices_h
+#define FEQT_INCLUDED_SRC_platform_UIDesktopServices_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
+
+/* Qt includes */
+#include <QUuid>
+
+#ifdef VBOX_GUI_WITH_SHARED_LIBRARY
+/** Name of the executable (image) used to start VMs. */
+# define VBOX_GUI_VMRUNNER_IMAGE "VirtualBoxVM"
+#endif
 
 /* Qt forward declarations */
 class QString;
@@ -24,9 +35,9 @@ class QString;
 class UIDesktopServices
 {
 public:
-    static bool createMachineShortcut(const QString &strSrcFile, const QString &strDstPath, const QString &strName, const QString &strUuid);
+    static bool createMachineShortcut(const QString &strSrcFile, const QString &strDstPath, const QString &strName, const QUuid &uUuid);
     static bool openInFileManager(const QString &strFile);
 };
 
-#endif /* !___UIDesktopServices_h___ */
+#endif /* !FEQT_INCLUDED_SRC_platform_UIDesktopServices_h */
 
