@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -72,6 +72,7 @@
 #include <iprt/param.h>
 #include <iprt/path.h>
 #include <iprt/string.h>
+#include <iprt/utf16.h>
 
 #include "SUPLibInternal.h"
 #if defined(RT_OS_WINDOWS) && defined(VBOX_WITH_HARDENING)
@@ -121,12 +122,10 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
     /* ---------------------------------------------------------------------- */
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VMMR0.r0" },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDDR0.r0" },
-    {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDD2R0.r0" },
 
 #ifdef VBOX_WITH_RAW_MODE
     {   kSupIFT_Rc,   kSupID_AppPrivArch,       false, "VMMRC.rc" },
     {   kSupIFT_Rc,   kSupID_AppPrivArch,       false, "VBoxDDRC.rc" },
-    {   kSupIFT_Rc,   kSupID_AppPrivArch,       false, "VBoxDD2RC.rc" },
 #endif
 
     {   kSupIFT_Dll,  kSupID_AppSharedLib,      false, "VBoxRT" SUPLIB_DLL_SUFF },
@@ -198,9 +197,15 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
 
 //#ifdef VBOX_WITH_QTGUI
     {   kSupIFT_Exe,  kSupID_AppBin,             true, "VirtualBox" SUPLIB_EXE_SUFF },
+# ifndef VBOX_GUI_WITH_SHARED_LIBRARY
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VirtualBox" SUPLIB_DLL_SUFF },
-# ifdef RT_OS_DARWIN
+#  ifdef RT_OS_DARWIN
     {   kSupIFT_Exe,  kSupID_AppBin,             true, "VirtualBoxVM" SUPLIB_EXE_SUFF },
+#  endif
+# else
+    {   kSupIFT_Exe,  kSupID_AppBin,             true, "VirtualBoxVM" SUPLIB_EXE_SUFF },
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VirtualBoxVM" SUPLIB_DLL_SUFF },
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxGlobal" SUPLIB_DLL_SUFF },
 # endif
 # if !defined(RT_OS_DARWIN) && !defined(RT_OS_WINDOWS) && !defined(RT_OS_OS2)
     {   kSupIFT_Dll,  kSupID_AppSharedLib,       true, "VBoxKeyboard" SUPLIB_DLL_SUFF },
@@ -1190,12 +1195,20 @@ static int supR3HardenedVerifyPathSanity(const char *pszPath, PRTERRINFO pErrInf
         || pszSrc[2] == '\0')
         return supR3HardenedSetError3(VERR_SUPLIB_PATH_TOO_SHORT, pErrInfo, "The path is too short: '", pszPath, "'");
 
+#if RTPATH_STYLE == RTPATH_STR_F_STYLE_UNIX
+    /*
+     * Skip double slashes.
+     */
+    while (RTPATH_IS_SLASH(*pszSrc))
+        pszSrc++;
+#else
     /*
      * The root slash should be alone to avoid UNC confusion.
      */
     if (RTPATH_IS_SLASH(pszSrc[0]))
         return supR3HardenedSetError3(VERR_SUPLIB_PATH_NOT_CLEAN, pErrInfo,
                                       "The path is not clean of leading double slashes: '", pszPath, "'");
+#endif
     /*
      * Check each component.  No parent references.
      */
@@ -1470,14 +1483,14 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
     if (   (pFsObjState->Stat.st_mode & S_IWGRP)
         && pFsObjState->Stat.st_gid != 0)
     {
-#ifdef RT_OS_DARWIN
+# ifdef RT_OS_DARWIN
         /* HACK ALERT: On Darwin /Applications is root:admin with admin having
            full access. So, to work around we relax the hardening a bit and
            permit grand parents and beyond to be group writable by admin. */
         /** @todo dynamically resolve the admin group? */
         bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 80 /*admin*/ || suplibHardenedStrCmp(pszPath, "/Applications");
 
-#elif defined(RT_OS_FREEBSD)
+# elif defined(RT_OS_FREEBSD)
         /* HACK ALERT: PC-BSD 9 has group-writable /usr/pib directory which is
            similar to /Applications on OS X (see above).
            On FreeBSD root is normally the only member of this group, on
@@ -1485,17 +1498,17 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
         /** @todo dynamically resolve the operator group? */
         bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 5 /*operator*/ || suplibHardenedStrCmp(pszPath, "/usr/pbi");
         NOREF(fRelaxed);
-#elif defined(RT_OS_SOLARIS)
+# elif defined(RT_OS_SOLARIS)
         /* HACK ALERT: Solaris has group-writable /usr/lib/iconv directory from
            which the appropriate module is loaded.
            By default only root and daemon are part of that group.
            . */
         /** @todo dynamically resolve the bin group? */
         bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 2 /*bin*/ || suplibHardenedStrCmp(pszPath, "/usr/lib/iconv");
-#else
+# else
         NOREF(fRelaxed);
         bool fBad = true;
-#endif
+# endif
         if (fBad)
             return supR3HardenedSetError3(VERR_SUPLIB_WRITE_NON_SYS_GROUP, pErrInfo,
                                           "An unknown (and thus untrusted) group has write access to '", pszPath,
