@@ -1,10 +1,10 @@
 /* $Id: service.cpp $ */
 /** @file
- * Shared Clipboard: Host service entry points.
+ * Shared Clipboard Service - Host service entry points.
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -69,6 +69,11 @@
  * This section may be written in the future :)
  */
 
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
+#define LOG_GROUP LOG_GROUP_SHARED_CLIPBOARD
 #include <VBox/HostServices/VBoxClipboardSvc.h>
 #include <VBox/HostServices/VBoxClipboardExt.h>
 
@@ -76,10 +81,34 @@
 #include <iprt/string.h>
 #include <iprt/assert.h>
 #include <iprt/critsect.h>
+#include <VBox/err.h>
 #include <VBox/vmm/ssm.h>
 
 #include "VBoxClipboard.h"
 
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
+static PVBOXHGCMSVCHELPERS g_pHelpers;
+
+static RTCRITSECT critsect;
+static uint32_t g_u32Mode;
+
+static PFNHGCMSVCEXT g_pfnExtension;
+static void *g_pvExtension;
+
+static VBOXCLIPBOARDCLIENTDATA *g_pClient;
+
+/* Serialization of data reading and format announcements from the RDP client. */
+static bool g_fReadingData = false;
+static bool g_fDelayedAnnouncement = false;
+static uint32_t g_u32DelayedFormats = 0;
+
+/** Is the clipboard running in headless mode? */
+static bool g_fHeadless = false;
+
+
 static void VBoxHGCMParmUInt32Set (VBOXHGCMSVCPARM *pParm, uint32_t u32)
 {
     pParm->type = VBOX_HGCM_SVC_PARM_32BIT;
@@ -118,23 +147,6 @@ static int VBoxHGCMParmPtrGet (VBOXHGCMSVCPARM *pParm, void **ppv, uint32_t *pcb
     return VERR_INVALID_PARAMETER;
 }
 
-static PVBOXHGCMSVCHELPERS g_pHelpers;
-
-static RTCRITSECT critsect;
-static uint32_t g_u32Mode;
-
-static PFNHGCMSVCEXT g_pfnExtension;
-static void *g_pvExtension;
-
-static VBOXCLIPBOARDCLIENTDATA *g_pClient;
-
-/* Serialization of data reading and format announcements from the RDP client. */
-static bool g_fReadingData = false;
-static bool g_fDelayedAnnouncement = false;
-static uint32_t g_u32DelayedFormats = 0;
-
-/** Is the clipboard running in headless mode? */
-static bool g_fHeadless = false;
 
 static uint32_t vboxSvcClipboardMode (void)
 {
@@ -196,10 +208,22 @@ static bool vboxSvcClipboardReturnMsg (VBOXCLIPBOARDCLIENTDATA *pClient, VBOXHGC
     }
     else if (pClient->fMsgReadData)
     {
+        uint32_t fFormat = 0;
+
         LogRelFlow(("vboxSvcClipboardReturnMsg: ReadData %02X\n", pClient->u32RequestedFormat));
+        if (pClient->u32RequestedFormat & VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT)
+            fFormat = VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT;
+        else if (pClient->u32RequestedFormat & VBOX_SHARED_CLIPBOARD_FMT_BITMAP)
+            fFormat = VBOX_SHARED_CLIPBOARD_FMT_BITMAP;
+        else if (pClient->u32RequestedFormat & VBOX_SHARED_CLIPBOARD_FMT_HTML)
+            fFormat = VBOX_SHARED_CLIPBOARD_FMT_HTML;
+        else
+            AssertStmt(pClient->u32RequestedFormat == 0, pClient->u32RequestedFormat = 0);
+        pClient->u32RequestedFormat &= ~fFormat;
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA);
-        VBoxHGCMParmUInt32Set (&paParms[1], pClient->u32RequestedFormat);
-        pClient->fMsgReadData = false;
+        VBoxHGCMParmUInt32Set (&paParms[1], fFormat);
+        if (pClient->u32RequestedFormat == 0)
+            pClient->fMsgReadData = false;
     }
     else if (pClient->fMsgFormats)
     {
@@ -329,8 +353,12 @@ static DECLCALLBACK(int) svcDisconnect (void *, uint32_t u32ClientID, void *pvCl
 {
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
+    LogRel2(("svcDisconnect: u32ClientID = %d\n", u32ClientID));
+
     vboxSvcClipboardReportMsg (pClient, VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT, 0);
 
+    vboxSvcClipboardCompleteReadData(pClient, VERR_NO_DATA, 0);
+
     vboxClipboardDisconnect (pClient);
 
     memset (pClient, 0, sizeof (*pClient));
@@ -340,8 +368,9 @@ static DECLCALLBACK(int) svcDisconnect (void *, uint32_t u32ClientID, void *pvCl
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) svcConnect (void *, uint32_t u32ClientID, void *pvClient)
+static DECLCALLBACK(int) svcConnect (void *, uint32_t u32ClientID, void *pvClient, uint32_t fRequestor, bool fRestoring)
 {
+    RT_NOREF(fRequestor, fRestoring);
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
     int rc = VINF_SUCCESS;
@@ -379,8 +408,10 @@ static DECLCALLBACK(void) svcCall (void *,
                                    void *pvClient,
                                    uint32_t u32Function,
                                    uint32_t cParms,
-                                   VBOXHGCMSVCPARM paParms[])
+                                   VBOXHGCMSVCPARM paParms[],
+                                   uint64_t tsArrival)
 {
+    RT_NOREF_PV(tsArrival);
     int rc = VINF_SUCCESS;
 
     LogRel2(("svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n",
@@ -744,6 +775,7 @@ static DECLCALLBACK(int) svcHostCall (void *,
     return rc;
 }
 
+#ifndef UNIT_TEST
 /**
  * SSM descriptor table for the VBOXCLIPBOARDCLIENTDATA structure.
  */
@@ -756,20 +788,16 @@ static SSMFIELD const g_aClipboardClientDataFields[] =
     SSMFIELD_ENTRY(VBOXCLIPBOARDCLIENTDATA, u32RequestedFormat),
     SSMFIELD_ENTRY_TERM()
 };
+#endif
 
 static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
 #ifndef UNIT_TEST
-    /* If there are any pending requests, they must be completed here. Since
-     * the service is single threaded, there could be only requests
-     * which the service itself has postponed.
-     *
-     * HGCM knows that the state is being saved and that the pfnComplete
-     * calls are just clean ups. These requests are saved by the VMMDev.
-     *
-     * When the state will be restored, these requests will be reissued
+    /*
+     * When the state will be restored, pending requests will be reissued
      * by VMMDev. The service therefore must save state as if there were no
      * pending request.
+     * Pending requests, if any, will be completed in svcDisconnect.
      */
     LogRel2 (("svcSaveState: u32ClientID = %d\n", u32ClientID));
 
@@ -781,15 +809,9 @@ static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClie
     int rc = SSMR3PutStructEx (pSSM, pClient, sizeof(*pClient), 0 /*fFlags*/, &g_aClipboardClientDataFields[0], NULL);
     AssertRCReturn (rc, rc);
 
-    if (pClient->fAsync)
-    {
-        g_pHelpers->pfnCallComplete (pClient->async.callHandle, VINF_SUCCESS /* error code is not important here. */);
-        pClient->fAsync = false;
-    }
-
-    vboxSvcClipboardCompleteReadData (pClient, VINF_SUCCESS, 0);
-
-#endif /* !UNIT_TEST */
+#else  /* UNIT_TEST */
+    RT_NOREF3(u32ClientID, pvClient, pSSM);
+#endif /* UNIT_TEST */
     return VINF_SUCCESS;
 }
 
@@ -835,9 +857,10 @@ typedef struct CLIPSAVEDSTATEDATA
 
 } CLIPSAVEDSTATEDATA;
 
-static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
+static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM, uint32_t uVersion)
 {
 #ifndef UNIT_TEST
+    RT_NOREF(uVersion);
     LogRel2 (("svcLoadState: u32ClientID = %d\n", u32ClientID));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
@@ -858,7 +881,7 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
         rc = SSMR3GetStructEx (pSSM, pClient, sizeof(*pClient), 0 /*fFlags*/, &g_aClipboardClientDataFields[0], NULL);
         AssertRCReturn (rc, rc);
     }
-    else if (lenOrVer == (SSMR3HandleHostBits (pSSM) == 64 ? 72 : 48))
+    else if (lenOrVer == (SSMR3HandleHostBits (pSSM) == 64 ? 72U : 48U))
     {
         /**
          * SSM descriptor table for the CLIPSAVEDSTATEDATA structure.
@@ -869,7 +892,7 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
             SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, pPrev),
             SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, pCtx),
             SSMFIELD_ENTRY(                 CLIPSAVEDSTATEDATA, u32ClientID),
-            SSMFIELD_ENTRY_CUSTOM(fMsgQuit+fMsgReadData+fMsgFormats, RT_OFFSETOF(CLIPSAVEDSTATEDATA, u32ClientID) + 4, 4),
+            SSMFIELD_ENTRY_CUSTOM(fMsgQuit + fMsgReadData + fMsgFormats, RT_UOFFSETOF(CLIPSAVEDSTATEDATA, u32ClientID) + 4, 4),
             SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, async.callHandle),
             SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, async.paParms),
             SSMFIELD_ENTRY_IGNORE(          CLIPSAVEDSTATEDATA, data.pv),
@@ -908,12 +931,15 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
     /* Actual host data are to be reported to guest (SYNC). */
     vboxClipboardSync (pClient);
 
-#endif /* !UNIT_TEST */
+#else  /* UNIT_TEST*/
+    RT_NOREF(u32ClientID, pvClient, pSSM, uVersion);
+#endif /* UNIT_TEST */
     return VINF_SUCCESS;
 }
 
 static DECLCALLBACK(int) extCallback (uint32_t u32Function, uint32_t u32Format, void *pvData, uint32_t cbData)
 {
+    RT_NOREF2(pvData, cbData);
     if (g_pClient != NULL)
     {
         switch (u32Function)
@@ -1009,6 +1035,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
             ptable->pfnSaveState  = svcSaveState;
             ptable->pfnLoadState  = svcLoadState;
             ptable->pfnRegisterExtension  = svcRegisterExtension;
+            ptable->pfnNotify     = NULL;
             ptable->pvService     = NULL;
 
             /* Service specific initialization. */
