@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ____H_SESSIONIMPL
-#define ____H_SESSIONIMPL
+#ifndef MAIN_INCLUDED_SessionImpl_h
+#define MAIN_INCLUDED_SessionImpl_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include "SessionWrap.h"
 #include "ConsoleImpl.h"
@@ -31,14 +34,15 @@
 class ATL_NO_VTABLE Session :
     public SessionWrap
 #ifdef RT_OS_WINDOWS
-    , public CComCoClass<Session, &CLSID_Session>
+    , public ATL::CComCoClass<Session, &CLSID_Session>
 #endif
 {
 public:
 
     DECLARE_CLASSFACTORY()
 
-    DECLARE_REGISTRY_RESOURCEID(IDR_VIRTUALBOX)
+    // Do not use any ATL registry support.
+    //DECLARE_REGISTRY_RESOURCEID(IDR_VIRTUALBOX)
 
     DECLARE_NOT_AGGREGATABLE(Session)
 
@@ -85,6 +89,7 @@ private:
     HRESULT uninitialize();
     HRESULT onNetworkAdapterChange(const ComPtr<INetworkAdapter> &aNetworkAdapter,
                                    BOOL aChangeAdapter);
+    HRESULT onAudioAdapterChange(const ComPtr<IAudioAdapter> &aAudioAdapter);
     HRESULT onSerialPortChange(const ComPtr<ISerialPort> &aSerialPort);
     HRESULT onParallelPortChange(const ComPtr<IParallelPort> &aParallelPort);
     HRESULT onStorageControllerChange();
@@ -99,7 +104,7 @@ private:
                         BOOL aAdd);
     HRESULT onCPUExecutionCapChange(ULONG aExecutionCap);
     HRESULT onVRDEServerChange(BOOL aRestart);
-    HRESULT onVideoCaptureChange();
+    HRESULT onRecordingChange(BOOL aEnable);
     HRESULT onUSBControllerChange();
     HRESULT onSharedFolderChange(BOOL aGlobal);
     HRESULT onUSBDeviceAttach(const ComPtr<IUSBDevice> &aDevice,
@@ -134,6 +139,7 @@ private:
     HRESULT resumeWithReason(Reason_T aReason);
     HRESULT saveStateWithReason(Reason_T aReason,
                                 const ComPtr<IProgress> &aProgress,
+                                const ComPtr<ISnapshot> &aSnapshot,
                                 const Utf8Str &aStateFilePath,
                                 BOOL aPauseVM,
                                 BOOL *aLeftPaused);
@@ -162,5 +168,5 @@ private:
     ClientTokenHolder *mClientTokenHolder;
 };
 
-#endif // !____H_SESSIONIMPL
+#endif /* !MAIN_INCLUDED_SessionImpl_h */
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
