@@ -1,11 +1,10 @@
 /* $Id: GuestDnDPrivate.cpp $ */
 /** @file
- * Private guest drag and drop code, used by GuestDnDTarget +
- * GuestDnDSource.
+ * Private guest drag and drop code, used by GuestDnDTarget + GuestDnDSource.
  */
 
 /*
- * Copyright (C) 2011-2016 Oracle Corporation
+ * Copyright (C) 2011-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,6 +15,9 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#define LOG_GROUP LOG_GROUP_GUEST_DND
+#include "LoggingNew.h"
+
 #include "GuestImpl.h"
 #include "AutoCaller.h"
 
@@ -38,13 +40,7 @@
 # include <VBox/HostServices/DragAndDropSvc.h>
 # include <VBox/version.h>
 
-# ifdef LOG_GROUP
-#  undef LOG_GROUP
-# endif
-# define LOG_GROUP LOG_GROUP_GUEST_DND
-# include <VBox/log.h>
-
-/**
+/** @page pg_main_dnd  Dungeons & Dragons - Overview
  * Overview:
  *
  * Drag and Drop is handled over the internal HGCM service for the host <->
@@ -191,8 +187,8 @@ int GuestDnDCallbackEvent::Wait(RTMSINTERVAL msTimeout)
 
 GuestDnDResponse::GuestDnDResponse(const ComObjPtr<Guest>& pGuest)
     : m_EventSem(NIL_RTSEMEVENT)
-    , m_defAction(0)
-    , m_allActions(0)
+    , m_dndActionDefault(0)
+    , m_dndLstActionsAllowed(0)
     , m_pParent(pGuest)
 {
     int rc = RTSemEventCreate(&m_EventSem);
@@ -217,8 +213,8 @@ void GuestDnDResponse::reset(void)
 {
     LogFlowThisFuncEnter();
 
-    m_defAction  = 0;
-    m_allActions = 0;
+    m_dndActionDefault     = 0;
+    m_dndLstActionsAllowed = 0;
 
     m_lstFormats.clear();
 }
@@ -280,6 +276,7 @@ int GuestDnDResponse::setProgress(unsigned uPercentage,
                                   uint32_t uStatus,
                                   int rcOp /* = VINF_SUCCESS */, const Utf8Str &strMsg /* = "" */)
 {
+    RT_NOREF(rcOp);
     LogFlowFunc(("uStatus=%RU32, uPercentage=%RU32, rcOp=%Rrc, strMsg=%s\n",
                  uStatus, uPercentage, rcOp, strMsg.c_str()));
 
@@ -386,7 +383,7 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
             AssertReturn(sizeof(DragAndDropSvc::VBOXDNDCBHGACKOPDATA) == cbParms, VERR_INVALID_PARAMETER);
             AssertReturn(DragAndDropSvc::CB_MAGIC_DND_HG_ACK_OP == pCBData->hdr.uMagic, VERR_INVALID_PARAMETER);
 
-            setDefAction(pCBData->uAction);
+            setActionDefault(pCBData->uAction);
             rc = notifyAboutGuestResponse();
             break;
         }
@@ -455,8 +452,8 @@ int GuestDnDResponse::onDispatch(uint32_t u32Function, void *pvParms, uint32_t c
             else
             {
                 setFormats   (GuestDnD::toFormatList(pCBData->pszFormat));
-                setDefAction (pCBData->uDefAction);
-                setAllActions(pCBData->uAllActions);
+                setActionDefault (pCBData->uDefAction);
+                setActionsAllowed(pCBData->uAllActions);
 
                 rc = VINF_SUCCESS;
             }
@@ -673,16 +670,16 @@ GuestDnDMIMEList GuestDnD::toFilteredFormatList(const GuestDnDMIMEList &lstForma
 }
 
 /* static */
-uint32_t GuestDnD::toHGCMAction(DnDAction_T enmAction)
+VBOXDNDACTION GuestDnD::toHGCMAction(DnDAction_T enmAction)
 {
-    uint32_t uAction = DND_IGNORE_ACTION;
+    VBOXDNDACTION dndAction = VBOX_DND_ACTION_IGNORE;
     switch (enmAction)
     {
         case DnDAction_Copy:
-            uAction = DND_COPY_ACTION;
+            dndAction = VBOX_DND_ACTION_COPY;
             break;
         case DnDAction_Move:
-            uAction = DND_MOVE_ACTION;
+            dndAction = VBOX_DND_ACTION_MOVE;
             break;
         case DnDAction_Link:
             /* For now it doesn't seems useful to allow a link
@@ -695,64 +692,64 @@ uint32_t GuestDnD::toHGCMAction(DnDAction_T enmAction)
             break;
     }
 
-    return uAction;
+    return dndAction;
 }
 
 /* static */
-void GuestDnD::toHGCMActions(DnDAction_T                    enmDefAction,
-                             uint32_t                      *puDefAction,
-                             const std::vector<DnDAction_T> vecAllowedActions,
-                             uint32_t                      *puAllowedActions)
+void GuestDnD::toHGCMActions(DnDAction_T                    enmDnDActionDefault,
+                             VBOXDNDACTION                 *pDnDActionDefault,
+                             const std::vector<DnDAction_T> vecDnDActionsAllowed,
+                             VBOXDNDACTIONLIST             *pDnDLstActionsAllowed)
 {
-    uint32_t uAllowedActions = DND_IGNORE_ACTION;
-    uint32_t uDefAction      = toHGCMAction(enmDefAction);
+    VBOXDNDACTIONLIST dndLstActionsAllowed = VBOX_DND_ACTION_IGNORE;
+    VBOXDNDACTION     dndActionDefault     = toHGCMAction(enmDnDActionDefault);
 
-    if (!vecAllowedActions.empty())
+    if (!vecDnDActionsAllowed.empty())
     {
         /* First convert the allowed actions to a bit array. */
-        for (size_t i = 0; i < vecAllowedActions.size(); i++)
-            uAllowedActions |= toHGCMAction(vecAllowedActions[i]);
+        for (size_t i = 0; i < vecDnDActionsAllowed.size(); i++)
+            dndLstActionsAllowed |= toHGCMAction(vecDnDActionsAllowed[i]);
 
         /*
          * If no default action is set (ignoring), try one of the
          * set allowed actions, preferring copy, move (in that order).
          */
-        if (isDnDIgnoreAction(uDefAction))
+        if (isDnDIgnoreAction(dndActionDefault))
         {
-            if (hasDnDCopyAction(uAllowedActions))
-                uDefAction = DND_COPY_ACTION;
-            else if (hasDnDMoveAction(uAllowedActions))
-                uDefAction = DND_MOVE_ACTION;
+            if (hasDnDCopyAction(dndLstActionsAllowed))
+                dndActionDefault = VBOX_DND_ACTION_COPY;
+            else if (hasDnDMoveAction(dndLstActionsAllowed))
+                dndActionDefault = VBOX_DND_ACTION_MOVE;
         }
     }
 
-    if (puDefAction)
-        *puDefAction      = uDefAction;
-    if (puAllowedActions)
-        *puAllowedActions = uAllowedActions;
+    if (pDnDActionDefault)
+        *pDnDActionDefault     = dndActionDefault;
+    if (pDnDLstActionsAllowed)
+        *pDnDLstActionsAllowed = dndLstActionsAllowed;
 }
 
 /* static */
-DnDAction_T GuestDnD::toMainAction(uint32_t uAction)
+DnDAction_T GuestDnD::toMainAction(VBOXDNDACTION dndAction)
 {
     /* For now it doesn't seems useful to allow a
      * link action between host & guest. Maybe later! */
-    return (isDnDCopyAction(uAction) ? (DnDAction_T)DnDAction_Copy :
-            isDnDMoveAction(uAction) ? (DnDAction_T)DnDAction_Move :
-            (DnDAction_T)DnDAction_Ignore);
+    return isDnDCopyAction(dndAction) ? DnDAction_Copy
+         : isDnDMoveAction(dndAction) ? DnDAction_Move
+         :                              DnDAction_Ignore;
 }
 
 /* static */
-std::vector<DnDAction_T> GuestDnD::toMainActions(uint32_t uActions)
+std::vector<DnDAction_T> GuestDnD::toMainActions(VBOXDNDACTIONLIST dndActionList)
 {
     std::vector<DnDAction_T> vecActions;
 
     /* For now it doesn't seems useful to allow a
      * link action between host & guest. Maybe later! */
     RTCList<DnDAction_T> lstActions;
-    if (hasDnDCopyAction(uActions))
+    if (hasDnDCopyAction(dndActionList))
         lstActions.append(DnDAction_Copy);
-    if (hasDnDMoveAction(uActions))
+    if (hasDnDMoveAction(dndActionList))
         lstActions.append(DnDAction_Move);
 
     for (size_t i = 0; i < lstActions.size(); ++i)
@@ -839,37 +836,35 @@ int GuestDnDBase::getProtocolVersion(uint32_t *puProto)
 
     int rc;
 
-    uint32_t uProto        = 0;
-    uint32_t uVerAdditions = 0;
-    uint32_t uRevAdditions = 0;
+    uint32_t uProto = 0;
+    uint32_t uVerAdditions;
+    uint32_t uRevAdditions;
     if (   m_pGuest
         && (uVerAdditions = m_pGuest->i_getAdditionsVersion())  > 0
         && (uRevAdditions = m_pGuest->i_getAdditionsRevision()) > 0)
     {
-#ifdef DEBUG
-# if 0
+#if 0 && defined(DEBUG)
         /* Hardcode the to-used protocol version; nice for testing side effects. */
-        uProto = 3;
-        rc = VINF_SUCCESS;
-# endif
+        if (true)
+            uProto = 3;
+        else
 #endif
-        if (!uProto) /* Protocol not set yet? */
+        if (uVerAdditions >= VBOX_FULL_VERSION_MAKE(5, 0, 0))
         {
-            if (uVerAdditions >= VBOX_FULL_VERSION_MAKE(5, 0, 0))
-            {
-                if (uRevAdditions >= 103344) /* Since r103344: Protocol v3. */
-                {
-                    uProto = 3;
-                }
-                else
-                    uProto = 2; /* VBox 5.0.0 - 5.0.8: Protocol v2. */
-            }
-
-            LogFlowFunc(("uVerAdditions=%RU32 (%RU32.%RU32.%RU32), r%RU32\n",
-                         uVerAdditions, VBOX_FULL_VERSION_GET_MAJOR(uVerAdditions), VBOX_FULL_VERSION_GET_MINOR(uVerAdditions),
-                                        VBOX_FULL_VERSION_GET_BUILD(uVerAdditions), uRevAdditions));
-            rc = VINF_SUCCESS;
+/** @todo
+ *  r=bird: This is just too bad for anyone using an OSE additions build...
+ */
+            if (uRevAdditions >= 103344) /* Since r103344: Protocol v3. */
+                uProto = 3;
+            else
+                uProto = 2; /* VBox 5.0.0 - 5.0.8: Protocol v2. */
         }
+        /* else: uProto: 0 */
+
+        LogFlowFunc(("uVerAdditions=%RU32 (%RU32.%RU32.%RU32), r%RU32\n",
+                     uVerAdditions, VBOX_FULL_VERSION_GET_MAJOR(uVerAdditions), VBOX_FULL_VERSION_GET_MINOR(uVerAdditions),
+                                    VBOX_FULL_VERSION_GET_BUILD(uVerAdditions), uRevAdditions));
+        rc = VINF_SUCCESS;
     }
     else
     {
