@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2016 Oracle Corporation
+ * Copyright (C) 2012-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,6 +19,9 @@
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
+#define LOG_GROUP LOG_GROUP_MAIN_GUESTFILE
+#include "LoggingNew.h"
+
 #ifndef VBOX_WITH_GUEST_CONTROL
 # error "VBOX_WITH_GUEST_CONTROL must defined in this file"
 #endif
@@ -38,12 +41,6 @@
 #include <VBox/com/array.h>
 #include <VBox/com/listeners.h>
 
-#ifdef LOG_GROUP
- #undef LOG_GROUP
-#endif
-#define LOG_GROUP LOG_GROUP_GUEST_CONTROL
-#include <VBox/log.h>
-
 
 /**
  * Internal listener class to serve events in an
@@ -137,14 +134,14 @@ void GuestFile::FinalRelease(void)
  * @return  IPRT status code.
  * @param   pConsole                Pointer to console object.
  * @param   pSession                Pointer to session object.
- * @param   uFileID                 Host-based file ID (part of the context ID).
+ * @param   aObjectID               The object's ID.
  * @param   openInfo                File opening information.
  */
 int GuestFile::init(Console *pConsole, GuestSession *pSession,
-                    ULONG uFileID, const GuestFileOpenInfo &openInfo)
+                    ULONG aObjectID, const GuestFileOpenInfo &openInfo)
 {
-    LogFlowThisFunc(("pConsole=%p, pSession=%p, uFileID=%RU32, strPath=%s\n",
-                     pConsole, pSession, uFileID, openInfo.mFileName.c_str()));
+    LogFlowThisFunc(("pConsole=%p, pSession=%p, aObjectID=%RU32, strPath=%s\n",
+                     pConsole, pSession, aObjectID, openInfo.mFilename.c_str()));
 
     AssertPtrReturn(pConsole, VERR_INVALID_POINTER);
     AssertPtrReturn(pSession, VERR_INVALID_POINTER);
@@ -153,12 +150,11 @@ int GuestFile::init(Console *pConsole, GuestSession *pSession,
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), VERR_OBJECT_DESTROYED);
 
-    int vrc = bindToSession(pConsole, pSession, uFileID /* Object ID */);
+    int vrc = bindToSession(pConsole, pSession, aObjectID);
     if (RT_SUCCESS(vrc))
     {
         mSession = pSession;
 
-        mData.mID = uFileID;
         mData.mInitialSize = 0;
         mData.mStatus = FileStatus_Undefined;
         mData.mOpenInfo = openInfo;
@@ -267,11 +263,11 @@ HRESULT GuestFile::getEventSource(ComPtr<IEventSource> &aEventSource)
     return S_OK;
 }
 
-HRESULT GuestFile::getFileName(com::Utf8Str &aFileName)
+HRESULT GuestFile::getFilename(com::Utf8Str &aFilename)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    aFileName = mData.mOpenInfo.mFileName;
+    aFilename = mData.mOpenInfo.mFilename;
 
     return S_OK;
 }
@@ -280,7 +276,7 @@ HRESULT GuestFile::getId(ULONG *aId)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *aId = mData.mID;
+    *aId = mObjectID;
 
     return S_OK;
 }
@@ -298,6 +294,10 @@ HRESULT GuestFile::getOffset(LONG64 *aOffset)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
+/** @todo r=bird: Why do you have both a offset and a tell() function?
+ * After a ReadAt or WriteAt with a non-current offset, the tell() result will
+ * differ from this value, because mOffCurrent is only ever incremented with
+ * data read or written.  */
     *aOffset = mData.mOffCurrent;
 
     return S_OK;
@@ -332,7 +332,7 @@ int GuestFile::i_callbackDispatcher(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCT
     AssertPtrReturn(pSvcCb, VERR_INVALID_POINTER);
 
     LogFlowThisFunc(("strName=%s, uContextID=%RU32, uFunction=%RU32, pSvcCb=%p\n",
-                     mData.mOpenInfo.mFileName.c_str(), pCbCtx->uContextID, pCbCtx->uFunction, pSvcCb));
+                     mData.mOpenInfo.mFilename.c_str(), pCbCtx->uContextID, pCbCtx->uFunction, pSvcCb));
 
     int vrc;
     switch (pCbCtx->uFunction)
@@ -357,9 +357,9 @@ int GuestFile::i_callbackDispatcher(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCT
     return vrc;
 }
 
-int GuestFile::i_closeFile(int *pGuestRc)
+int GuestFile::i_closeFile(int *prcGuest)
 {
-    LogFlowThisFunc(("strFile=%s\n", mData.mOpenInfo.mFileName.c_str()));
+    LogFlowThisFunc(("strFile=%s\n", mData.mOpenInfo.mFilename.c_str()));
 
     int vrc;
 
@@ -371,7 +371,7 @@ int GuestFile::i_closeFile(int *pGuestRc)
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -382,13 +382,13 @@ int GuestFile::i_closeFile(int *pGuestRc)
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[4];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setUInt32(mData.mID /* Guest file ID */);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetU32(&paParms[i++], mObjectID /* Guest file ID */);
 
     vrc = sendCommand(HOST_FILE_CLOSE, i, paParms);
     if (RT_SUCCESS(vrc))
         vrc = i_waitForStatusChange(pEvent, 30 * 1000 /* Timeout in ms */,
-                                    NULL /* FileStatus */, pGuestRc);
+                                    NULL /* FileStatus */, prcGuest);
     unregisterWaitEvent(pEvent);
 
     LogFlowFuncLeaveRC(vrc);
@@ -396,13 +396,17 @@ int GuestFile::i_closeFile(int *pGuestRc)
 }
 
 /* static */
-Utf8Str GuestFile::i_guestErrorToString(int guestRc)
+Utf8Str GuestFile::i_guestErrorToString(int rcGuest)
 {
     Utf8Str strError;
 
     /** @todo pData->u32Flags: int vs. uint32 -- IPRT errors are *negative* !!! */
-    switch (guestRc)
+    switch (rcGuest)
     {
+        case VERR_ACCESS_DENIED:
+            strError += Utf8StrFmt(tr("Access denied"));
+            break;
+
         case VERR_ALREADY_EXISTS:
             strError += Utf8StrFmt(tr("File already exists"));
             break;
@@ -420,7 +424,7 @@ Utf8Str GuestFile::i_guestErrorToString(int guestRc)
             break;
 
         default:
-            strError += Utf8StrFmt("%Rrc", guestRc);
+            strError += Utf8StrFmt("%Rrc", rcGuest);
             break;
     }
 
@@ -437,38 +441,37 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
     if (pSvcCbData->mParms < 3)
         return VERR_INVALID_PARAMETER;
 
-    int vrc = VINF_SUCCESS;
-
     int idx = 1; /* Current parameter index. */
     CALLBACKDATA_FILE_NOTIFY dataCb;
     /* pSvcCb->mpaParms[0] always contains the context ID. */
-    pSvcCbData->mpaParms[idx++].getUInt32(&dataCb.uType);
-    pSvcCbData->mpaParms[idx++].getUInt32(&dataCb.rc);
+    HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.uType);
+    HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.rc);
 
-    int guestRc = (int)dataCb.rc; /* uint32_t vs. int. */
+    int rcGuest = (int)dataCb.rc; /* uint32_t vs. int. */
 
-    LogFlowFunc(("uType=%RU32, guestRc=%Rrc\n",
-                 dataCb.uType, guestRc));
+    LogFlowThisFunc(("uType=%RU32, rcGuest=%Rrc\n", dataCb.uType, rcGuest));
 
-    if (RT_FAILURE(guestRc))
+    if (RT_FAILURE(rcGuest))
     {
-        int rc2 = i_setFileStatus(FileStatus_Error, guestRc);
-        AssertRC(rc2);
-
-        rc2 = signalWaitEventInternal(pCbCtx,
-                                      guestRc, NULL /* pPayload */);
+        int rc2 = i_setFileStatus(FileStatus_Error, rcGuest);
         AssertRC(rc2);
 
+        /* Ignore rc, as the event to signal might not be there (anymore). */
+        signalWaitEventInternal(pCbCtx, rcGuest, NULL /* pPayload */);
         return VINF_SUCCESS; /* Report to the guest. */
     }
 
+    AssertMsg(mObjectID == VBOX_GUESTCTRL_CONTEXTID_GET_OBJECT(pCbCtx->uContextID),
+              ("File ID %RU32 does not match object ID %RU32\n", mObjectID,
+               VBOX_GUESTCTRL_CONTEXTID_GET_OBJECT(pCbCtx->uContextID)));
+
+    int rc = VERR_NOT_SUPPORTED; /* Play safe by default. */
+
     switch (dataCb.uType)
     {
         case GUEST_FILE_NOTIFYTYPE_ERROR:
         {
-            int rc2 = i_setFileStatus(FileStatus_Error, guestRc);
-            AssertRC(rc2);
-
+            rc = i_setFileStatus(FileStatus_Error, rcGuest);
             break;
         }
 
@@ -476,27 +479,19 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
         {
             if (pSvcCbData->mParms == 4)
             {
-                pSvcCbData->mpaParms[idx++].getUInt32(&dataCb.u.open.uHandle);
-
-                AssertMsg(mData.mID == VBOX_GUESTCTRL_CONTEXTID_GET_OBJECT(pCbCtx->uContextID),
-                          ("File ID %RU32 does not match context ID %RU32\n", mData.mID,
-                           VBOX_GUESTCTRL_CONTEXTID_GET_OBJECT(pCbCtx->uContextID)));
+                rc = HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.u.open.uHandle);
+                if (RT_FAILURE(rc))
+                    break;
 
                 /* Set the process status. */
-                int rc2 = i_setFileStatus(FileStatus_Open, guestRc);
-                AssertRC(rc2);
+                rc = i_setFileStatus(FileStatus_Open, rcGuest);
             }
-            else
-                vrc = VERR_NOT_SUPPORTED;
-
             break;
         }
 
         case GUEST_FILE_NOTIFYTYPE_CLOSE:
         {
-            int rc2 = i_setFileStatus(FileStatus_Closed, guestRc);
-            AssertRC(rc2);
-
+            rc = i_setFileStatus(FileStatus_Closed, rcGuest);
             break;
         }
 
@@ -504,9 +499,14 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
         {
             if (pSvcCbData->mParms == 4)
             {
-                pSvcCbData->mpaParms[idx++].getPointer(&dataCb.u.read.pvData,
-                                                       &dataCb.u.read.cbData);
-                uint32_t cbRead = dataCb.u.read.cbData;
+                rc = HGCMSvcGetPv(&pSvcCbData->mpaParms[idx++], &dataCb.u.read.pvData,
+                                  &dataCb.u.read.cbData);
+                if (RT_FAILURE(rc))
+                    break;
+
+                const uint32_t cbRead = dataCb.u.read.cbData;
+
+                Log3ThisFunc(("cbRead=%RU32\n", cbRead));
 
                 AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -520,8 +520,6 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
                 fireGuestFileReadEvent(mEventSource, mSession, this, mData.mOffCurrent,
                                        cbRead, ComSafeArrayAsInParam(data));
             }
-            else
-                vrc = VERR_NOT_SUPPORTED;
             break;
         }
 
@@ -529,20 +527,21 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
         {
             if (pSvcCbData->mParms == 4)
             {
-                pSvcCbData->mpaParms[idx++].getUInt32(&dataCb.u.write.cbWritten);
+                rc = HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.u.write.cbWritten);
+                if (RT_FAILURE(rc))
+                    break;
+
+                Log3ThisFunc(("cbWritten=%RU32\n", dataCb.u.write.cbWritten));
 
                 AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
                 mData.mOffCurrent += dataCb.u.write.cbWritten;
-                uint64_t uOffCurrent = mData.mOffCurrent;
 
                 alock.release();
 
-                fireGuestFileWriteEvent(mEventSource, mSession, this, uOffCurrent,
+                fireGuestFileWriteEvent(mEventSource, mSession, this, mData.mOffCurrent,
                                         dataCb.u.write.cbWritten);
             }
-            else
-                vrc = VERR_NOT_SUPPORTED;
             break;
         }
 
@@ -550,7 +549,11 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
         {
             if (pSvcCbData->mParms == 4)
             {
-                pSvcCbData->mpaParms[idx++].getUInt64(&dataCb.u.seek.uOffActual);
+                rc = HGCMSvcGetU64(&pSvcCbData->mpaParms[idx++], &dataCb.u.seek.uOffActual);
+                if (RT_FAILURE(rc))
+                    break;
+
+                Log3ThisFunc(("uOffActual=%RU64\n", dataCb.u.seek.uOffActual));
 
                 AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -558,11 +561,8 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
 
                 alock.release();
 
-                fireGuestFileOffsetChangedEvent(mEventSource, mSession, this,
-                                                dataCb.u.seek.uOffActual, 0 /* Processed */);
+                fireGuestFileOffsetChangedEvent(mEventSource, mSession, this, mData.mOffCurrent, 0 /* Processed */);
             }
-            else
-                vrc = VERR_NOT_SUPPORTED;
             break;
         }
 
@@ -570,7 +570,11 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
         {
             if (pSvcCbData->mParms == 4)
             {
-                pSvcCbData->mpaParms[idx++].getUInt64(&dataCb.u.tell.uOffActual);
+                rc = HGCMSvcGetU64(&pSvcCbData->mpaParms[idx++], &dataCb.u.tell.uOffActual);
+                if (RT_FAILURE(rc))
+                    break;
+
+                Log3ThisFunc(("uOffActual=%RU64\n", dataCb.u.tell.uOffActual));
 
                 AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -578,31 +582,25 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
 
                 alock.release();
 
-                fireGuestFileOffsetChangedEvent(mEventSource, mSession, this,
-                                                dataCb.u.tell.uOffActual, 0 /* Processed */);
+                fireGuestFileOffsetChangedEvent(mEventSource, mSession, this, mData.mOffCurrent, 0 /* Processed */);
             }
-            else
-                vrc = VERR_NOT_SUPPORTED;
             break;
         }
 
         default:
-            vrc = VERR_NOT_SUPPORTED;
             break;
     }
 
-    if (RT_SUCCESS(vrc))
+    if (RT_SUCCESS(rc))
     {
         GuestWaitEventPayload payload(dataCb.uType, &dataCb, sizeof(dataCb));
-        int rc2 = signalWaitEventInternal(pCbCtx, guestRc, &payload);
-        AssertRC(rc2);
+
+        /* Ignore rc, as the event to signal might not be there (anymore). */
+        signalWaitEventInternal(pCbCtx, rcGuest, &payload);
     }
 
-    LogFlowThisFunc(("uType=%RU32, guestRc=%Rrc\n",
-                     dataCb.uType, dataCb.rc));
-
-    LogFlowFuncLeaveRC(vrc);
-    return vrc;
+    LogFlowThisFunc(("uType=%RU32, rcGuest=%Rrc, rc=%Rrc\n", dataCb.uType, rcGuest, rc));
+    return rc;
 }
 
 int GuestFile::i_onGuestDisconnected(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOSTCALLBACK pSvcCbData)
@@ -644,16 +642,60 @@ int GuestFile::i_onRemove(void)
     return vrc;
 }
 
-int GuestFile::i_openFile(uint32_t uTimeoutMS, int *pGuestRc)
+int GuestFile::i_openFile(uint32_t uTimeoutMS, int *prcGuest)
 {
+    AssertReturn(mData.mOpenInfo.mFilename.isNotEmpty(), VERR_INVALID_PARAMETER);
+
     LogFlowThisFuncEnter();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    LogFlowThisFunc(("strFile=%s, enmAccessMode=%d (%s) enmOpenAction=%d (%s) uCreationMode=%RU32, mfOpenEx=%RU32\n",
-                     mData.mOpenInfo.mFileName.c_str(), mData.mOpenInfo.mAccessMode, mData.mOpenInfo.mpszAccessMode,
-                     mData.mOpenInfo.mOpenAction, mData.mOpenInfo.mpszOpenAction, mData.mOpenInfo.mCreationMode,
-                     mData.mOpenInfo.mfOpenEx));
+    LogFlowThisFunc(("strFile=%s, enmAccessMode=0x%x, enmOpenAction=0x%x, uCreationMode=%RU32, mfOpenEx=%RU32\n",
+                     mData.mOpenInfo.mFilename.c_str(), mData.mOpenInfo.mAccessMode, mData.mOpenInfo.mOpenAction,
+                     mData.mOpenInfo.mCreationMode, mData.mOpenInfo.mfOpenEx));
+
+    /* Validate and translate open action. */
+    const char *pszOpenAction = NULL;
+    switch (mData.mOpenInfo.mOpenAction)
+    {
+        case FileOpenAction_OpenExisting:          pszOpenAction = "oe"; break;
+        case FileOpenAction_OpenOrCreate:          pszOpenAction = "oc"; break;
+        case FileOpenAction_CreateNew:             pszOpenAction = "ce"; break;
+        case FileOpenAction_CreateOrReplace:       pszOpenAction = "ca"; break;
+        case FileOpenAction_OpenExistingTruncated: pszOpenAction = "ot"; break;
+        case FileOpenAction_AppendOrCreate:
+            pszOpenAction = "oa"; /** @todo get rid of this one and implement AppendOnly/AppendRead. */
+            break;
+        default:
+            return VERR_INVALID_PARAMETER;
+    }
+
+    /* Validate and translate access mode. */
+    const char *pszAccessMode = NULL;
+    switch (mData.mOpenInfo.mAccessMode)
+    {
+        case FileAccessMode_ReadOnly:   pszAccessMode = "r";  break;
+        case FileAccessMode_WriteOnly:  pszAccessMode = "w";  break;
+        case FileAccessMode_ReadWrite:  pszAccessMode = "r+"; break;
+        case FileAccessMode_AppendOnly: RT_FALL_THRU();
+        case FileAccessMode_AppendRead: return VERR_NOT_IMPLEMENTED;
+        default:                        return VERR_INVALID_PARAMETER;
+    }
+
+    /* Validate and translate sharing mode. */
+    const char *pszSharingMode = NULL;
+    switch (mData.mOpenInfo.mSharingMode)
+    {
+        case FileSharingMode_All:           pszSharingMode = ""; break;
+        case FileSharingMode_Read:          RT_FALL_THRU();
+        case FileSharingMode_Write:         RT_FALL_THRU();
+        case FileSharingMode_ReadWrite:     RT_FALL_THRU();
+        case FileSharingMode_Delete:        RT_FALL_THRU();
+        case FileSharingMode_ReadDelete:    RT_FALL_THRU();
+        case FileSharingMode_WriteDelete:   return VERR_NOT_IMPLEMENTED;
+        default:                            return VERR_INVALID_PARAMETER;
+    }
+
     int vrc;
 
     GuestWaitEvent *pEvent = NULL;
@@ -664,7 +706,7 @@ int GuestFile::i_openFile(uint32_t uTimeoutMS, int *pGuestRc)
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -675,22 +717,21 @@ int GuestFile::i_openFile(uint32_t uTimeoutMS, int *pGuestRc)
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[8];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setPointer((void*)mData.mOpenInfo.mFileName.c_str(),
-                            (ULONG)mData.mOpenInfo.mFileName.length() + 1);
-    paParms[i++].setString(mData.mOpenInfo.mpszAccessMode);
-    paParms[i++].setString(mData.mOpenInfo.mpszOpenAction);
-    paParms[i++].setString(""); /** @todo sharing mode. */
-    paParms[i++].setUInt32(mData.mOpenInfo.mCreationMode);
-    paParms[i++].setUInt64(0 /* initial offset */);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetPv(&paParms[i++], (void*)mData.mOpenInfo.mFilename.c_str(),
+                 (ULONG)mData.mOpenInfo.mFilename.length() + 1);
+    HGCMSvcSetStr(&paParms[i++], pszAccessMode);
+    HGCMSvcSetStr(&paParms[i++], pszOpenAction);
+    HGCMSvcSetStr(&paParms[i++], pszSharingMode);
+    HGCMSvcSetU32(&paParms[i++], mData.mOpenInfo.mCreationMode);
+    HGCMSvcSetU64(&paParms[i++], mData.mOpenInfo.muOffset);
     /** @todo Next protocol version: add flags, replace strings, remove initial offset. */
 
     alock.release(); /* Drop write lock before sending. */
 
     vrc = sendCommand(HOST_FILE_OPEN, i, paParms);
     if (RT_SUCCESS(vrc))
-        vrc = i_waitForStatusChange(pEvent, uTimeoutMS,
-                                    NULL /* FileStatus */, pGuestRc);
+        vrc = i_waitForStatusChange(pEvent, uTimeoutMS, NULL /* FileStatus */, prcGuest);
 
     unregisterWaitEvent(pEvent);
 
@@ -698,6 +739,12 @@ int GuestFile::i_openFile(uint32_t uTimeoutMS, int *pGuestRc)
     return vrc;
 }
 
+int GuestFile::i_queryInfo(GuestFsObjData &objData, int *prcGuest)
+{
+    AssertPtr(mSession);
+    return mSession->i_fsQueryInfo(mData.mOpenInfo.mFilename, FALSE /* fFollowSymlinks */, objData, prcGuest);
+}
+
 int GuestFile::i_readData(uint32_t uSize, uint32_t uTimeoutMS,
                           void* pvData, uint32_t cbData, uint32_t* pcbRead)
 {
@@ -720,7 +767,7 @@ int GuestFile::i_readData(uint32_t uSize, uint32_t uTimeoutMS,
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -731,9 +778,9 @@ int GuestFile::i_readData(uint32_t uSize, uint32_t uTimeoutMS,
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[4];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setUInt32(mData.mID /* File handle */);
-    paParms[i++].setUInt32(uSize /* Size (in bytes) to read */);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetU32(&paParms[i++], mObjectID /* File handle */);
+    HGCMSvcSetU32(&paParms[i++], uSize /* Size (in bytes) to read */);
 
     alock.release(); /* Drop write lock before sending. */
 
@@ -748,6 +795,10 @@ int GuestFile::i_readData(uint32_t uSize, uint32_t uTimeoutMS,
             if (pcbRead)
                 *pcbRead = cbRead;
         }
+        else if (pEvent->HasGuestError()) /* Return guest rc if available. */
+        {
+            vrc = pEvent->GetGuestError();
+        }
     }
 
     unregisterWaitEvent(pEvent);
@@ -775,7 +826,7 @@ int GuestFile::i_readDataAt(uint64_t uOffset, uint32_t uSize, uint32_t uTimeoutM
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -786,10 +837,10 @@ int GuestFile::i_readDataAt(uint64_t uOffset, uint32_t uSize, uint32_t uTimeoutM
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[4];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setUInt32(mData.mID /* File handle */);
-    paParms[i++].setUInt64(uOffset /* Offset (in bytes) to start reading */);
-    paParms[i++].setUInt32(uSize /* Size (in bytes) to read */);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetU32(&paParms[i++], mObjectID /* File handle */);
+    HGCMSvcSetU64(&paParms[i++], uOffset /* Offset (in bytes) to start reading */);
+    HGCMSvcSetU32(&paParms[i++], uSize /* Size (in bytes) to read */);
 
     alock.release(); /* Drop write lock before sending. */
 
@@ -805,6 +856,10 @@ int GuestFile::i_readDataAt(uint64_t uOffset, uint32_t uSize, uint32_t uTimeoutM
             if (pcbRead)
                 *pcbRead = cbRead;
         }
+        else if (pEvent->HasGuestError()) /* Return guest rc if available. */
+        {
+            vrc = pEvent->GetGuestError();
+        }
     }
 
     unregisterWaitEvent(pEvent);
@@ -832,7 +887,7 @@ int GuestFile::i_seekAt(int64_t iOffset, GUEST_FILE_SEEKTYPE eSeekType,
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -843,17 +898,31 @@ int GuestFile::i_seekAt(int64_t iOffset, GUEST_FILE_SEEKTYPE eSeekType,
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[4];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setUInt32(mData.mID /* File handle */);
-    paParms[i++].setUInt32(eSeekType /* Seek method */);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetU32(&paParms[i++], mObjectID /* File handle */);
+    HGCMSvcSetU32(&paParms[i++], eSeekType /* Seek method */);
     /** @todo uint64_t vs. int64_t! */
-    paParms[i++].setUInt64((uint64_t)iOffset /* Offset (in bytes) to start reading */);
+    HGCMSvcSetU64(&paParms[i++], (uint64_t)iOffset /* Offset (in bytes) to start reading */);
 
     alock.release(); /* Drop write lock before sending. */
 
     vrc = sendCommand(HOST_FILE_SEEK, i, paParms);
     if (RT_SUCCESS(vrc))
-        vrc = i_waitForOffsetChange(pEvent, uTimeoutMS, puOffset);
+    {
+        uint64_t uOffset;
+        vrc = i_waitForOffsetChange(pEvent, uTimeoutMS, &uOffset);
+        if (RT_SUCCESS(vrc))
+        {
+            LogFlowThisFunc(("uOffset=%RU64\n", uOffset));
+
+            if (puOffset)
+                *puOffset = uOffset;
+        }
+        else if (pEvent->HasGuestError()) /* Return guest rc if available. */
+        {
+            vrc = pEvent->GetGuestError();
+        }
+    }
 
     unregisterWaitEvent(pEvent);
 
@@ -862,12 +931,12 @@ int GuestFile::i_seekAt(int64_t iOffset, GUEST_FILE_SEEKTYPE eSeekType,
 }
 
 /* static */
-HRESULT GuestFile::i_setErrorExternal(VirtualBoxBase *pInterface, int guestRc)
+HRESULT GuestFile::i_setErrorExternal(VirtualBoxBase *pInterface, int rcGuest)
 {
     AssertPtr(pInterface);
-    AssertMsg(RT_FAILURE(guestRc), ("Guest rc does not indicate a failure when setting error\n"));
+    AssertMsg(RT_FAILURE(rcGuest), ("Guest rc does not indicate a failure when setting error\n"));
 
-    return pInterface->setError(VBOX_E_IPRT_ERROR, GuestFile::i_guestErrorToString(guestRc).c_str());
+    return pInterface->setError(VBOX_E_IPRT_ERROR, GuestFile::i_guestErrorToString(rcGuest).c_str());
 }
 
 int GuestFile::i_setFileStatus(FileStatus_T fileStatus, int fileRc)
@@ -964,14 +1033,16 @@ int GuestFile::i_waitForRead(GuestWaitEvent *pEvent, uint32_t uTimeoutMS,
                 com::SafeArray <BYTE> data;
                 hr = pFileEvent->COMGETTER(Data)(ComSafeArrayAsOutParam(data));
                 ComAssertComRC(hr);
-                size_t cbRead = data.size();
-                if (   cbRead
-                    && cbRead <= cbData)
+                const size_t cbRead = data.size();
+                if (cbRead)
                 {
-                    memcpy(pvData, data.raw(), data.size());
+                    if (cbRead <= cbData)
+                        memcpy(pvData, data.raw(), cbRead);
+                    else
+                        vrc = VERR_BUFFER_OVERFLOW;
                 }
                 else
-                    vrc = VERR_BUFFER_OVERFLOW;
+                    vrc = VERR_NO_DATA;
             }
             if (pcbRead)
             {
@@ -987,7 +1058,7 @@ int GuestFile::i_waitForRead(GuestWaitEvent *pEvent, uint32_t uTimeoutMS,
 }
 
 int GuestFile::i_waitForStatusChange(GuestWaitEvent *pEvent, uint32_t uTimeoutMS,
-                                     FileStatus_T *pFileStatus, int *pGuestRc)
+                                     FileStatus_T *pFileStatus, int *prcGuest)
 {
     AssertPtrReturn(pEvent, VERR_INVALID_POINTER);
     /* pFileStatus is optional. */
@@ -1023,8 +1094,8 @@ int GuestFile::i_waitForStatusChange(GuestWaitEvent *pEvent, uint32_t uTimeoutMS
         if (RT_FAILURE((int)lGuestRc))
             vrc = VERR_GSTCTL_GUEST_ERROR;
 
-        if (pGuestRc)
-            *pGuestRc = (int)lGuestRc;
+        if (prcGuest)
+            *prcGuest = (int)lGuestRc;
     }
 
     return vrc;
@@ -1081,7 +1152,7 @@ int GuestFile::i_writeData(uint32_t uTimeoutMS, void *pvData, uint32_t cbData,
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -1092,10 +1163,10 @@ int GuestFile::i_writeData(uint32_t uTimeoutMS, void *pvData, uint32_t cbData,
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[8];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setUInt32(mData.mID /* File handle */);
-    paParms[i++].setUInt32(cbData /* Size (in bytes) to write */);
-    paParms[i++].setPointer(pvData, cbData);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetU32(&paParms[i++], mObjectID /* File handle */);
+    HGCMSvcSetU32(&paParms[i++], cbData /* Size (in bytes) to write */);
+    HGCMSvcSetPv(&paParms[i++], pvData, cbData);
 
     alock.release(); /* Drop write lock before sending. */
 
@@ -1107,9 +1178,13 @@ int GuestFile::i_writeData(uint32_t uTimeoutMS, void *pvData, uint32_t cbData,
         if (RT_SUCCESS(vrc))
         {
             LogFlowThisFunc(("cbWritten=%RU32\n", cbWritten));
-            if (cbWritten)
+            if (pcbWritten)
                 *pcbWritten = cbWritten;
         }
+        else if (pEvent->HasGuestError()) /* Return guest rc if available. */
+        {
+            vrc = pEvent->GetGuestError();
+        }
     }
 
     unregisterWaitEvent(pEvent);
@@ -1140,7 +1215,7 @@ int GuestFile::i_writeDataAt(uint64_t uOffset, uint32_t uTimeoutMS,
 
         vrc = registerWaitEvent(eventTypes, &pEvent);
     }
-    catch (std::bad_alloc)
+    catch (std::bad_alloc &)
     {
         vrc = VERR_NO_MEMORY;
     }
@@ -1151,11 +1226,11 @@ int GuestFile::i_writeDataAt(uint64_t uOffset, uint32_t uTimeoutMS,
     /* Prepare HGCM call. */
     VBOXHGCMSVCPARM paParms[8];
     int i = 0;
-    paParms[i++].setUInt32(pEvent->ContextID());
-    paParms[i++].setUInt32(mData.mID /* File handle */);
-    paParms[i++].setUInt64(uOffset /* Offset where to starting writing */);
-    paParms[i++].setUInt32(cbData /* Size (in bytes) to write */);
-    paParms[i++].setPointer(pvData, cbData);
+    HGCMSvcSetU32(&paParms[i++], pEvent->ContextID());
+    HGCMSvcSetU32(&paParms[i++], mObjectID /* File handle */);
+    HGCMSvcSetU64(&paParms[i++], uOffset /* Offset where to starting writing */);
+    HGCMSvcSetU32(&paParms[i++], cbData /* Size (in bytes) to write */);
+    HGCMSvcSetPv(&paParms[i++], pvData, cbData);
 
     alock.release(); /* Drop write lock before sending. */
 
@@ -1167,9 +1242,13 @@ int GuestFile::i_writeDataAt(uint64_t uOffset, uint32_t uTimeoutMS,
         if (RT_SUCCESS(vrc))
         {
             LogFlowThisFunc(("cbWritten=%RU32\n", cbWritten));
-            if (cbWritten)
+            if (pcbWritten)
                 *pcbWritten = cbWritten;
         }
+        else if (pEvent->HasGuestError()) /* Return guest rc if available. */
+        {
+            vrc = pEvent->GetGuestError();
+        }
     }
 
     unregisterWaitEvent(pEvent);
@@ -1182,49 +1261,106 @@ int GuestFile::i_writeDataAt(uint64_t uOffset, uint32_t uTimeoutMS,
 /////////////////////////////////////////////////////////////////////////////
 HRESULT GuestFile::close()
 {
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
     LogFlowThisFuncEnter();
 
     /* Close file on guest. */
-    int guestRc;
-    int rc = i_closeFile(&guestRc);
+    int rcGuest;
+    int vrc = i_closeFile(&rcGuest);
     /* On failure don't return here, instead do all the cleanup
      * work first and then return an error. */
 
     AssertPtr(mSession);
-    int rc2 = mSession->i_fileRemoveFromList(this);
-    if (RT_SUCCESS(rc))
-        rc = rc2;
+    int vrc2 = mSession->i_fileUnregister(this);
+    if (RT_SUCCESS(vrc))
+        vrc = vrc2;
 
-    if (RT_FAILURE(rc))
+    if (RT_FAILURE(vrc))
     {
-        if (rc == VERR_GSTCTL_GUEST_ERROR)
-            return GuestFile::i_setErrorExternal(this, guestRc);
-
-        return setError(VBOX_E_IPRT_ERROR,
-                        tr("Closing guest file failed with %Rrc\n"), rc);
+        if (vrc == VERR_GSTCTL_GUEST_ERROR)
+            return GuestFile::i_setErrorExternal(this, rcGuest);
+        return setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Closing guest file failed with %Rrc\n"), vrc);
     }
 
-    LogFlowThisFunc(("Returning rc=%Rrc\n", rc));
+    LogFlowThisFunc(("Returning S_OK / vrc=%Rrc\n", vrc));
     return S_OK;
 }
 
 HRESULT GuestFile::queryInfo(ComPtr<IFsObjInfo> &aObjInfo)
 {
-    RT_NOREF(aObjInfo);
-    ReturnComNotImplemented();
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    LogFlowThisFuncEnter();
+
+    HRESULT hr = S_OK;
+
+    GuestFsObjData fsObjData; int rcGuest;
+    int vrc = i_queryInfo(fsObjData, &rcGuest);
+    if (RT_SUCCESS(vrc))
+    {
+        ComObjPtr<GuestFsObjInfo> ptrFsObjInfo;
+        hr = ptrFsObjInfo.createObject();
+        if (SUCCEEDED(hr))
+        {
+            vrc = ptrFsObjInfo->init(fsObjData);
+            if (RT_SUCCESS(vrc))
+                hr = ptrFsObjInfo.queryInterfaceTo(aObjInfo.asOutParam());
+            else
+                hr = setErrorVrc(vrc);
+        }
+    }
+    else
+    {
+        if (GuestProcess::i_isGuestError(vrc))
+            hr = GuestProcess::i_setErrorExternal(this, rcGuest);
+        else
+            hr = setErrorVrc(vrc, tr("Querying file information failed: %Rrc"), vrc);
+    }
+
+    LogFlowFuncLeaveRC(vrc);
+    return hr;
 }
 
 HRESULT GuestFile::querySize(LONG64 *aSize)
 {
-    RT_NOREF(aSize);
-    ReturnComNotImplemented();
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    LogFlowThisFuncEnter();
+
+    HRESULT hr = S_OK;
+
+    GuestFsObjData fsObjData; int rcGuest;
+    int vrc = i_queryInfo(fsObjData, &rcGuest);
+    if (RT_SUCCESS(vrc))
+    {
+        *aSize = fsObjData.mObjectSize;
+    }
+    else
+    {
+        if (GuestProcess::i_isGuestError(vrc))
+            hr = GuestProcess::i_setErrorExternal(this, rcGuest);
+        else
+            hr = setErrorVrc(vrc, tr("Querying file size failed: %Rrc"), vrc);
+    }
+
+    LogFlowFuncLeaveRC(vrc);
+    return hr;
 }
 
 HRESULT GuestFile::read(ULONG aToRead, ULONG aTimeoutMS, std::vector<BYTE> &aData)
 {
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
     if (aToRead == 0)
         return setError(E_INVALIDARG, tr("The size to read is zero"));
 
+    LogFlowThisFuncEnter();
+
     aData.resize(aToRead);
 
     HRESULT hr = S_OK;
@@ -1242,25 +1378,24 @@ HRESULT GuestFile::read(ULONG aToRead, ULONG aTimeoutMS, std::vector<BYTE> &aDat
     {
         aData.resize(0);
 
-        switch (vrc)
-        {
-            default:
-                hr = setError(VBOX_E_IPRT_ERROR,
-                              tr("Reading from file \"%s\" failed: %Rrc"),
-                              mData.mOpenInfo.mFileName.c_str(), vrc);
-                break;
-        }
+        hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Reading from file \"%s\" failed: %Rrc"),
+                          mData.mOpenInfo.mFilename.c_str(), vrc);
     }
 
     LogFlowFuncLeaveRC(vrc);
     return hr;
 }
-HRESULT GuestFile::readAt(LONG64 aOffset, ULONG aToRead, ULONG aTimeoutMS, std::vector<BYTE> &aData)
 
+HRESULT GuestFile::readAt(LONG64 aOffset, ULONG aToRead, ULONG aTimeoutMS, std::vector<BYTE> &aData)
 {
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
     if (aToRead == 0)
         return setError(E_INVALIDARG, tr("The size to read is zero"));
 
+    LogFlowThisFuncEnter();
+
     aData.resize(aToRead);
 
     HRESULT hr = S_OK;
@@ -1277,14 +1412,8 @@ HRESULT GuestFile::readAt(LONG64 aOffset, ULONG aToRead, ULONG aTimeoutMS, std::
     {
         aData.resize(0);
 
-        switch (vrc)
-        {
-            default:
-                hr = setError(VBOX_E_IPRT_ERROR,
-                              tr("Reading from file \"%s\" (at offset %RU64) failed: %Rrc"),
-                              mData.mOpenInfo.mFileName.c_str(), aOffset, vrc);
-                break;
-        }
+        hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Reading from file \"%s\" (at offset %RU64) failed: %Rrc"),
+                          mData.mOpenInfo.mFilename.c_str(), aOffset, vrc);
     }
 
     LogFlowFuncLeaveRC(vrc);
@@ -1293,7 +1422,8 @@ HRESULT GuestFile::readAt(LONG64 aOffset, ULONG aToRead, ULONG aTimeoutMS, std::
 
 HRESULT GuestFile::seek(LONG64 aOffset, FileSeekOrigin_T aWhence, LONG64 *aNewOffset)
 {
-    LogFlowThisFuncEnter();
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     HRESULT hr = S_OK;
 
@@ -1314,25 +1444,18 @@ HRESULT GuestFile::seek(LONG64 aOffset, FileSeekOrigin_T aWhence, LONG64 *aNewOf
 
         default:
             return setError(E_INVALIDARG, tr("Invalid seek type specified"));
-            break; /* Never reached. */
     }
 
+    LogFlowThisFuncEnter();
+
     uint64_t uNewOffset;
     int vrc = i_seekAt(aOffset, eSeekType,
                        30 * 1000 /* 30s timeout */, &uNewOffset);
     if (RT_SUCCESS(vrc))
         *aNewOffset = RT_MIN(uNewOffset, (uint64_t)INT64_MAX);
     else
-    {
-        switch (vrc)
-        {
-            default:
-                hr = setError(VBOX_E_IPRT_ERROR,
-                              tr("Seeking file \"%s\" (to offset %RI64) failed: %Rrc"),
-                              mData.mOpenInfo.mFileName.c_str(), aOffset, vrc);
-                break;
-        }
-    }
+        hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Seeking file \"%s\" (to offset %RI64) failed: %Rrc"),
+                          mData.mOpenInfo.mFilename.c_str(), aOffset, vrc);
 
     LogFlowFuncLeaveRC(vrc);
     return hr;
@@ -1352,52 +1475,39 @@ HRESULT GuestFile::setSize(LONG64 aSize)
 
 HRESULT GuestFile::write(const std::vector<BYTE> &aData, ULONG aTimeoutMS, ULONG *aWritten)
 {
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
     LogFlowThisFuncEnter();
 
     HRESULT hr = S_OK;
 
     uint32_t cbData = (uint32_t)aData.size();
     void *pvData = cbData > 0? (void *)&aData.front(): NULL;
-    int vrc = i_writeData(aTimeoutMS, pvData, cbData,
-                          (uint32_t*)aWritten);
+    int vrc = i_writeData(aTimeoutMS, pvData, cbData, (uint32_t*)aWritten);
     if (RT_FAILURE(vrc))
-    {
-        switch (vrc)
-        {
-            default:
-                hr = setError(VBOX_E_IPRT_ERROR,
-                              tr("Writing %zubytes to file \"%s\" failed: %Rrc"),
-                              aData.size(), mData.mOpenInfo.mFileName.c_str(), vrc);
-                break;
-        }
-    }
+        hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Writing %zubytes to file \"%s\" failed: %Rrc"),
+                          aData.size(), mData.mOpenInfo.mFilename.c_str(), vrc);
 
     LogFlowFuncLeaveRC(vrc);
     return hr;
 }
 
 HRESULT GuestFile::writeAt(LONG64 aOffset, const std::vector<BYTE> &aData, ULONG aTimeoutMS, ULONG *aWritten)
-
 {
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
     LogFlowThisFuncEnter();
 
     HRESULT hr = S_OK;
 
     uint32_t cbData = (uint32_t)aData.size();
     void *pvData = cbData > 0? (void *)&aData.front(): NULL;
-    int vrc = i_writeData(aTimeoutMS, pvData, cbData,
-                          (uint32_t*)aWritten);
+    int vrc = i_writeData(aTimeoutMS, pvData, cbData, (uint32_t*)aWritten);
     if (RT_FAILURE(vrc))
-    {
-        switch (vrc)
-        {
-            default:
-                hr = setError(VBOX_E_IPRT_ERROR,
-                              tr("Writing %zubytes to file \"%s\" (at offset %RU64) failed: %Rrc"),
-                              aData.size(), mData.mOpenInfo.mFileName.c_str(), aOffset, vrc);
-                break;
-        }
-    }
+        hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Writing %zubytes to file \"%s\" (at offset %RU64) failed: %Rrc"),
+                          aData.size(), mData.mOpenInfo.mFilename.c_str(), aOffset, vrc);
 
     LogFlowFuncLeaveRC(vrc);
     return hr;
