@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -39,14 +39,17 @@
 #include <iprt/err.h>
 #include <iprt/file.h>
 #include <iprt/log.h>
+#include <iprt/utf16.h>
 #include "internal/fs.h"
 #include "internal/dir.h"
+#include "internal/path.h"
+#include "../win/internal-r3-win.h"
 
 
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
 *********************************************************************************************************************************/
-/** Whether to return a single record (TRUE) or multiple (FALSE)o. */
+/** Whether to return a single record (TRUE) or multiple (FALSE). */
 #define RTDIR_NT_SINGLE_RECORD  FALSE
 
 /** Go hard on record chaining (has slight performance impact). */
@@ -55,7 +58,7 @@
 #endif
 
 
-/* ASSUMES FileID comes after ShortName and the structus are identical up to that point. */
+/* ASSUMES FileID comes after ShortName and the structs are identical up to that point. */
 AssertCompileMembersSameSizeAndOffset(FILE_BOTH_DIR_INFORMATION, NextEntryOffset, FILE_ID_BOTH_DIR_INFORMATION, NextEntryOffset);
 AssertCompileMembersSameSizeAndOffset(FILE_BOTH_DIR_INFORMATION, FileIndex      , FILE_ID_BOTH_DIR_INFORMATION, FileIndex      );
 AssertCompileMembersSameSizeAndOffset(FILE_BOTH_DIR_INFORMATION, CreationTime   , FILE_ID_BOTH_DIR_INFORMATION, CreationTime   );
@@ -75,11 +78,11 @@ AssertCompileMembersSameSizeAndOffset(FILE_BOTH_DIR_INFORMATION, ShortName
 size_t rtDirNativeGetStructSize(const char *pszPath)
 {
     NOREF(pszPath);
-    return sizeof(RTDIR);
+    return sizeof(RTDIRINTERNAL);
 }
 
 
-int rtDirNativeOpen(PRTDIR pDir, char *pszPathBuf)
+int rtDirNativeOpen(PRTDIRINTERNAL pDir, char *pszPathBuf, uintptr_t hRelativeDir, void *pvNativeRelative)
 {
     /*
      * Convert the filter to UTF-16.
@@ -102,26 +105,101 @@ int rtDirNativeOpen(PRTDIR pDir, char *pszPathBuf)
      * Try open the directory
      */
 #ifdef IPRT_WITH_NT_PATH_PASSTHRU
-    bool fObjDir;
+    bool fObjDir = false;
 #endif
-    rc = RTNtPathOpenDir(pszPathBuf,
-                         FILE_READ_DATA | SYNCHRONIZE,
-                         FILE_SHARE_READ | FILE_SHARE_WRITE,
-                         FILE_DIRECTORY_FILE | FILE_OPEN_FOR_BACKUP_INTENT | FILE_SYNCHRONOUS_IO_NONALERT,
-                         OBJ_CASE_INSENSITIVE,
-                         &pDir->hDir,
+    if (hRelativeDir != ~(uintptr_t)0 && pvNativeRelative == NULL)
+    {
+        /* Caller already opened it, easy! */
+        pDir->hDir = (HANDLE)hRelativeDir;
+        rc = VINF_SUCCESS;
+    }
+    else
+    {
+        /*
+         * If we have to check for reparse points, this gets complicated!
+         */
+        static int volatile g_fReparsePoints = -1;
+        uint32_t            fOptions         = FILE_DIRECTORY_FILE | FILE_OPEN_FOR_BACKUP_INTENT | FILE_SYNCHRONOUS_IO_NONALERT;
+        int fReparsePoints = g_fReparsePoints;
+        if (   fReparsePoints != 0
+            && (pDir->fFlags & RTDIR_F_NO_FOLLOW)
+            && !pDir->fDirSlash)
+            fOptions |= FILE_OPEN_REPARSE_POINT;
+
+        for (;;)
+        {
+            if (pvNativeRelative == NULL)
+                rc = RTNtPathOpenDir(pszPathBuf,
+                                     FILE_LIST_DIRECTORY | FILE_READ_ATTRIBUTES | FILE_TRAVERSE | SYNCHRONIZE,
+                                     FILE_SHARE_READ | FILE_SHARE_WRITE,
+                                     fOptions,
+                                     OBJ_CASE_INSENSITIVE,
+                                     &pDir->hDir,
 #ifdef IPRT_WITH_NT_PATH_PASSTHRU
-                         &fObjDir
+                                     &fObjDir
 #else
-                         NULL
+                                     NULL
 #endif
-                         );
+                                     );
+            else
+                rc = RTNtPathOpenDirEx((HANDLE)hRelativeDir,
+                                       (struct _UNICODE_STRING *)pvNativeRelative,
+                                       FILE_LIST_DIRECTORY | FILE_READ_ATTRIBUTES | FILE_TRAVERSE | SYNCHRONIZE,
+                                       FILE_SHARE_READ | FILE_SHARE_WRITE,
+                                       fOptions,
+                                       OBJ_CASE_INSENSITIVE,
+                                       &pDir->hDir,
+#ifdef IPRT_WITH_NT_PATH_PASSTHRU
+                                       &fObjDir
+#else
+                                       NULL
+#endif
+                                       );
+            if (   !(fOptions & FILE_OPEN_REPARSE_POINT)
+                || (rc != VINF_SUCCESS && rc != VERR_INVALID_PARAMETER) )
+                break;
+            if (rc == VINF_SUCCESS)
+            {
+                if (fReparsePoints == -1)
+                    g_fReparsePoints = 1;
+
+                /*
+                 * We now need to check if we opened a symbolic directory link.
+                 * (These can be enumerated, but contains only '.' and '..'.)
+                 */
+                FILE_ATTRIBUTE_TAG_INFORMATION  TagInfo = { 0, 0 };
+                IO_STATUS_BLOCK                 Ios = RTNT_IO_STATUS_BLOCK_INITIALIZER;
+                NTSTATUS rcNt = NtQueryInformationFile(pDir->hDir, &Ios, &TagInfo, sizeof(TagInfo), FileAttributeTagInformation);
+                AssertMsg(NT_SUCCESS(rcNt), ("%#x\n", rcNt));
+                if (!NT_SUCCESS(rcNt))
+                    TagInfo.FileAttributes = TagInfo.ReparseTag = 0;
+                if (!(TagInfo.FileAttributes & FILE_ATTRIBUTE_REPARSE_POINT))
+                    break;
+
+                NtClose(pDir->hDir);
+                pDir->hDir = RTNT_INVALID_HANDLE_VALUE;
+
+                if (TagInfo.ReparseTag == IO_REPARSE_TAG_SYMLINK)
+                {
+                    rc = VERR_IS_A_SYMLINK;
+                    break;
+                }
+
+                /* Reparse point that isn't a symbolic link, try follow the reparsing. */
+            }
+            else if (fReparsePoints == -1)
+                g_fReparsePoints = fReparsePoints = 0;
+            fOptions &= ~FILE_OPEN_REPARSE_POINT;
+        }
+
+    }
     if (RT_SUCCESS(rc))
     {
         /*
          * Init data.
          */
         pDir->fDataUnread = false; /* spelling it out */
+        pDir->uDirDev     = 0;
 #ifdef IPRT_WITH_NT_PATH_PASSTHRU
         if (fObjDir)
             pDir->enmInfoClass = FileMaximumInformation; /* object directory. */
@@ -131,8 +209,10 @@ int rtDirNativeOpen(PRTDIR pDir, char *pszPathBuf)
 }
 
 
-RTDECL(int) RTDirClose(PRTDIR pDir)
+RTDECL(int) RTDirClose(RTDIR hDir)
 {
+    PRTDIRINTERNAL pDir = hDir;
+
     /*
      * Validate input.
      */
@@ -172,9 +252,9 @@ RTDECL(int) RTDirClose(PRTDIR pDir)
  * @returns IPRT status code
  * @param   pThis       The directory instance data.
  */
-static int rtDirNtCheckRecord(PRTDIR pThis)
+static int rtDirNtCheckRecord(PRTDIRINTERNAL pThis)
 {
-#ifdef RTDIR_NT_STRICT
+#if defined(RTDIR_NT_STRICT) || defined(RT_ARCH_X86)
 # ifdef IPRT_WITH_NT_PATH_PASSTHRU
     if (pThis->enmInfoClass != FileMaximumInformation)
 # endif
@@ -184,8 +264,32 @@ static int rtDirNtCheckRecord(PRTDIR pThis)
             uEndAddr = (uintptr_t)&pThis->uCurData.pBothId->FileName[0];
         else
             uEndAddr = (uintptr_t)&pThis->uCurData.pBoth->FileName[0];
-        AssertReturn(uEndAddr < (uintptr_t)&pThis->pabBuffer[pThis->cbBuffer], VERR_IO_GEN_FAILURE);
 
+# ifdef RT_ARCH_X86
+        /* Workaround for NT 3.1 bug where FAT returns a too short buffer length.
+           Including all NT 3.x versions in case it bug was fixed till NT 4. */
+        uintptr_t const uEndBuffer = (uintptr_t)&pThis->pabBuffer[pThis->cbBuffer];
+        if (   uEndAddr < uEndBuffer
+            && uEndAddr + pThis->uCurData.pBoth->FileNameLength <= uEndBuffer)
+        { /* likely */ }
+        else if (   (   g_enmWinVer == kRTWinOSType_NT310
+                     || g_enmWinVer == kRTWinOSType_NT350 // not sure when it was fixed...
+                     || g_enmWinVer == kRTWinOSType_NT351)
+                 && pThis->enmInfoClass == FileBothDirectoryInformation)
+        {
+            size_t cbLeft = (uintptr_t)&pThis->pabBuffer[pThis->cbBufferAlloc] - (uintptr_t)pThis->uCurData.pBoth;
+            if (   cbLeft >= RT_UOFFSETOF(FILE_BOTH_DIR_INFORMATION, FileName)
+                && pThis->uCurData.pBoth->FileNameLength > 0
+                && cbLeft >= RT_UOFFSETOF(FILE_BOTH_DIR_INFORMATION, FileName) + pThis->uCurData.pBoth->FileNameLength)
+            {
+                pThis->cbBuffer = ((uintptr_t)&pThis->uCurData.pBoth->FileName[0] + pThis->uCurData.pBoth->FileNameLength)
+                                - (uintptr_t)&pThis->pabBuffer[0];
+            }
+        }
+# endif
+
+# ifdef RTDIR_NT_STRICT
+        AssertReturn(uEndAddr < (uintptr_t)&pThis->pabBuffer[pThis->cbBuffer], VERR_IO_GEN_FAILURE);
         AssertReturn(pThis->uCurData.pBoth->FileNameLength < _64K, VERR_FILENAME_TOO_LONG);
         AssertReturn((pThis->uCurData.pBoth->FileNameLength & 1) == 0, VERR_IO_GEN_FAILURE);
 
@@ -194,7 +298,10 @@ static int rtDirNtCheckRecord(PRTDIR pThis)
 
         AssertReturn((unsigned)pThis->uCurData.pBoth->ShortNameLength <= sizeof(pThis->uCurData.pBoth->ShortName),
                      VERR_IO_GEN_FAILURE);
+# endif
     }
+#else
+    RT_NOREF_PV(pThis);
 #endif
 
     return VINF_SUCCESS;
@@ -206,7 +313,7 @@ static int rtDirNtCheckRecord(PRTDIR pThis)
  *
  * @param   pThis       The directory instance data.
  */
-static int rtDirNtAdvanceBuffer(PRTDIR pThis)
+static int rtDirNtAdvanceBuffer(PRTDIRINTERNAL pThis)
 {
     int rc;
 
@@ -249,7 +356,7 @@ static int rtDirNtAdvanceBuffer(PRTDIR pThis)
  * @returns IPRT status code
  * @param   pThis       The directory instance data.
  */
-static int rtDirNtFetchMore(PRTDIR pThis)
+static int rtDirNtFetchMore(PRTDIRINTERNAL pThis)
 {
     Assert(!pThis->fDataUnread);
 
@@ -292,6 +399,22 @@ static int rtDirNtFetchMore(PRTDIR pThis)
             if (!pThis->pabBuffer)
                 return VERR_NO_MEMORY;
         }
+
+        /*
+         * Also try determining the device number.
+         */
+        PFILE_FS_VOLUME_INFORMATION pVolInfo = (PFILE_FS_VOLUME_INFORMATION)pThis->pabBuffer;
+        pVolInfo->VolumeSerialNumber = 0;
+        IO_STATUS_BLOCK Ios = RTNT_IO_STATUS_BLOCK_INITIALIZER;
+        NTSTATUS rcNt = NtQueryVolumeInformationFile(pThis->hDir, &Ios,
+                                                     pVolInfo, RT_MIN(_2K, pThis->cbBufferAlloc),
+                                                     FileFsVolumeInformation);
+        if (NT_SUCCESS(rcNt) && NT_SUCCESS(Ios.Status))
+            pThis->uDirDev = pVolInfo->VolumeSerialNumber;
+        else
+            pThis->uDirDev = 0;
+        AssertCompile(sizeof(pThis->uDirDev) == sizeof(pVolInfo->VolumeSerialNumber));
+        /** @todo Grow RTDEV to 64-bit and add low dword of VolumeCreationTime to the top of uDirDev. */
     }
 
     /*
@@ -309,7 +432,7 @@ static int rtDirNtFetchMore(PRTDIR pThis)
                                                        pThis->pabBuffer,
                                                        pThis->cbBufferAlloc,
                                                        RTDIR_NT_SINGLE_RECORD /*ReturnSingleEntry */,
-                                                       FALSE /*RestartScan*/,
+                                                       pThis->fRestartScan,
                                                        &pThis->uObjDirCtx,
                                                        (PULONG)&Ios.Information);
         }
@@ -325,7 +448,7 @@ static int rtDirNtFetchMore(PRTDIR pThis)
                                         pThis->enmInfoClass,
                                         RTDIR_NT_SINGLE_RECORD /*ReturnSingleEntry */,
                                         pThis->pNtFilterStr,
-                                        FALSE /*RestartScan */);
+                                        pThis->fRestartScan);
     }
     else
     {
@@ -363,12 +486,12 @@ static int rtDirNtFetchMore(PRTDIR pThis)
                                     pThis->enmInfoClass,
                                     RTDIR_NT_SINGLE_RECORD /*ReturnSingleEntry */,
                                     pThis->pNtFilterStr,
-                                    FALSE /*RestartScan */);
+                                    pThis->fRestartScan);
         if (NT_SUCCESS(rcNt))
         { /* likely */ }
         else
         {
-            bool fRestartScan = false;
+            bool fRestartScan = pThis->fRestartScan;
             for (unsigned iRetry = 0; iRetry < 2; iRetry++)
             {
                 if (   rcNt == STATUS_INVALID_INFO_CLASS
@@ -428,7 +551,10 @@ static int rtDirNtFetchMore(PRTDIR pThis)
             return VERR_NO_MORE_FILES;
         return RTErrConvertFromNtStatus(rcNt);
     }
-    Assert(Ios.Information > sizeof(*pThis->uCurData.pBoth));
+    pThis->fRestartScan = false;
+    AssertMsg(  Ios.Information
+              > (pThis->enmInfoClass == FileMaximumInformation ? sizeof(*pThis->uCurData.pObjDir) : sizeof(*pThis->uCurData.pBoth)),
+              ("Ios.Information=%#x\n", Ios.Information));
 
     /*
      * Set up the data members.
@@ -454,7 +580,7 @@ static int rtDirNtFetchMore(PRTDIR pThis)
  * @param   cbName      The file name length in bytes.
  * @param   pwsName     The file name, not terminated.
  */
-static int rtDirNtConvertName(PRTDIR pThis, uint32_t cbName, PCRTUTF16 pwsName)
+static int rtDirNtConvertName(PRTDIRINTERNAL pThis, uint32_t cbName, PCRTUTF16 pwsName)
 {
     int rc = RTUtf16ToUtf8Ex(pwsName, cbName / 2, &pThis->pszName, pThis->cbNameAlloc, &pThis->cchName);
     if (RT_SUCCESS(rc))
@@ -483,7 +609,7 @@ static int rtDirNtConvertName(PRTDIR pThis, uint32_t cbName, PCRTUTF16 pwsName)
  * @returns IPRT status code.
  * @param   pThis       The directory instance data.
  */
-static int rtDirNtConvertCurName(PRTDIR pThis)
+static int rtDirNtConvertCurName(PRTDIRINTERNAL pThis)
 {
     switch (pThis->enmInfoClass)
     {
@@ -502,8 +628,9 @@ static int rtDirNtConvertCurName(PRTDIR pThis)
 }
 
 
-RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
+RTDECL(int) RTDirRead(RTDIR hDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
 {
+    PRTDIRINTERNAL pDir = hDir;
     int rc;
 
     /*
@@ -517,7 +644,7 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
     {
         cbDirEntry = *pcbDirEntry;
         AssertMsgReturn(cbDirEntry >= RT_UOFFSETOF(RTDIRENTRY, szName[2]),
-                        ("Invalid *pcbDirEntry=%d (min %d)\n", *pcbDirEntry, RT_OFFSETOF(RTDIRENTRY, szName[2])),
+                        ("Invalid *pcbDirEntry=%zu (min %zu)\n", *pcbDirEntry, RT_UOFFSETOF(RTDIRENTRY, szName[2])),
                         VERR_INVALID_PARAMETER);
     }
 
@@ -543,7 +670,7 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
      */
     const char  *pszName    = pDir->pszName;
     const size_t cchName    = pDir->cchName;
-    const size_t cbRequired = RT_OFFSETOF(RTDIRENTRY, szName[1]) + cchName;
+    const size_t cbRequired = RT_UOFFSETOF(RTDIRENTRY, szName[1]) + cchName;
     if (pcbDirEntry)
         *pcbDirEntry = cbRequired;
     if (cbRequired > cbDirEntry)
@@ -577,7 +704,9 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
 
             case FILE_ATTRIBUTE_REPARSE_POINT:
             case FILE_ATTRIBUTE_REPARSE_POINT | FILE_ATTRIBUTE_DIRECTORY:
-                pDirEntry->enmType = RTDIRENTRYTYPE_SYMLINK;
+                /* EaSize is here reused for returning the repharse tag value. */
+                if (pDir->uCurData.pBoth->EaSize == IO_REPARSE_TAG_SYMLINK)
+                    pDirEntry->enmType = RTDIRENTRYTYPE_SYMLINK;
                 break;
         }
     }
@@ -598,9 +727,10 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
 }
 
 
-RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntry,
+RTDECL(int) RTDirReadEx(RTDIR hDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntry,
                         RTFSOBJATTRADD enmAdditionalAttribs, uint32_t fFlags)
 {
+    PRTDIRINTERNAL pDir = hDir;
     int rc;
 
     /*
@@ -619,7 +749,7 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
     {
         cbDirEntry = *pcbDirEntry;
         AssertMsgReturn(cbDirEntry >= RT_UOFFSETOF(RTDIRENTRYEX, szName[2]),
-                        ("Invalid *pcbDirEntry=%d (min %d)\n", *pcbDirEntry, RT_OFFSETOF(RTDIRENTRYEX, szName[2])),
+                        ("Invalid *pcbDirEntry=%zu (min %zu)\n", *pcbDirEntry, RT_UOFFSETOF(RTDIRENTRYEX, szName[2])),
                         VERR_INVALID_PARAMETER);
     }
 
@@ -645,7 +775,7 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
      */
     const char  *pszName    = pDir->pszName;
     const size_t cchName    = pDir->cchName;
-    const size_t cbRequired = RT_OFFSETOF(RTDIRENTRYEX, szName[1]) + cchName;
+    const size_t cbRequired = RT_UOFFSETOF(RTDIRENTRYEX, szName[1]) + cchName;
     if (pcbDirEntry)
         *pcbDirEntry = cbRequired;
     if (cbRequired > cbDirEntry)
@@ -683,7 +813,7 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
         RTTimeSpecSetNtTime(&pDirEntry->Info.ChangeTime,        pBoth->ChangeTime.QuadPart);
 
         pDirEntry->Info.Attr.fMode  = rtFsModeFromDos((pBoth->FileAttributes << RTFS_DOS_SHIFT) & RTFS_DOS_MASK_NT,
-                                                       pszName, cchName);
+                                                       pszName, cchName, pBoth->EaSize);
     }
 #ifdef IPRT_WITH_NT_PATH_PASSTHRU
     else
@@ -727,11 +857,14 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
 
         case RTFSOBJATTRADD_UNIX:
             pDirEntry->Info.Attr.enmAdditional          = RTFSOBJATTRADD_UNIX;
-            pDirEntry->Info.Attr.u.Unix.uid             = ~0U;
-            pDirEntry->Info.Attr.u.Unix.gid             = ~0U;
+            pDirEntry->Info.Attr.u.Unix.uid             = NIL_RTUID;
+            pDirEntry->Info.Attr.u.Unix.gid             = NIL_RTGID;
             pDirEntry->Info.Attr.u.Unix.cHardlinks      = 1;
-            pDirEntry->Info.Attr.u.Unix.INodeIdDevice   = 0; /** @todo Use the volume serial number (see GetFileInformationByHandle). */
-            pDirEntry->Info.Attr.u.Unix.INodeId         = 0; /** @todo Use the fileid (see GetFileInformationByHandle). */
+            pDirEntry->Info.Attr.u.Unix.INodeIdDevice   = pDir->uDirDev;
+            pDirEntry->Info.Attr.u.Unix.INodeId         = 0;
+            if (   pDir->enmInfoClass == FileIdBothDirectoryInformation
+                && pDir->uCurData.pBothId->FileId.QuadPart != UINT64_MAX)
+                pDirEntry->Info.Attr.u.Unix.INodeId     = pDir->uCurData.pBothId->FileId.QuadPart;
             pDirEntry->Info.Attr.u.Unix.fFlags          = 0;
             pDirEntry->Info.Attr.u.Unix.GenerationId    = 0;
             pDirEntry->Info.Attr.u.Unix.Device          = 0;
@@ -743,13 +876,13 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
 
         case RTFSOBJATTRADD_UNIX_OWNER:
             pDirEntry->Info.Attr.enmAdditional          = RTFSOBJATTRADD_UNIX_OWNER;
-            pDirEntry->Info.Attr.u.UnixOwner.uid        = ~0U;
+            pDirEntry->Info.Attr.u.UnixOwner.uid        = NIL_RTUID;
             pDirEntry->Info.Attr.u.UnixOwner.szName[0]  = '\0'; /** @todo return something sensible here. */
             break;
 
         case RTFSOBJATTRADD_UNIX_GROUP:
             pDirEntry->Info.Attr.enmAdditional          = RTFSOBJATTRADD_UNIX_GROUP;
-            pDirEntry->Info.Attr.u.UnixGroup.gid        = ~0U;
+            pDirEntry->Info.Attr.u.UnixGroup.gid        = NIL_RTGID;
             pDirEntry->Info.Attr.u.UnixGroup.szName[0]  = '\0';
             break;
 
@@ -774,3 +907,88 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
     return rtDirNtAdvanceBuffer(pDir);
 }
 
+
+RTDECL(int) RTDirRewind(RTDIR hDir)
+{
+    /*
+     * Validate and digest input.
+     */
+    PRTDIRINTERNAL pThis = hDir;
+    AssertPtrReturn(pThis, VERR_INVALID_POINTER);
+    AssertReturn(pThis->u32Magic == RTDIR_MAGIC, VERR_INVALID_HANDLE);
+
+    /*
+     * The work is done on the next call to rtDirNtFetchMore.
+     */
+    pThis->fRestartScan = true;
+    pThis->fDataUnread  = false;
+
+    return VINF_SUCCESS;
+}
+
+
+RTR3DECL(int) RTDirQueryInfo(RTDIR hDir, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs)
+{
+    PRTDIRINTERNAL pDir = hDir;
+    AssertPtrReturn(pDir, VERR_INVALID_POINTER);
+    AssertReturn(pDir->u32Magic == RTDIR_MAGIC, VERR_INVALID_HANDLE);
+    AssertReturn(enmAdditionalAttribs >= RTFSOBJATTRADD_NOTHING && enmAdditionalAttribs <= RTFSOBJATTRADD_LAST,
+                 VERR_INVALID_PARAMETER);
+
+    if (pDir->enmInfoClass == FileMaximumInformation)
+    {
+        /*
+         * Directory object (see similar code above and rtPathNtQueryInfoInDirectoryObject).
+         */
+        pObjInfo->cbObject    = 0;
+        pObjInfo->cbAllocated = 0;
+        RTTimeSpecSetNtTime(&pObjInfo->BirthTime,         0);
+        RTTimeSpecSetNtTime(&pObjInfo->AccessTime,        0);
+        RTTimeSpecSetNtTime(&pObjInfo->ModificationTime,  0);
+        RTTimeSpecSetNtTime(&pObjInfo->ChangeTime,        0);
+        pObjInfo->Attr.fMode = RTFS_DOS_DIRECTORY | RTFS_TYPE_DIRECTORY | 0777;
+        pObjInfo->Attr.enmAdditional = enmAdditionalAttribs;
+        switch (enmAdditionalAttribs)
+        {
+            case RTFSOBJATTRADD_NOTHING:
+            case RTFSOBJATTRADD_UNIX:
+                pObjInfo->Attr.u.Unix.uid             = NIL_RTUID;
+                pObjInfo->Attr.u.Unix.gid             = NIL_RTGID;
+                pObjInfo->Attr.u.Unix.cHardlinks      = 1;
+                pObjInfo->Attr.u.Unix.INodeIdDevice   = pDir->uDirDev;
+                pObjInfo->Attr.u.Unix.INodeId         = 0;
+                pObjInfo->Attr.u.Unix.fFlags          = 0;
+                pObjInfo->Attr.u.Unix.GenerationId    = 0;
+                pObjInfo->Attr.u.Unix.Device          = 0;
+                break;
+
+            case RTFSOBJATTRADD_EASIZE:
+                pObjInfo->Attr.u.EASize.cb            = 0;
+                break;
+
+            case RTFSOBJATTRADD_UNIX_OWNER:
+                pObjInfo->Attr.enmAdditional          = RTFSOBJATTRADD_UNIX_OWNER;
+                pObjInfo->Attr.u.UnixOwner.uid        = NIL_RTUID;
+                pObjInfo->Attr.u.UnixOwner.szName[0]  = '\0'; /** @todo return something sensible here. */
+                break;
+
+            case RTFSOBJATTRADD_UNIX_GROUP:
+                pObjInfo->Attr.enmAdditional          = RTFSOBJATTRADD_UNIX_GROUP;
+                pObjInfo->Attr.u.UnixGroup.gid        = NIL_RTGID;
+                pObjInfo->Attr.u.UnixGroup.szName[0]  = '\0';
+                break;
+
+            default:
+                AssertMsgFailed(("Impossible!\n"));
+                return VERR_INTERNAL_ERROR_2;
+        }
+        return VINF_SUCCESS;
+    }
+
+    /*
+     * Regular directory file.
+     */
+    uint8_t abBuf[_2K];
+    return rtPathNtQueryInfoFromHandle(pDir->hDir, abBuf, sizeof(abBuf), pObjInfo, enmAdditionalAttribs, "", 0);
+}
+
