@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013-2015 Oracle Corporation
+ * Copyright (C) 2013-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -52,11 +52,13 @@ typedef struct CPUMDBENTRY
     CPUMMICROARCH   enmMicroarch;
     /** Scalable bus frequency used for reporting other frequencies. */
     uint64_t        uScalableBusFreq;
-    /** Flags (TBD). */
+    /** Flags - CPUDB_F_XXX. */
     uint32_t        fFlags;
     /** The maximum physical address with of the CPU.  This should correspond to
      * the value in CPUID leaf 0x80000008 when present. */
     uint8_t         cMaxPhysAddrWidth;
+    /** The MXCSR mask. */
+    uint32_t        fMxCsrMask;
     /** Pointer to an array of CPUID leaves.  */
     PCCPUMCPUIDLEAF paCpuIdLeaves;
     /** The number of CPUID leaves in the array paCpuIdLeaves points to. */
@@ -66,7 +68,8 @@ typedef struct CPUMDBENTRY
     /** The default unknown CPUID value. */
     CPUMCPUID       DefUnknownCpuId;
 
-    /** MSR mask.  Several microarchitectures ignore higher bits of the    */
+    /** MSR mask.  Several microarchitectures ignore the higher bits of ECX in
+     *  the RDMSR and WRMSR instructions. */
     uint32_t        fMsrMask;
 
     /** The number of ranges in the table pointed to b paMsrRanges. */
@@ -79,6 +82,13 @@ typedef struct CPUMDBENTRY
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
 *********************************************************************************************************************************/
+/** @name CPUDB_F_XXX - CPUDBENTRY::fFlags
+ * @{ */
+/** Should execute all in IEM.
+ * @todo Implement this - currently done in Main...  */
+#define CPUDB_F_EXECUTE_ALL_IN_IEM          RT_BIT_32(0)
+/** @} */
+
 
 /** @def NULL_ALONE
  * For eliminating an unnecessary data dependency in standalone builds (for
@@ -175,15 +185,26 @@ typedef struct CPUMDBENTRY
 #endif
 /** @} */
 
+#ifndef CPUM_DB_STANDALONE
 
+#include "cpus/Intel_Core_i7_6700K.h"
 #include "cpus/Intel_Core_i7_5600U.h"
 #include "cpus/Intel_Core_i7_3960X.h"
 #include "cpus/Intel_Core_i5_3570.h"
 #include "cpus/Intel_Core_i7_2635QM.h"
 #include "cpus/Intel_Xeon_X5482_3_20GHz.h"
+#include "cpus/Intel_Core2_X6800_2_93GHz.h"
+#include "cpus/Intel_Core2_T7600_2_33GHz.h"
+#include "cpus/Intel_Core_Duo_T2600_2_16GHz.h"
 #include "cpus/Intel_Pentium_M_processor_2_00GHz.h"
 #include "cpus/Intel_Pentium_4_3_00GHz.h"
+#include "cpus/Intel_Pentium_N3530_2_16GHz.h"
 #include "cpus/Intel_Atom_330_1_60GHz.h"
+#include "cpus/Intel_80486.h"
+#include "cpus/Intel_80386.h"
+#include "cpus/Intel_80286.h"
+#include "cpus/Intel_80186.h"
+#include "cpus/Intel_8086.h"
 
 #include "cpus/AMD_FX_8150_Eight_Core.h"
 #include "cpus/AMD_Phenom_II_X6_1100T.h"
@@ -209,54 +230,91 @@ typedef struct CPUMDBENTRY
  */
 static CPUMDBENTRY const * const g_apCpumDbEntries[] =
 {
-#ifdef VBOX_CPUDB_Intel_Core_i7_5600U
+#ifdef VBOX_CPUDB_Intel_Core_i7_6700K_h
+    &g_Entry_Intel_Core_i7_6700K,
+#endif
+#ifdef VBOX_CPUDB_Intel_Core_i7_5600U_h
     &g_Entry_Intel_Core_i7_5600U,
 #endif
-#ifdef VBOX_CPUDB_Intel_Core_i5_3570
+#ifdef VBOX_CPUDB_Intel_Core_i5_3570_h
     &g_Entry_Intel_Core_i5_3570,
 #endif
-#ifdef VBOX_CPUDB_Intel_Core_i7_3960X
+#ifdef VBOX_CPUDB_Intel_Core_i7_3960X_h
     &g_Entry_Intel_Core_i7_3960X,
 #endif
-#ifdef VBOX_CPUDB_Intel_Core_i7_2635QM
+#ifdef VBOX_CPUDB_Intel_Core_i7_2635QM_h
     &g_Entry_Intel_Core_i7_2635QM,
 #endif
-#ifdef VBOX_CPUDB_Intel_Atom_330_1_60GHz
+#ifdef VBOX_CPUDB_Intel_Pentium_N3530_2_16GHz_h
+    &g_Entry_Intel_Pentium_N3530_2_16GHz,
+#endif
+#ifdef VBOX_CPUDB_Intel_Atom_330_1_60GHz_h
     &g_Entry_Intel_Atom_330_1_60GHz,
 #endif
-#ifdef Intel_Pentium_M_processor_2_00GHz
+#ifdef VBOX_CPUDB_Intel_Pentium_M_processor_2_00GHz_h
     &g_Entry_Intel_Pentium_M_processor_2_00GHz,
 #endif
-#ifdef VBOX_CPUDB_Intel_Xeon_X5482_3_20GHz
+#ifdef VBOX_CPUDB_Intel_Xeon_X5482_3_20GHz_h
     &g_Entry_Intel_Xeon_X5482_3_20GHz,
 #endif
-#ifdef VBOX_CPUDB_Intel_Pentium_4_3_00GHz
+#ifdef VBOX_CPUDB_Intel_Core2_X6800_2_93GHz_h
+    &g_Entry_Intel_Core2_X6800_2_93GHz,
+#endif
+#ifdef VBOX_CPUDB_Intel_Core2_T7600_2_33GHz_h
+    &g_Entry_Intel_Core2_T7600_2_33GHz,
+#endif
+#ifdef VBOX_CPUDB_Intel_Core_Duo_T2600_2_16GHz_h
+    &g_Entry_Intel_Core_Duo_T2600_2_16GHz,
+#endif
+#ifdef VBOX_CPUDB_Intel_Pentium_4_3_00GHz_h
     &g_Entry_Intel_Pentium_4_3_00GHz,
 #endif
+#ifdef VBOX_CPUDB_Intel_Pentium_4_3_00GHz_h
+    &g_Entry_Intel_Pentium_4_3_00GHz,
+#endif
+/** @todo pentium, pentium mmx, pentium pro, pentium II, pentium III */
+#ifdef VBOX_CPUDB_Intel_80486_h
+    &g_Entry_Intel_80486,
+#endif
+#ifdef VBOX_CPUDB_Intel_80386_h
+    &g_Entry_Intel_80386,
+#endif
+#ifdef VBOX_CPUDB_Intel_80286_h
+    &g_Entry_Intel_80286,
+#endif
+#ifdef VBOX_CPUDB_Intel_80186_h
+    &g_Entry_Intel_80186,
+#endif
+#ifdef VBOX_CPUDB_Intel_8086_h
+    &g_Entry_Intel_8086,
+#endif
 
-#ifdef VBOX_CPUDB_AMD_FX_8150_Eight_Core
+#ifdef VBOX_CPUDB_AMD_FX_8150_Eight_Core_h
     &g_Entry_AMD_FX_8150_Eight_Core,
 #endif
-#ifdef VBOX_CPUDB_AMD_Phenom_II_X6_1100T
+#ifdef VBOX_CPUDB_AMD_Phenom_II_X6_1100T_h
     &g_Entry_AMD_Phenom_II_X6_1100T,
 #endif
-#ifdef VBOX_CPUDB_Quad_Core_AMD_Opteron_2384
+#ifdef VBOX_CPUDB_Quad_Core_AMD_Opteron_2384_h
     &g_Entry_Quad_Core_AMD_Opteron_2384,
 #endif
-#ifdef VBOX_CPUDB_AMD_Athlon_64_X2_Dual_Core_4200
+#ifdef VBOX_CPUDB_AMD_Athlon_64_X2_Dual_Core_4200_h
     &g_Entry_AMD_Athlon_64_X2_Dual_Core_4200,
 #endif
-#ifdef VBOX_CPUDB_AMD_Athlon_64_3200
+#ifdef VBOX_CPUDB_AMD_Athlon_64_3200_h
     &g_Entry_AMD_Athlon_64_3200,
 #endif
 
-#ifdef VBOX_CPUDB_VIA_QuadCore_L4700_1_2_GHz
+#ifdef VBOX_CPUDB_VIA_QuadCore_L4700_1_2_GHz_h
     &g_Entry_VIA_QuadCore_L4700_1_2_GHz,
 #endif
+
+#ifdef VBOX_CPUDB_NEC_V20_h
+    &g_Entry_NEC_V20,
+#endif
 };
 
 
-#ifndef CPUM_DB_STANDALONE
 
 /**
  * Binary search used by cpumR3MsrRangesInsert and has some special properties
@@ -408,6 +466,11 @@ int cpumR3MsrRangesInsert(PVM pVM, PCPUMMSRRANGE *ppaMsrRanges, uint32_t *pcMsrR
         ppaMsrRanges = &pVM->cpum.s.GuestInfo.paMsrRangesR3;
         pcMsrRanges  = &pVM->cpum.s.GuestInfo.cMsrRanges;
     }
+    else
+    {
+        AssertReturn(ppaMsrRanges, VERR_INVALID_POINTER);
+        AssertReturn(pcMsrRanges, VERR_INVALID_POINTER);
+    }
 
     uint32_t        cMsrRanges  = *pcMsrRanges;
     PCPUMMSRRANGE   paMsrRanges = *ppaMsrRanges;
@@ -530,6 +593,54 @@ int cpumR3MsrRangesInsert(PVM pVM, PCPUMMSRRANGE *ppaMsrRanges, uint32_t *pcMsrR
 }
 
 
+/**
+ * Reconciles CPUID info with MSRs (selected ones).
+ *
+ * @returns VBox status code.
+ * @param   pVM                 The cross context VM structure.
+ */
+int cpumR3MsrReconcileWithCpuId(PVM pVM)
+{
+    PCCPUMMSRRANGE papToAdd[10];
+    uint32_t      cToAdd = 0;
+
+    /*
+     * The IA32_FLUSH_CMD MSR was introduced in MCUs for CVS-2018-3646 and associates.
+     */
+    if (pVM->cpum.s.GuestFeatures.fFlushCmd && !cpumLookupMsrRange(pVM, MSR_IA32_FLUSH_CMD))
+    {
+        static CPUMMSRRANGE const s_FlushCmd =
+        {
+            /*.uFirst =*/       MSR_IA32_FLUSH_CMD,
+            /*.uLast =*/        MSR_IA32_FLUSH_CMD,
+            /*.enmRdFn =*/      kCpumMsrRdFn_WriteOnly,
+            /*.enmWrFn =*/      kCpumMsrWrFn_Ia32FlushCmd,
+            /*.offCpumCpu =*/   UINT16_MAX,
+            /*.fReserved =*/    0,
+            /*.uValue =*/       0,
+            /*.fWrIgnMask =*/   0,
+            /*.fWrGpMask =*/    ~MSR_IA32_FLUSH_CMD_F_L1D,
+            /*.szName = */      "IA32_FLUSH_CMD"
+        };
+        papToAdd[cToAdd++] = &s_FlushCmd;
+    }
+
+    /*
+     * Do the adding.
+     */
+    for (uint32_t i = 0; i < cToAdd; i++)
+    {
+        PCCPUMMSRRANGE pRange = papToAdd[i];
+        LogRel(("CPUM: MSR/CPUID reconciliation insert: %#010x %s\n", pRange->uFirst, pRange->szName));
+        int rc = cpumR3MsrRangesInsert(NULL /* pVM */, &pVM->cpum.s.GuestInfo.paMsrRangesR3, &pVM->cpum.s.GuestInfo.cMsrRanges,
+                                       pRange);
+        if (RT_FAILURE(rc))
+            return rc;
+    }
+    return VINF_SUCCESS;
+}
+
+
 /**
  * Worker for cpumR3MsrApplyFudge that applies one table.
  *
@@ -756,6 +867,7 @@ int cpumR3DbGetCpuInfo(const char *pszName, PCPUMINFO pInfo)
         rc = CPUMR3CpuIdCollectLeaves(&pInfo->paCpuIdLeavesR3, &pInfo->cCpuIdLeaves);
         if (RT_FAILURE(rc))
             return rc;
+        pInfo->fMxCsrMask = CPUMR3DeterminHostMxCsrMask();
 
         /* Lookup database entry for MSRs. */
         CPUMCPUVENDOR const enmVendor    = CPUMR3CpuIdDetectVendorEx(pInfo->paCpuIdLeavesR3[0].uEax,
@@ -861,8 +973,11 @@ int cpumR3DbGetCpuInfo(const char *pszName, PCPUMINFO pInfo)
         pInfo->cCpuIdLeaves = pEntry->cCpuIdLeaves;
         if (pEntry->cCpuIdLeaves)
         {
-            pInfo->paCpuIdLeavesR3 = (PCPUMCPUIDLEAF)RTMemDup(pEntry->paCpuIdLeaves,
-                                                              sizeof(pEntry->paCpuIdLeaves[0]) * pEntry->cCpuIdLeaves);
+            /* Must allocate a multiple of 16 here, matching cpumR3CpuIdEnsureSpace. */
+            size_t cbExtra = sizeof(pEntry->paCpuIdLeaves[0]) * (RT_ALIGN(pEntry->cCpuIdLeaves, 16) - pEntry->cCpuIdLeaves);
+            pInfo->paCpuIdLeavesR3 = (PCPUMCPUIDLEAF)RTMemDupEx(pEntry->paCpuIdLeaves,
+                                                                sizeof(pEntry->paCpuIdLeaves[0]) * pEntry->cCpuIdLeaves,
+                                                                cbExtra);
             if (!pInfo->paCpuIdLeavesR3)
                 return VERR_NO_MEMORY;
         }
@@ -870,7 +985,8 @@ int cpumR3DbGetCpuInfo(const char *pszName, PCPUMINFO pInfo)
             pInfo->paCpuIdLeavesR3 = NULL;
 
         pInfo->enmUnknownCpuIdMethod = pEntry->enmUnknownCpuId;
-        pInfo->DefCpuId         = pEntry->DefUnknownCpuId;
+        pInfo->DefCpuId              = pEntry->DefUnknownCpuId;
+        pInfo->fMxCsrMask            = pEntry->fMxCsrMask;
 
         LogRel(("CPUM: Using CPU DB entry '%s' (%s %#x/%#x/%#x %s)\n",
                 pEntry->pszName, CPUMR3CpuVendorName((CPUMCPUVENDOR)pEntry->enmVendor),
@@ -879,7 +995,6 @@ int cpumR3DbGetCpuInfo(const char *pszName, PCPUMINFO pInfo)
 
     pInfo->fMsrMask             = pEntry->fMsrMask;
     pInfo->iFirstExtCpuIdLeaf   = 0; /* Set by caller. */
-    pInfo->uPadding             = 0;
     pInfo->uScalableBusFreq     = pEntry->uScalableBusFreq;
     pInfo->paCpuIdLeavesR0      = NIL_RTR0PTR;
     pInfo->paMsrRangesR0        = NIL_RTR0PTR;
@@ -954,7 +1069,7 @@ int cpumR3MsrRegStats(PVM pVM)
     STAM_REL_REG(pVM, &pCpum->cMsrReadsUnknown,         STAMTYPE_COUNTER,   "/CPUM/MSR-Totals/ReadsUnknown",
                  STAMUNIT_OCCURENCES, "RDMSR on unknown MSRs (raises #GP).");
     STAM_REL_REG(pVM, &pCpum->cMsrWrites,               STAMTYPE_COUNTER,   "/CPUM/MSR-Totals/Writes",
-                 STAMUNIT_OCCURENCES, "All RDMSRs making it to CPUM.");
+                 STAMUNIT_OCCURENCES, "All WRMSRs making it to CPUM.");
     STAM_REL_REG(pVM, &pCpum->cMsrWritesRaiseGp,        STAMTYPE_COUNTER,   "/CPUM/MSR-Totals/WritesRaisingGP",
                  STAMUNIT_OCCURENCES, "WRMSR raising #GPs, except unknown MSRs.");
     STAM_REL_REG(pVM, &pCpum->cMsrWritesToIgnoredBits,  STAMTYPE_COUNTER,   "/CPUM/MSR-Totals/WritesToIgnoredBits",
