@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___PGMInline_h
-#define ___PGMInline_h
+#ifndef VMM_INCLUDED_SRC_include_PGMInline_h
+#define VMM_INCLUDED_SRC_include_PGMInline_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <VBox/cdefs.h>
 #include <VBox/types.h>
@@ -32,6 +35,9 @@
 #include <VBox/log.h>
 #include <VBox/vmm/gmm.h>
 #include <VBox/vmm/hm.h>
+#ifndef IN_RC
+# include <VBox/vmm/nem.h>
+#endif
 #include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/avl.h>
@@ -513,6 +519,39 @@ DECLINLINE(int) pgmPhysPageQueryTlbeWithPage(PVM pVM, PPGMPAGE pPage, RTGCPHYS G
     return rc;
 }
 
+
+/**
+ * Calculates NEM page protection flags.
+ */
+DECL_FORCE_INLINE(uint32_t) pgmPhysPageCalcNemProtection(PPGMPAGE pPage, PGMPAGETYPE enmType)
+{
+    /*
+     * Deal with potentially writable pages first.
+     */
+    if (PGMPAGETYPE_IS_RWX(enmType))
+    {
+        if (!PGM_PAGE_HAS_ACTIVE_HANDLERS(pPage))
+        {
+            if (PGM_PAGE_IS_ALLOCATED(pPage))
+                return NEM_PAGE_PROT_READ | NEM_PAGE_PROT_EXECUTE | NEM_PAGE_PROT_WRITE;
+            return NEM_PAGE_PROT_READ | NEM_PAGE_PROT_EXECUTE;
+        }
+        if (!PGM_PAGE_HAS_ACTIVE_ALL_HANDLERS(pPage))
+            return NEM_PAGE_PROT_READ | NEM_PAGE_PROT_EXECUTE;
+    }
+    /*
+     * Potentially readable & executable pages.
+     */
+    else if (   PGMPAGETYPE_IS_ROX(enmType)
+             && !PGM_PAGE_HAS_ACTIVE_ALL_HANDLERS(pPage))
+        return NEM_PAGE_PROT_READ | NEM_PAGE_PROT_EXECUTE;
+
+    /*
+     * The rest is needs special access handling.
+     */
+    return NEM_PAGE_PROT_NONE;
+}
+
 #endif /* !IN_RC */
 
 /**
@@ -545,6 +584,18 @@ DECLINLINE(void) pgmPhysPageWriteMonitor(PVM pVM, PPGMPAGE pPage, RTGCPHYS GCPhy
         else
             Assert(PGM_PAGE_GET_PDE_TYPE(pFirstPage) == PGM_PAGE_PDE_TYPE_PDE_DISABLED);
     }
+
+#ifndef IN_RC
+    /* Tell NEM. */
+    if (VM_IS_NEM_ENABLED(pVM))
+    {
+        uint8_t     u2State = PGM_PAGE_GET_NEM_STATE(pPage);
+        PGMPAGETYPE enmType = (PGMPAGETYPE)PGM_PAGE_GET_TYPE(pPage);
+        NEMHCNotifyPhysPageProtChanged(pVM, GCPhysPage, PGM_PAGE_GET_HCPHYS(pPage),
+                                       pgmPhysPageCalcNemProtection(pPage, enmType), enmType, &u2State);
+        PGM_PAGE_SET_NEM_STATE(pPage, u2State);
+    }
+#endif
 }
 
 
@@ -720,7 +771,7 @@ DECLINLINE(PX86PDPE) pgmGstGetPaePDPEPtr(PVMCPU pVCpu, RTGCPTR GCPtr)
             return NULL;
     }
 #endif
-    return &pGuestPDPT->a[(GCPtr >> X86_PDPT_SHIFT) & X86_PDPT_MASK_PAE];
+    return &pGuestPDPT->a[(uint32_t)GCPtr >> X86_PDPT_SHIFT];
 }
 
 
@@ -738,7 +789,7 @@ DECLINLINE(X86PDEPAE) pgmGstGetPaePDE(PVMCPU pVCpu, RTGCPTR GCPtr)
     PX86PDPT    pGuestPDPT = pgmGstGetPaePDPTPtr(pVCpu);
     if (RT_LIKELY(pGuestPDPT))
     {
-        const unsigned iPdpt = (GCPtr >> X86_PDPT_SHIFT) & X86_PDPT_MASK_PAE;
+        const unsigned iPdpt = (uint32_t)GCPtr >> X86_PDPT_SHIFT;
         if (    pGuestPDPT->a[iPdpt].n.u1Present
             &&  !(pGuestPDPT->a[iPdpt].u & pVCpu->pgm.s.fGstPaeMbzPdpeMask) )
         {
@@ -787,7 +838,7 @@ DECLINLINE(PX86PDPAE) pgmGstGetPaePDPtr(PVMCPU pVCpu, RTGCPTR GCPtr, unsigned *p
     PX86PDPT        pGuestPDPT = pgmGstGetPaePDPTPtr(pVCpu);
     if (RT_UNLIKELY(!pGuestPDPT))
         return NULL;
-    const unsigned  iPdpt = (GCPtr >> X86_PDPT_SHIFT) & X86_PDPT_MASK_PAE;
+    const unsigned  iPdpt = (uint32_t)GCPtr >> X86_PDPT_SHIFT;
     if (pPdpe)
         *pPdpe = pGuestPDPT->a[iPdpt];
     if (!pGuestPDPT->a[iPdpt].n.u1Present)
@@ -1007,15 +1058,13 @@ DECLINLINE(PX86PD) pgmShwGet32BitPDPtr(PVMCPU pVCpu)
  */
 DECLINLINE(X86PDE) pgmShwGet32BitPDE(PVMCPU pVCpu, RTGCPTR GCPtr)
 {
-    const unsigned iPd = (GCPtr >> X86_PD_SHIFT) & X86_PD_MASK;
-
     PX86PD pShwPde = pgmShwGet32BitPDPtr(pVCpu);
     if (!pShwPde)
     {
         X86PDE ZeroPde = {0};
         return ZeroPde;
     }
-    return pShwPde->a[iPd];
+    return pShwPde->a[(uint32_t)GCPtr >> X86_PD_SHIFT];
 }
 
 
@@ -1029,11 +1078,9 @@ DECLINLINE(X86PDE) pgmShwGet32BitPDE(PVMCPU pVCpu, RTGCPTR GCPtr)
  */
 DECLINLINE(PX86PDE) pgmShwGet32BitPDEPtr(PVMCPU pVCpu, RTGCPTR GCPtr)
 {
-    const unsigned iPd = (GCPtr >> X86_PD_SHIFT) & X86_PD_MASK;
-
     PX86PD pPde = pgmShwGet32BitPDPtr(pVCpu);
     AssertReturn(pPde, NULL);
-    return &pPde->a[iPd];
+    return &pPde->a[(uint32_t)GCPtr >> X86_PD_SHIFT];
 }
 
 
@@ -1058,7 +1105,7 @@ DECLINLINE(PX86PDPT) pgmShwGetPaePDPTPtr(PVMCPU pVCpu)
  */
 DECLINLINE(PX86PDPAE) pgmShwGetPaePDPtr(PVMCPU pVCpu, RTGCPTR GCPtr)
 {
-    const unsigned  iPdpt = (GCPtr >> X86_PDPT_SHIFT) & X86_PDPT_MASK_PAE;
+    const unsigned  iPdpt = (uint32_t)GCPtr >> X86_PDPT_SHIFT;
     PX86PDPT        pPdpt = pgmShwGetPaePDPTPtr(pVCpu);
 
     if (!pPdpt->a[iPdpt].n.u1Present)
@@ -1083,7 +1130,7 @@ DECLINLINE(PX86PDPAE) pgmShwGetPaePDPtr(PVMCPU pVCpu, RTGCPTR GCPtr)
  */
 DECLINLINE(PX86PDPAE) pgmShwGetPaePDPtr(PVMCPU pVCpu, PX86PDPT pPdpt, RTGCPTR GCPtr)
 {
-    const unsigned  iPdpt = (GCPtr >> X86_PDPT_SHIFT) & X86_PDPT_MASK_PAE;
+    const unsigned  iPdpt = (uint32_t)GCPtr >> X86_PDPT_SHIFT;
 
     if (!pPdpt->a[iPdpt].n.u1Present)
         return NULL;
@@ -1456,12 +1503,12 @@ DECLINLINE(bool) pgmPoolIsPageLocked(PPGMPOOLPAGE pPage)
 DECL_FORCE_INLINE(bool) pgmMapAreMappingsEnabled(PVM pVM)
 {
 #ifdef PGM_WITHOUT_MAPPINGS
-    /* There are no mappings in VT-x and AMD-V mode. */
-    Assert(HMIsEnabled(pVM)); NOREF(pVM);
+    /* Only raw-mode has mappings. */
+    Assert(!VM_IS_RAW_MODE_ENABLED(pVM)); NOREF(pVM);
     return false;
 #else
-    Assert(pVM->cCpus == 1 || HMIsEnabled(pVM));
-    return !HMIsEnabled(pVM);
+    Assert(pVM->cCpus == 1 || !VM_IS_RAW_MODE_ENABLED(pVM));
+    return VM_IS_RAW_MODE_ENABLED(pVM);
 #endif
 }
 
@@ -1475,8 +1522,8 @@ DECL_FORCE_INLINE(bool) pgmMapAreMappingsEnabled(PVM pVM)
 DECL_FORCE_INLINE(bool) pgmMapAreMappingsFloating(PVM pVM)
 {
 #ifdef PGM_WITHOUT_MAPPINGS
-    /* There are no mappings in VT-x and AMD-V mode. */
-    Assert(HMIsEnabled(pVM)); NOREF(pVM);
+    /* Only raw-mode has mappings. */
+    Assert(!VM_IS_RAW_MODE_ENABLED(pVM)); NOREF(pVM);
     return false;
 #else
     return !pVM->pgm.s.fMappingsFixed
@@ -1486,5 +1533,5 @@ DECL_FORCE_INLINE(bool) pgmMapAreMappingsFloating(PVM pVM)
 
 /** @} */
 
-#endif
+#endif /* !VMM_INCLUDED_SRC_include_PGMInline_h */
 
