@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007-2016 Oracle Corporation
+ * Copyright (C) 2007-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,9 +24,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-
-#ifndef ___bs3_cpu_generated_1_h___
-#define ___bs3_cpu_generated_1_h___
+#ifndef VBOX_INCLUDED_SRC_bootsectors_bs3_cpu_generated_1_h
+#define VBOX_INCLUDED_SRC_bootsectors_bs3_cpu_generated_1_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <bs3kit.h>
 #include <iprt/assert.h>
@@ -42,25 +44,68 @@ typedef enum BS3CG1OP
     BS3CG1OP_INVALID = 0,
 
     BS3CG1OP_Eb,
+    BS3CG1OP_Ed,
+    BS3CG1OP_Ed_WO,
+    BS3CG1OP_Eq,
+    BS3CG1OP_Eq_WO,
     BS3CG1OP_Ev,
+    BS3CG1OP_Qq,
+    BS3CG1OP_Qq_WO,
     BS3CG1OP_Wss,
+    BS3CG1OP_Wss_WO,
     BS3CG1OP_Wsd,
+    BS3CG1OP_Wsd_WO,
     BS3CG1OP_Wps,
+    BS3CG1OP_Wps_WO,
     BS3CG1OP_Wpd,
+    BS3CG1OP_Wpd_WO,
     BS3CG1OP_Wdq,
-    BS3CG1OP_WqZxReg,
+    BS3CG1OP_Wdq_WO,
+    BS3CG1OP_Wq,
+    BS3CG1OP_Wq_WO,
+    BS3CG1OP_WqZxReg_WO,
+    BS3CG1OP_Wx,
+    BS3CG1OP_Wx_WO,
 
     BS3CG1OP_Gb,
     BS3CG1OP_Gv,
+    BS3CG1OP_Gv_RO,
+    BS3CG1OP_HssHi,
+    BS3CG1OP_HsdHi,
+    BS3CG1OP_HqHi,
+    BS3CG1OP_Nq,
+    BS3CG1OP_Pd,
+    BS3CG1OP_PdZx_WO,
+    BS3CG1OP_Pq,
+    BS3CG1OP_Pq_WO,
     BS3CG1OP_Uq,
     BS3CG1OP_UqHi,
+    BS3CG1OP_Uss,
+    BS3CG1OP_Uss_WO,
+    BS3CG1OP_Usd,
+    BS3CG1OP_Usd_WO,
+    BS3CG1OP_Vd,
+    BS3CG1OP_Vd_WO,
+    BS3CG1OP_VdZx_WO,
     BS3CG1OP_Vss,
-    BS3CG1OP_VssZxReg,
+    BS3CG1OP_Vss_WO,
+    BS3CG1OP_VssZx_WO,
     BS3CG1OP_Vsd,
+    BS3CG1OP_Vsd_WO,
+    BS3CG1OP_VsdZx_WO,
     BS3CG1OP_Vps,
+    BS3CG1OP_Vps_WO,
     BS3CG1OP_Vpd,
+    BS3CG1OP_Vpd_WO,
     BS3CG1OP_Vq,
+    BS3CG1OP_Vq_WO,
     BS3CG1OP_Vdq,
+    BS3CG1OP_Vdq_WO,
+    BS3CG1OP_VqHi,
+    BS3CG1OP_VqHi_WO,
+    BS3CG1OP_VqZx_WO,
+    BS3CG1OP_Vx,
+    BS3CG1OP_Vx_WO,
 
     BS3CG1OP_Ib,
     BS3CG1OP_Iz,
@@ -69,10 +114,18 @@ typedef enum BS3CG1OP
     BS3CG1OP_rAX,
 
     BS3CG1OP_Ma,
-    BS3CG1OP_MbRO,
-    BS3CG1OP_MdRO,
-    BS3CG1OP_MdWO,
+    BS3CG1OP_Mb_RO,
+    BS3CG1OP_Md,
+    BS3CG1OP_Md_RO,
+    BS3CG1OP_Md_WO,
+    BS3CG1OP_Mdq,
+    BS3CG1OP_Mdq_WO,
     BS3CG1OP_Mq,
+    BS3CG1OP_Mq_WO,
+    BS3CG1OP_Mps_WO,
+    BS3CG1OP_Mpd_WO,
+    BS3CG1OP_Mx,
+    BS3CG1OP_Mx_WO,
 
     BS3CG1OP_END
 } BS3CG1OP;
@@ -92,31 +145,88 @@ typedef enum BS3CG1ENC
 
     BS3CG1ENC_MODRM_Eb_Gb,
     BS3CG1ENC_MODRM_Ev_Gv,
-    BS3CG1ENC_MODRM_Wss_Vss,
-    BS3CG1ENC_MODRM_Wsd_Vsd,
-    BS3CG1ENC_MODRM_Wps_Vps,
-    BS3CG1ENC_MODRM_Wpd_Vpd,
-    BS3CG1ENC_MODRM_WqZxReg_Vq,
+    BS3CG1ENC_MODRM_Ed_WO_Pd_WZ,
+    BS3CG1ENC_MODRM_Eq_WO_Pq_WNZ,
+    BS3CG1ENC_MODRM_Ed_WO_Vd_WZ,
+    BS3CG1ENC_MODRM_Eq_WO_Vq_WNZ,
+    BS3CG1ENC_MODRM_Pq_WO_Qq,
+    BS3CG1ENC_MODRM_Wss_WO_Vss,
+    BS3CG1ENC_MODRM_Wsd_WO_Vsd,
+    BS3CG1ENC_MODRM_Wps_WO_Vps,
+    BS3CG1ENC_MODRM_Wpd_WO_Vpd,
+    BS3CG1ENC_MODRM_WqZxReg_WO_Vq,
 
     BS3CG1ENC_MODRM_Gb_Eb,
     BS3CG1ENC_MODRM_Gv_Ev,
-    BS3CG1ENC_MODRM_Gv_Ma, /**< bound instruction */
-    BS3CG1ENC_MODRM_Vq_UqHi,
-    BS3CG1ENC_MODRM_Vq_Mq,
-    BS3CG1ENC_MODRM_Vdq_Wdq,
-    BS3CG1ENC_MODRM_VssZxReg_Wss,
-    BS3CG1ENC_MODRM_MbRO,
-    BS3CG1ENC_MODRM_MdRO,
-    BS3CG1ENC_MODRM_MdWO,
+    BS3CG1ENC_MODRM_Gv_RO_Ma, /**< bound instruction */
+    BS3CG1ENC_MODRM_Pq_WO_Uq,
+    BS3CG1ENC_MODRM_PdZx_WO_Ed_WZ,
+    BS3CG1ENC_MODRM_Pq_WO_Eq_WNZ,
+    BS3CG1ENC_MODRM_VdZx_WO_Ed_WZ,
+    BS3CG1ENC_MODRM_Vq_WO_UqHi,
+    BS3CG1ENC_MODRM_Vq_WO_Mq,
+    BS3CG1ENC_MODRM_VqHi_WO_Uq,
+    BS3CG1ENC_MODRM_VqHi_WO_Mq,
+    BS3CG1ENC_MODRM_VqZx_WO_Eq_WNZ,
+    BS3CG1ENC_MODRM_Vdq_WO_Mdq,
+    BS3CG1ENC_MODRM_Vdq_WO_Wdq,
+    BS3CG1ENC_MODRM_Vpd_WO_Wpd,
+    BS3CG1ENC_MODRM_Vps_WO_Wps,
+    BS3CG1ENC_MODRM_VssZx_WO_Wss,
+    BS3CG1ENC_MODRM_VsdZx_WO_Wsd,
+    BS3CG1ENC_MODRM_VqZx_WO_Wq,
+    BS3CG1ENC_MODRM_VqZx_WO_Nq,
+    BS3CG1ENC_MODRM_Mb_RO,
+    BS3CG1ENC_MODRM_Md_RO,
+    BS3CG1ENC_MODRM_Md_WO,
+    BS3CG1ENC_MODRM_Mdq_WO_Vdq,
+    BS3CG1ENC_MODRM_Mq_WO_Pq,
+    BS3CG1ENC_MODRM_Mq_WO_Vq,
+    BS3CG1ENC_MODRM_Mq_WO_VqHi,
+    BS3CG1ENC_MODRM_Mps_WO_Vps,
+    BS3CG1ENC_MODRM_Mpd_WO_Vpd,
 
-    BS3CG1ENC_VEX_MODRM_MdWO,
+    BS3CG1ENC_VEX_MODRM_Vd_WO_Ed_WZ,
+    BS3CG1ENC_VEX_MODRM_Vps_WO_Wps,
+    BS3CG1ENC_VEX_MODRM_Vpd_WO_Wpd,
+    BS3CG1ENC_VEX_MODRM_Vss_WO_HssHi_Uss,
+    BS3CG1ENC_VEX_MODRM_Vsd_WO_HsdHi_Usd,
+    BS3CG1ENC_VEX_MODRM_Vq_WO_Eq_WNZ,
+    BS3CG1ENC_VEX_MODRM_Vq_WO_HqHi_UqHi,
+    BS3CG1ENC_VEX_MODRM_Vq_WO_HqHi_Mq,
+    BS3CG1ENC_VEX_MODRM_Vq_WO_Wq,
+    BS3CG1ENC_VEX_MODRM_VssZx_WO_Md,
+    BS3CG1ENC_VEX_MODRM_VsdZx_WO_Mq,
+    BS3CG1ENC_VEX_MODRM_Vx_WO_Mx_L0,
+    BS3CG1ENC_VEX_MODRM_Vx_WO_Mx_L1,
+    BS3CG1ENC_VEX_MODRM_Vx_WO_Wx,
+    BS3CG1ENC_VEX_MODRM_Ed_WO_Vd_WZ,
+    BS3CG1ENC_VEX_MODRM_Eq_WO_Vq_WNZ,
+    BS3CG1ENC_VEX_MODRM_Md_WO,
+    BS3CG1ENC_VEX_MODRM_Mq_WO_Vq,
+    BS3CG1ENC_VEX_MODRM_Md_WO_Vss,
+    BS3CG1ENC_VEX_MODRM_Mq_WO_Vsd,
+    BS3CG1ENC_VEX_MODRM_Mps_WO_Vps,
+    BS3CG1ENC_VEX_MODRM_Mpd_WO_Vpd,
+    BS3CG1ENC_VEX_MODRM_Mx_WO_Vx,
+    BS3CG1ENC_VEX_MODRM_Uss_WO_HssHi_Vss,
+    BS3CG1ENC_VEX_MODRM_Usd_WO_HsdHi_Vsd,
+    BS3CG1ENC_VEX_MODRM_Wps_WO_Vps,
+    BS3CG1ENC_VEX_MODRM_Wpd_WO_Vpd,
+    BS3CG1ENC_VEX_MODRM_Wq_WO_Vq,
+    BS3CG1ENC_VEX_MODRM_Wx_WO_Vx,
 
     BS3CG1ENC_FIXED,
     BS3CG1ENC_FIXED_AL_Ib,
     BS3CG1ENC_FIXED_rAX_Iz,
 
+
     BS3CG1ENC_MODRM_MOD_EQ_3, /**< Unused or invalid instruction. */
     BS3CG1ENC_MODRM_MOD_NE_3, /**< Unused or invalid instruction. */
+    //BS3CG1ENC_VEX_FIXED,          /**< Unused or invalid instruction. */
+    BS3CG1ENC_VEX_MODRM_MOD_EQ_3, /**< Unused or invalid instruction. */
+    BS3CG1ENC_VEX_MODRM_MOD_NE_3, /**< Unused or invalid instruction. */
+    BS3CG1ENC_VEX_MODRM,          /**< Unused or invalid instruction. */
 
     BS3CG1ENC_END
 } BS3CG1ENC;
@@ -154,9 +264,11 @@ typedef enum BS3CG1CPU
     BS3CG1CPU_GE_80486,
     BS3CG1CPU_GE_Pentium,
 
+    BS3CG1CPU_MMX,
     BS3CG1CPU_SSE,
     BS3CG1CPU_SSE2,
     BS3CG1CPU_SSE3,
+    BS3CG1CPU_SSE4_1,
     BS3CG1CPU_AVX,
     BS3CG1CPU_AVX2,
     BS3CG1CPU_CLFSH,
@@ -179,8 +291,10 @@ typedef enum BS3CG1XCPTTYPE
     BS3CG1XCPTTYPE_4,
     BS3CG1XCPTTYPE_4UA,
     BS3CG1XCPTTYPE_5,
+    BS3CG1XCPTTYPE_5LZ,
     BS3CG1XCPTTYPE_6,
     BS3CG1XCPTTYPE_7,
+    BS3CG1XCPTTYPE_7LZ,
     BS3CG1XCPTTYPE_8,
     BS3CG1XCPTTYPE_11,
     BS3CG1XCPTTYPE_12,
@@ -225,6 +339,8 @@ typedef struct BS3CG1INSTR
     uint32_t    offTests : 23;
     /** BS3CG1ENC values. */
     uint32_t    enmEncoding : 10;
+    /** The VEX, EVEX or XOP opcode map number (VEX.mmmm). */
+    uint32_t    uOpcodeMap : 4;
     /** BS3CG1PFXKIND values. */
     uint32_t    enmPrefixKind : 4;
     /** CPU test / CPU ID bit test (BS3CG1CPU). */
@@ -232,7 +348,7 @@ typedef struct BS3CG1INSTR
     /** Exception type (BS3CG1XCPTTYPE)   */
     uint32_t    enmXcptType : 5;
     /** Currently unused bits. */
-    uint32_t    uUnused : 6;
+    uint32_t    uUnused : 3;
     /** BS3CG1INSTR_F_XXX. */
     uint32_t    fFlags;
 } BS3CG1INSTR;
@@ -254,6 +370,10 @@ typedef BS3CG1INSTR const BS3_FAR *PCBS3CG1INSTR;
 /** Only intel does full ModR/M(, ++) decoding for invalid instruction.
  * Always used with BS3CG1INSTR_F_INVALID or BS3CG1INSTR_F_UNUSED. */
 #define BS3CG1INSTR_F_INTEL_DECODES_INVALID UINT32_C(0x00000010)
+/** VEX.L must be zero (IEMOPHINT_VEX_L_ZERO). */
+#define BS3CG1INSTR_F_VEX_L_ZERO            UINT32_C(0x00000020)
+/** VEX.L is ignored (IEMOPHINT_VEX_L_IGNORED). */
+#define BS3CG1INSTR_F_VEX_L_IGNORED         UINT32_C(0x00000040)
 /** @} */
 
 
@@ -452,6 +572,14 @@ typedef enum BS3CG1DST
     BS3CG1DST_MM5,
     BS3CG1DST_MM6,
     BS3CG1DST_MM7,
+    BS3CG1DST_MM0_LO_ZX,
+    BS3CG1DST_MM1_LO_ZX,
+    BS3CG1DST_MM2_LO_ZX,
+    BS3CG1DST_MM3_LO_ZX,
+    BS3CG1DST_MM4_LO_ZX,
+    BS3CG1DST_MM5_LO_ZX,
+    BS3CG1DST_MM6_LO_ZX,
+    BS3CG1DST_MM7_LO_ZX,
     /* SSE registers. */
     BS3CG1DST_XMM0,
     BS3CG1DST_XMM1,
@@ -549,6 +677,22 @@ typedef enum BS3CG1DST
     BS3CG1DST_XMM13_DW0_ZX,
     BS3CG1DST_XMM14_DW0_ZX,
     BS3CG1DST_XMM15_DW0_ZX,
+    BS3CG1DST_XMM0_HI96,
+    BS3CG1DST_XMM1_HI96,
+    BS3CG1DST_XMM2_HI96,
+    BS3CG1DST_XMM3_HI96,
+    BS3CG1DST_XMM4_HI96,
+    BS3CG1DST_XMM5_HI96,
+    BS3CG1DST_XMM6_HI96,
+    BS3CG1DST_XMM7_HI96,
+    BS3CG1DST_XMM8_HI96,
+    BS3CG1DST_XMM9_HI96,
+    BS3CG1DST_XMM10_HI96,
+    BS3CG1DST_XMM11_HI96,
+    BS3CG1DST_XMM12_HI96,
+    BS3CG1DST_XMM13_HI96,
+    BS3CG1DST_XMM14_HI96,
+    BS3CG1DST_XMM15_HI96,
     /* AVX registers. */
     BS3CG1DST_YMM0,
     BS3CG1DST_YMM1,
@@ -601,6 +745,9 @@ typedef enum BS3CG1PRED
     BS3CG1PRED_SIZE_O16,
     BS3CG1PRED_SIZE_O32,
     BS3CG1PRED_SIZE_O64,
+    /* VEX.L values. */
+    BS3CG1PRED_VEXL_0,
+    BS3CG1PRED_VEXL_1,
     /* Execution ring. */
     BS3CG1PRED_RING_0,
     BS3CG1PRED_RING_1,
@@ -653,5 +800,5 @@ extern const uint8_t BS3_FAR_DATA       g_abBs3Cg1Operands[];
 extern const uint8_t BS3_FAR_DATA       g_abBs3Cg1Tests[];
 
 
-#endif
+#endif /* !VBOX_INCLUDED_SRC_bootsectors_bs3_cpu_generated_1_h */
 
