@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2015 Oracle Corporation
+ * Copyright (C) 2010-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -233,11 +233,14 @@ static const RTGETOPTDEF g_aCmdOptions[] =
     { "--daemonize",        'd', RTGETOPT_REQ_NOTHING },
     { "--daemonized",       'D', RTGETOPT_REQ_NOTHING },
     { "--check-data",       'C', RTGETOPT_REQ_NOTHING },
+    { "--verbose",          'v', RTGETOPT_REQ_NOTHING },
     { "--help",             'h', RTGETOPT_REQ_NOTHING } /* for Usage() */
 };
 
 /** The test handle. */
 static RTTEST g_hTest;
+/** Verbosity level. */
+static uint32_t g_uVerbosity = 0;
 
 
 
@@ -300,6 +303,9 @@ static void Usage(PRTSTREAM pStrm)
             case 'C':
                 pszHelp = "Check payload data at the receiving end";
                 break;
+            case 'v':
+                pszHelp = "Verbose execution.";
+                break;
             default:
                 pszHelp = "Option undocumented";
                 break;
@@ -322,7 +328,8 @@ static void Usage(PRTSTREAM pStrm)
 static DECLCALLBACK(void) netperfStopTimerCallback(RTTIMERLR hTimer, void *pvUser, uint64_t iTick)
 {
     bool volatile *pfStop = (bool volatile *)pvUser;
-/*    RTPrintf("Time's Up!\n");*/
+    if (g_uVerbosity > 0)
+        RTPrintf("Time's Up!\n");
     ASMAtomicWriteBool(pfStop, true);
     NOREF(hTimer); NOREF(iTick);
 }
@@ -517,6 +524,8 @@ static int netperfTCPThroughputSend(NETPERFPARAMS const *pParams, NETPERFHDR *pB
         /*
          * Warm up.
          */
+        if (g_uVerbosity > 0)
+            RTPrintf("Warmup...\n");
         pBuf->u32State = RT_H2LE_U32_C(NETPERFHDR_WARMUP);
         rc = RTTimerLRStart(hTimer, pParams->cMsWarmup * UINT64_C(1000000) /* nsec */);
         if (RT_SUCCESS(rc))
@@ -541,6 +550,8 @@ static int netperfTCPThroughputSend(NETPERFPARAMS const *pParams, NETPERFHDR *pB
          */
         if (RT_SUCCESS(rc))
         {
+            if (g_uVerbosity > 0)
+                RTPrintf("The real thing...\n");
             pBuf->u32State = RT_H2LE_U32_C(NETPERFHDR_TESTING);
             fStop          = false;
             rc = RTTimerLRStart(hTimer, pParams->cSecTimeout * UINT64_C(1000000000) /* nsec */);
@@ -570,6 +581,8 @@ static int netperfTCPThroughputSend(NETPERFPARAMS const *pParams, NETPERFHDR *pB
          */
         if (RT_SUCCESS(rc))
         {
+            if (g_uVerbosity > 0)
+                RTPrintf("Cool down...\n");
             pBuf->u32State = RT_H2LE_U32_C(NETPERFHDR_COOL_DOWN);
             fStop          = false;
             rc = RTTimerLRStart(hTimer, pParams->cMsCoolDown * UINT64_C(1000000) /* nsec */);
@@ -594,6 +607,8 @@ static int netperfTCPThroughputSend(NETPERFPARAMS const *pParams, NETPERFHDR *pB
         /*
          * Send DONE packet.
          */
+        if (g_uVerbosity > 0)
+            RTPrintf("Done\n");
         if (RT_SUCCESS(rc))
         {
             u32Seq++;
@@ -718,6 +733,7 @@ static int netperfTCPThroughputRecv(NETPERFPARAMS const *pParams, NETPERFHDR *pB
  * Prints the statistics for the latency test.
  *
  * @param   pStats              The statistics.
+ * @param   cbPacket            The packet size in bytes.
  */
 static void netperfPrintLatencyStats(NETPERFSTATS const *pStats, uint32_t cbPacket)
 {
@@ -1441,6 +1457,8 @@ static int netperfTCPClientDoLatency(NETPERFPARAMS *pParams)
         /*
          * Warm up.
          */
+        if (g_uVerbosity > 0)
+            RTPrintf("Warmup...\n");
         rc = RTTimerLRStart(hTimer, pParams->cMsWarmup * UINT64_C(1000000) /* nsec */);
         if (RT_SUCCESS(rc))
         {
@@ -1472,6 +1490,8 @@ static int netperfTCPClientDoLatency(NETPERFPARAMS *pParams)
          */
         if (RT_SUCCESS(rc))
         {
+            if (g_uVerbosity > 0)
+                RTPrintf("The real thing...\n");
             fStop = false;
             rc = RTTimerLRStart(hTimer, pParams->cSecTimeout * UINT64_C(1000000000) /* nsec */);
             if (RT_SUCCESS(rc))
@@ -1515,6 +1535,8 @@ static int netperfTCPClientDoLatency(NETPERFPARAMS *pParams)
          */
         if (RT_SUCCESS(rc))
         {
+            if (g_uVerbosity > 0)
+                RTPrintf("Cool down...\n");
             fStop = false;
             rc = RTTimerLRStart(hTimer, pParams->cMsCoolDown * UINT64_C(1000000) /* nsec */);
             if (RT_SUCCESS(rc))
@@ -1546,6 +1568,8 @@ static int netperfTCPClientDoLatency(NETPERFPARAMS *pParams)
         /*
          * Send DONE packet.
          */
+        if (g_uVerbosity > 0)
+            RTPrintf("Done\n");
         if (RT_SUCCESS(rc))
         {
             u32Seq++;
@@ -1703,7 +1727,8 @@ static int netperfTCPClient(const char *pszServer, NETPERFPARAMS *pParams)
  *
  * @returns Exit code.
  * @param   enmProto            The protocol.
- * @param   pParams             The parameter block.
+ * @param   pszServer           The server name.
+ * @param   pvUser              The parameter block as opaque user data.
  */
 static RTEXITCODE netperfClient(NETPERFPROTO enmProto, const char *pszServer, void *pvUser)
 {
@@ -1755,8 +1780,8 @@ int main(int argc, char *argv[])
     bool                fDaemonized     = false;
     bool                fPacketSizeSet  = false;
     const char         *pszServerAddress= NULL;
-    NETPERFPARAMS       Params;
 
+    NETPERFPARAMS       Params;
     Params.uPort            = NETPERF_DEFAULT_PORT;
     Params.fServerStats     = false;
     Params.fSingleClient    = false;
@@ -1858,12 +1883,16 @@ int main(int argc, char *argv[])
                 Params.fSingleClient = true;
                 break;
 
+            case 'v':
+                g_uVerbosity++;
+                break;
+
             case 'h':
                 Usage(g_pStdOut);
                 return RTEXITCODE_SUCCESS;
 
             case 'V':
-                RTPrintf("$Revision: 103604 $\n");
+                RTPrintf("$Revision: 127855 $\n");
                 return RTEXITCODE_SUCCESS;
 
             case 'w':
